@@ -25,9 +25,7 @@
 #include <time.h>
 #include <regex.h>
 
-#include <ncursesw/ncurses.h>
-#include <ncursesw/menu.h>
-#include <ncursesw/form.h>
+#include "cdw_ncurses.h"
 
 #include "config.h"
 
