@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1068,6 +1068,7 @@ cdw_rv_t cdw_config_var_set_defaults(cdw_config_t *config)
 
 
 	/* page D - iso filesystem */
+	/* 2TRANS: this is a default label for ISO9660 volume */
 	strncpy(config->volume_id, _("my volume"), CDW_ISO9660_VOLI_LEN);
 	config->volume_id[CDW_ISO9660_VOLI_LEN] = '\0';
 	config->iso_level = 3;
@@ -1259,12 +1260,7 @@ cdw_rv_t cdw_config_var_read_from_file(FILE *config_file, cdw_config_t *config)
 		}
 
 		cdw_option_t option;
-		/* these will be only pointers to places in existing string */
-		option.name = (char *) NULL;
-		option.value = (char *) NULL;
-
-		option = cdw_config_split_options_line(line);
-		if (option.name == (char *) NULL || option.value == (char *) NULL) {
+		if (!cdw_config_split_options_line(&option, line)) {
 			continue; /* empty or invalid line, or comment */
 		}
 
@@ -1495,52 +1491,61 @@ cdw_rv_t cdw_config_var_read_from_file(FILE *config_file, cdw_config_t *config)
 
 
 /**
-   \brief Split option containing '=' char into 'name' and 'value' part
+   \brief Split line containing '=' char into 'name' and 'value' part
+
+   \date Function's top-level comment reviewed on 2012-02-07
+   \date Function's body reviewed on 2012-02-07
 
    The function takes one 'char *' string, recognizes where is first
    '=' char in it. What is on the left side of the char is treated
    as option name, and what is on the right side of the char is treated
    as option value. Pointers to beginning of both name and value are placed
-   in variable of type cdw_option_t. These pointers point to substrings of
-   original string.
+   in \p option. These pointers point to substrings of original string.
 
    Original string is modified so that strings representing name and value
    don't have any white chars at the beginning and end. '\0' chars are
    also inserted into original string (function's argument) to properly
    delimit the two substrings.
 
-   The function heavily modifies its argument!
+   The function heavily modifies its \p line argument!
 
    The function recognizes '#' as comment char and erases everything from
    line starting from '#' char;
 
-   line must be proper char * string ending with '\0'
+   \p line must be proper char * string ending with '\0'
 
-   Both fields of returned value are set to NULL if:
-   \li function's argument is NULL;
-   \li function's argument is empty;
-   \li function's argument does not contain '=' char;
-   \li function's argument is a comment line.
-   \li function's argument has '=' char but don't have any option name
+   Both fields of \p option are set to NULL if:
+   \li function's \p line argument is NULL;
+   \li function's \p line argument is empty;
+   \li function's \p line argument does not contain '=' char;
+   \li function's \p line argument is a comment line.
+   \li function's \p line argument has '=' char but don't have any option name
 
-   Otherwise the fields are set with valid pointers pointing to two
-   substrings in function's argument. Again: these substrings are proper
-   C strings and are ended with '\0';
+   Otherwise the fields of \p option are set with valid pointers pointing
+   to two substrings in function's \p line argument. Again: these substrings
+   are proper C strings and are ended with '\0'.
 
+   Don't use cdw_config_option_free_new() on \p option!
+
+   This function is similar to cdw_config_split_options_line_new(),
+   the difference being that name and value are substrings in \p line,
+   and not freshly allocated strings.
+
+   \param option - variable where pointers to substrings are placed
    \param line - line that you want to extract option from
 
-   \return value of type cdw_option_t
+   \return true on success (both fields of \p option are set)
+   \return false otherwise
 */
-cdw_option_t cdw_config_split_options_line(char *line)
+bool cdw_config_split_options_line(cdw_option_t *option, char *line)
 {
 	cdw_sdm ("input line = '%s'\n", line);
 
-	cdw_option_t option;
-	option.name = (char *) NULL;
-	option.value = (char *) NULL;
+	option->name = (char *) NULL;
+	option->value = (char *) NULL;
 
 	if (line == (char *) NULL) {
-		return option;
+		return false;
 	}
 
 	/* this will make sure that comment, starting at any position
@@ -1558,32 +1563,29 @@ cdw_option_t cdw_config_split_options_line(char *line)
 	char *eq = strstr(tline, "="); /* first occurrence of '=' in line */
 
 	if (eq == (char *) NULL) {
-		option.name = (char *) NULL;
-		option.value = (char *) NULL;
+		cdw_vdm ("ERROR: line is invalid: \"%s\"\n", line);
 
-		cdw_sdm ("  line is invalid\n");
-
-		return option;
+		return false;
 	} else {
-		option.name = tline;
+		option->name = tline;
 		*eq = '\0';
-		option.name = cdw_string_rtrim(option.name);
+		option->name = cdw_string_rtrim(option->name);
 
-		if (!strcmp(option.name, "")) {
-			option.name = (char *) NULL;
-			option.value = (char *) NULL;
-			return option;
+		if (!strcmp(option->name, "")) {
+			option->name = (char *) NULL;
+			option->value = (char *) NULL;
+			return false;
 		}
 
-		option.value = cdw_string_ltrim(eq + 1);
+		option->value = cdw_string_ltrim(eq + 1);
 		/* value may be empty string too, but this is not an error
 		   condition, since given option value can be empty
 		   (no value set) - this is correct situation */
 
-		cdw_sdm ("   option.name = '%s'\n", option.name);
-		cdw_sdm ("   option.value = '%s'\n", option.value);
+		cdw_sdm ("   option.name = '%s'\n", option->name);
+		cdw_sdm ("   option.value = '%s'\n", option->value);
 
-		return option;
+		return true;
 	}
 }
 
@@ -1591,6 +1593,137 @@ cdw_option_t cdw_config_split_options_line(char *line)
 
 
 
+/**
+   \brief Split line containing '=' char into 'name' and 'value' part
+
+   \date Function's top-level comment reviewed on 2012-02-07
+   \date Function's body reviewed on 2012-02-07
+
+   The function takes one 'char *' string, recognizes where is first
+   '=' char in it. What is on the left side of the char is treated
+   as option name, and what is on the right side of the char is treated
+   as option value. Both name and value of an option are duplicated
+   into appropriate fields of \p option. Both strings are trimmed of
+   whitespaces.
+
+   The function modifies its \p line argument!
+
+   The function recognizes '#' as comment char and erases everything from
+   line starting from '#' char;
+
+   \p line must be proper char * string ending with '\0'
+
+   Both fields of \p option are set to NULL if:
+   \li function's \p line argument is NULL;
+   \li function's \p line argument is empty;
+   \li function's \p line argument does not contain '=' char;
+   \li function's \p line argument is a comment line.
+   \li function's \p line argument has '=' char but don't have any option name
+
+   Otherwise the fields of \p option are set with freshly allocated
+   strings: name and value of option.
+
+   Use cdw_config_option_free_new() on \p option to conveniently
+   deallocate both strings.
+
+   This function is similar to cdw_config_split_options_line(), the
+   difference being that name and value are freshly allocated strings,
+   and not substrings in \p line.
+
+   \param option - variable where new strings with name and value are placed
+   \param line - line that you want to extract option from
+
+   \return true on success (both fields of \p option are set)
+   \return false otherwise
+*/
+bool cdw_config_split_options_line_new(cdw_option_t *option, char *line)
+{
+	if (!line) {
+		cdw_vdm ("ERROR: input line is NULL\n");
+		return false;
+	}
+
+	cdw_vdm ("INFO: input line = '%s'\n", line);
+
+	option->name = (char *) NULL;
+	option->value = (char *) NULL;
+
+
+	/* this will make sure that comment, starting at any position
+	   in line, will be erased from line */
+	size_t n = strcspn(line, "#"); /* number of chars before start of comment */
+	if (n != 0) {
+		line[n] = '\0';
+	}
+
+	char *eq = strstr(line, "=");
+	if (!eq) {
+		cdw_vdm ("WARNING: line is invalid (\"%s\")\n", line);
+		return false;
+	}
+	*eq = '\0';
+
+	option->name = cdw_string_trim(line);
+	option->value = cdw_string_trim(eq + 1);
+	cdw_vdm ("INFO: trimmed name: \"%s\"\n", option->name);
+	cdw_vdm ("INFO: trimmed value: \"%s\"\n", option->value);
+
+	if (!option->name || !option->value || !strcmp(option->name, "")) {
+		if (option->name) {
+			free(option->name);
+			option->name = (char *) NULL;
+		}
+		if (option->value) {
+			free(option->value);
+			option->value = (char *) NULL;
+		}
+		cdw_vdm ("ERROR: some string was empty\n");
+		return false;
+	}
+
+	return true;
+}
+
+
+
+
+
+/**
+   \brief Deallocate strings from an option
+
+   \date Function's top-level comment reviewed on 2012-02-07
+   \date Function's body reviewed on 2012-02-07
+
+   Function deallocates two strings from given \p option.
+   It checks for NULL pointers prior to deallocation, and sets
+   pointers to NULL after deallocation.
+
+   Call this function only for \p option processed by
+   cdw_config_split_options_line_new().
+
+   \param option - variable which you want to process
+*/
+void cdw_config_option_free_new(cdw_option_t *option)
+{
+	cdw_assert (option, "ERROR: pointer is NULL\n");
+
+	if (option->value) {
+		free(option->value);
+		option->value = (char *) NULL;
+	}
+
+	if (option->name) {
+		free(option->name);
+		option->name = (char *) NULL;
+	}
+
+	return;
+}
+
+
+
+
+
 /* simple getter - some files use global config variable just to get this field */
 const char *cdw_config_get_custom_drive(void)
 {
@@ -1740,6 +1873,7 @@ bool cdw_config_follow_symlinks(void)
 
 
 static void test_cdw_config_split_options_line(void);
+static void test_cdw_config_split_options_line_new(void);
 
 
 void cdw_config_run_tests(void)
@@ -1747,6 +1881,7 @@ void cdw_config_run_tests(void)
 	fprintf(stderr, "testing cdw_config.c\n");
 
 	test_cdw_config_split_options_line();
+	test_cdw_config_split_options_line_new();
 
 	fprintf(stderr, "done\n\n");
 
@@ -1755,110 +1890,127 @@ void cdw_config_run_tests(void)
 
 
 
+
+
 void test_cdw_config_split_options_line(void)
 {
 	fprintf(stderr, "\ttesting cdw_config_split_options_line()... ");
 
-	int d;
+	struct {
+		const char *line;
+		bool expected_return_value;
+		bool expected_null_name;
+		bool expected_null_value;
+		const char *expected_name;
+		const char *expected_value;
+
+	} input_data[] = {
+		/* correct line */
+		{ "option name=value string",
+		  true,
+		  false, false,
+		  "option name", "value string" },
+
+		/* correct line */
+		{ "\t option name \t\t = \t value string \t",
+		  true,
+		  false, false,
+		  "option name", "value string" },
+
+		/* correct line, but no value */
+		{ "\t option name = \t \t",
+		  true,
+		  false, false,
+		  "option name", "" },
+
+		/* correct line, but no value (2) */
+		{ "\t option name = #value string\t \t",
+		  true,
+		  false, false,
+		  "option name", "" },
+
+		/* incorrect line, no '=' char (1) */
+		{ "\t option name \t # = value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, no '=' char (2) */
+		{ "\t option name \t  value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, all content is comment */
+		{ " \t #\t option name = value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, empty line */
+		{ "",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, empty line (2) */
+		{ " \t \t    \t\t\t\t     \t               \t#    ",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, line = NULL */
+		{ (char *) NULL,
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		{ (char *) NULL,
+		  false,
+		  false, true, /* <-- guard */
+		  (char *) NULL, (char *) NULL }
+	};
 
 	/* remember that option fields are pointers to places inside
 	   input strings, not pointers to newly allocated strings */
 	cdw_option_t option;
-	option.name = NULL;
-	option.value = NULL;
 
-	/* correct line 1 */
-	char *o1 = strdup("option name=value string");
-	cdw_assert_test (o1 != NULL, "failed creating test line\n");
-	option = cdw_config_split_options_line(o1);
-	cdw_assert_test (option.name != NULL && option.value != NULL, "failed to split correct line\n");
-	d = strcmp(option.name, "option name");
-	cdw_assert_test (d == 0, "failed at comparing option name\n");
-	d = strcmp(option.value, "value string");
-	cdw_assert_test (d == 0, "failed at comparing option value\n");
-	free(o1);
-	o1 = (char *) NULL;
+	int i = 0;
+	while (input_data[i].expected_null_name == input_data[i].expected_null_value) {
+		char *line = (char *) NULL;
+		if (input_data[i].line) {
+			line = strdup(input_data[i].line);
+		}
 
-	/* correct line 2 - with some white chars */
-	char *o2 = strdup("\t option name \t\t = \t value string \t");
-	cdw_assert_test (o2 != NULL, "failed creating test line\n");
-	option = cdw_config_split_options_line(o2);
-	cdw_assert_test (option.name != NULL && option.value != NULL, "failed to split correct line\n");
-	d = strcmp(option.name, "option name");
-	cdw_assert_test (d == 0, "failed at comparing option name\n");
-	d = strcmp(option.value, "value string");
-	cdw_assert_test (d == 0, "failed at comparing option value\n");
-	free(o2);
-	o2 = (char *) NULL;
+		bool rv = cdw_config_split_options_line(&option, line);
+		cdw_assert (input_data[i].expected_return_value == rv, "ERROR: failed at function call#%d\n", i);
 
-	/* correct line, but no value */
-	char *o3 = strdup("\t option name = \t \t");
-	cdw_assert_test (o3 != NULL, "failed creating test line\n");
-	option = cdw_config_split_options_line(o3);
-	cdw_assert_test (option.name != NULL && option.value != NULL, "failed to split correct line\n");
-	d = strcmp(option.name, "option name");
-	cdw_assert_test (d == 0, "failed comparing option name\n");
-	d = strcmp(option.value, "");
-	cdw_assert_test (d == 0, "failed comparint option value\n");
-	free(o3);
-	o3 = (char *) NULL;
+		if (input_data[i].expected_null_name) {
+			cdw_assert (!option.name, "ERROR: name is not NULL (#%d)\n", i);
+		} else {
+			cdw_assert (option.name, "ERROR: name is NULL (#%d)\n", i);
+		}
 
-	/* correct line, but no value 2 */
-	char *o4 = strdup("\t option name = #value string\t \t");
-	cdw_assert_test (o4 != NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o4);
-	cdw_assert_test (option.name != NULL && option.value != NULL, "failed to split correct line\n");
-	d = strcmp(option.name, "option name");
-	cdw_assert_test (d == 0, "failed at comparing option name\n");
-	d = strcmp(option.value, "");
-	cdw_assert_test (d == 0, "failed at comparing option value\n");
-	free(o4);
-	o4 = (char *) NULL;
+		if (input_data[i].expected_null_value) {
+			cdw_assert (!option.value, "ERROR: value is not NULL (#%d)\n", i);
+		} else {
+			cdw_assert (option.value, "ERROR: value is NULL (#%d)\n", i);
+		}
 
-	/* incorrect line, no '=' char 1 */
-	char *o5 = strdup("\t option name \t # = value string\t \t");
-	cdw_assert_test (o5 != NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o5);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
-	free(o5);
-	o5 = (char *) NULL;
+		if (!input_data[i].expected_null_name && input_data[i].expected_null_value) {
 
-	/* incorrect line, no '=' char 2 */
-	char *o6 = strdup("\t option name \t  value string\t \t");
-	cdw_assert_test (o6!= NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o6);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
-	free(o6);
-	o6 = (char *) NULL;
+			cdw_assert (!strcmp(option.name, input_data[i].expected_name), "ERROR: unexpected name #%d: \"%s\"\n", i, option.name);
+			cdw_assert (!strcmp(option.value, input_data[i].expected_value), "ERROR: unexpected value #%d: \"%s\"\n", i, option.value);
+		}
 
-	/* incorrect line, all content is comment */
-	char *o7 = strdup(" \t #\t option name = value string\t \t");
-	cdw_assert_test (o7 != NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o7);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
-	free(o7);
-	o7 = (char *) NULL;
+		if (line) {
+			free(line);
+			line = (char *) NULL;
+		}
 
-	/* incorrect line, empty line */
-	char *o8 = strdup("");
-	cdw_assert_test (o8 != NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o8);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
-	free(o8);
-	o8 = (char *) NULL;
+		i++;
+	}
 
-	/* incorrect line, empty line 2 */
-	char *o9 = strdup(" \t \t    \t\t\t\t     \t               \t#    ");
-	cdw_assert_test (o9 != NULL, "failed creatint test line\n");
-	option = cdw_config_split_options_line(o9);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
-	free(o9);
-	o9 = (char *) NULL;
-
-	/* incorrect line, line = NULL */
-	char *o10 = (char *) NULL;
-	option = cdw_config_split_options_line(o10);
-	cdw_assert_test (option.name == NULL && option.value == NULL, "failed at recognizing incorrect line\n");
 
 	fprintf(stderr, "OK\n");
 
@@ -1866,4 +2018,133 @@ void test_cdw_config_split_options_line(void)
 }
 
 
+
+
+
+void test_cdw_config_split_options_line_new(void)
+{
+	fprintf(stderr, "\ttesting cdw_config_split_options_line_new()... ");
+
+	struct {
+		const char *line;
+		bool expected_return_value;
+		bool expected_null_name;
+		bool expected_null_value;
+		const char *expected_name;
+		const char *expected_value;
+
+	} input_data[] = {
+		/* correct line */
+		{ "option name=value string",
+		  true,
+		  false, false,
+		  "option name", "value string" },
+
+		/* correct line */
+		{ "\t option name \t\t = \t value string \t",
+		  true,
+		  false, false,
+		  "option name", "value string" },
+
+		/* correct line, but no value */
+		{ "\t option name = \t \t",
+		  true,
+		  false, false,
+		  "option name", "" },
+
+		/* correct line, but no value (2) */
+		{ "\t option name = #value string\t \t",
+		  true,
+		  false, false,
+		  "option name", "" },
+
+		/* incorrect line, no '=' char (1) */
+		{ "\t option name \t # = value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, no '=' char (2) */
+		{ "\t option name \t  value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, all content is comment */
+		{ " \t #\t option name = value string\t \t",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, empty line */
+		{ "",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, empty line (2) */
+		{ " \t \t    \t\t\t\t     \t               \t#    ",
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		/* incorrect line, line = NULL */
+		{ (char *) NULL,
+		  false,
+		  true, true,
+		  (char *) NULL, (char *) NULL },
+
+		{ (char *) NULL,
+		  false,
+		  false, true, /* <-- guard */
+		  (char *) NULL, (char *) NULL }
+	};
+
+	cdw_option_t option;
+	option.name = (char *) NULL;
+	option.value = (char *) NULL;
+	int i = 0;
+	while (input_data[i].expected_null_name == input_data[i].expected_null_value) {
+		char *line = (char *) NULL;
+		if (input_data[i].line) {
+			line = strdup(input_data[i].line);
+		}
+
+		bool rv = cdw_config_split_options_line_new(&option, line);
+		cdw_assert (input_data[i].expected_return_value == rv, "ERROR: failed at function call#%d\n", i);
+
+		if (input_data[i].expected_null_name) {
+			cdw_assert (!option.name, "ERROR: name is not NULL (#%d)\n", i);
+		} else {
+			cdw_assert (option.name, "ERROR: name is NULL (#%d)\n", i);
+		}
+
+		if (input_data[i].expected_null_value) {
+			cdw_assert (!option.value, "ERROR: value is not NULL (#%d)\n", i);
+		} else {
+			cdw_assert (option.value, "ERROR: value is NULL (#%d)\n", i);
+		}
+
+		if (!input_data[i].expected_null_name && input_data[i].expected_null_value) {
+			cdw_assert (!strcmp(option.name, input_data[i].expected_name), "ERROR: unexpected name #%d: \"%s\"\n", i, option.name);
+			cdw_assert (!strcmp(option.value, input_data[i].expected_value), "ERROR: unexpected value #%d: \"%s\"\n", i, option.value);
+		}
+
+		cdw_config_option_free_new(&option);
+		if (line) {
+			free(line);
+			line = (char *) NULL;
+		}
+
+		i++;
+	}
+
+
+	fprintf(stderr, "OK\n");
+
+	return;
+}
+
+
+
 #endif /* #ifdef CDW_UNIT_TEST_CODE */
