@@ -41,9 +41,10 @@ typedef struct {
 	char *value; /**< \brief Value of option, string that stands on right-hand side of '=' char in configuration file */
 } cdw_option_t;
 
-/* this is in header file only because unit testing code requires prototype */
-cdw_option_t cdw_config_split_options_line(char *line);
 
+bool cdw_config_split_options_line(cdw_option_t *option, char *line);
+bool cdw_config_split_options_line_new(cdw_option_t *option, char *line);
+void cdw_config_option_free_new(cdw_option_t *option);
 
 
 /** \brief Main cdw configuration variable, storing all major options used by cdw */
