@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -147,7 +147,7 @@ cdw_rv_t cdw_config_window(void)
 	cdw_config_window_destroy();
 	/* user may have changed "ISO volume size", make this change
 	   visible in main window */
-	cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+	cdw_main_window_volume_info_view_update(-1, -1, false, global_config.follow_symlinks);
 
 	cdw_main_ui_main_window_wrefresh();
 
@@ -189,7 +189,7 @@ cdw_rv_t cdw_config_window(void)
 #define PAGE_HW_N_FIELDS         7  /* hardware */
 #define PAGE_AUDIO_N_FIELDS      3
 #define PAGE_TOOLS_N_FIELDS     15
-#define PAGE_OTHER_N_FIELDS      9  /* log and other */
+#define PAGE_OTHER_N_FIELDS     11  /* log and other */
 
 
 /* number of pages in configuration window */
@@ -294,7 +294,9 @@ cdw_rv_t cdw_config_window_create(cdw_config_t *config)
 	/* 2TRANS: this is message at the bottom of configuration window;
 	   cancel means: quit without saving; '%d' is an integer used to
 	   create label of function key, e.g. "F10" */
-	int rv = asprintf(&cdw_config_window_tip, _("Press F%d key to save changes or ESC to cancel"), CDW_CONFIG_UI_SnC_KEY);
+	int rv = asprintf(&cdw_config_window_tip,
+			  _("Press F%d / F%d key to save changes or ESC to cancel"),
+			  CDW_CONFIG_UI_SnC_KEY_B, CDW_CONFIG_UI_SnC_KEY_A);
 	if (cdw_config_window_tip == (char *) NULL || rv == -1) {
 		cdw_vdm ("ERROR: failed to create window tip\n");
 		return CDW_ERROR;
@@ -324,10 +326,8 @@ cdw_rv_t cdw_config_window_create(cdw_config_t *config)
 	cdw_tabs_window_append_tab(c_twindow, id, KEY_F(ind + 2), c_pages[id].label); /* hardware */
 	ind++;
 
-	cdw_tabs_window_add_return_key(c_twindow, KEY_F(CDW_CONFIG_UI_SnC_KEY));
-	cdw_tabs_window_add_return_key(c_twindow, CDW_KEY_ESCAPE);
-	cdw_tabs_window_add_return_key(c_twindow, 'q');
-	cdw_tabs_window_add_return_key(c_twindow, 'Q');
+	cdw_tabs_window_add_return_keys(c_twindow, KEY_F(CDW_CONFIG_UI_SnC_KEY_A), KEY_F(CDW_CONFIG_UI_SnC_KEY_B),
+					CDW_KEY_ESCAPE, 'q', 'Q', 0);
 	c_twindow->driver_reader = cdw_config_ui_tabs_window_form_reader;
 	cdw_tabs_window_finalize(c_twindow);
 
@@ -357,7 +357,8 @@ cdw_rv_t cdw_config_window_create(cdw_config_t *config)
 		for (int j = 0; j < N_CONFIG_PAGES_ALL; j++) {
 			cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(j + CDW_CONFIG_UI_FX_START_KEY));
 		}
-		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_A));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_B));
 		cdw_form_add_return_char(c_pages[id].cdw_form, 'q'); /* will work only for non-text fields */
 	}
 	c_twindow->user_data = (void *) c_pages;
@@ -426,7 +427,8 @@ cdw_rv_t cdw_config_window_driver(cdw_config_t *config)
 		int key = cdw_tabs_window_driver(c_twindow);
 		if (key == CDW_KEY_ESCAPE) {
 			break;
-		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY)) { /* SnC = Save and Close */
+		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY_A)
+			   || key == KEY_F(CDW_CONFIG_UI_SnC_KEY_B)) { /* SnC = Save and Close */
 
 			/* flush */
 			for (int i = 0; i < c_twindow->n_tabs; i++) {
@@ -434,6 +436,8 @@ cdw_rv_t cdw_config_window_driver(cdw_config_t *config)
 				form_driver(c_pages[id].cdw_form->form, REQ_VALIDATION);
 			}
 
+			bool old_follow_symlinks = config->follow_symlinks;
+
 			/* this function also saves indexes of selected
 			   tool paths into cdw_tools[] */
 			cdw_config_window_save_config_fields(config);
@@ -469,6 +473,8 @@ cdw_rv_t cdw_config_window_driver(cdw_config_t *config)
 				   be saved in other places */
 				cdw_config_window_save_non_config_fields();
 
+				cdw_main_ui_handle_follow_symlinks_change(old_follow_symlinks, config->follow_symlinks);
+
 				/* CDW_OK means: values from "config" function
 				   argument can be saved to global config variable */
 				return CDW_OK;
@@ -562,6 +568,7 @@ cdw_rv_t page_audio_build_fields(cdw_form_t *cdw_form, void *cdw_config, int fir
 		{ CDW_WIDGET_INPUT,   2,  first_col,  width_wide - 3,  1,  f_audiodir_i,  config->audiodir,       0 },
 		/* this fields is created only because there will be a button
 		   here, that I need to be able to navigate to, so string can be empty */
+		/* 2TRANS: do not translate */
 		{ CDW_WIDGET_BUTTON,  2,  audio_dir_button_col,  1,    1,  f_audiodir_b,  _(">"), CDW_COLORS_DIALOG },
 		/* guard */
 		{ -1,                 0,  0,          0,               0,  0,             (void *) NULL,          0 } };
@@ -685,6 +692,7 @@ cdw_rv_t page_other_build_fields(cdw_form_t *cdw_form, void *cdw_config, int fir
 		{ CDW_WIDGET_INPUT,     2,  first_col,       width_wide - 3,  1,  f_log_fp_i,            config->log_full_path,                0 },
 		/* this fields is created only because there will be a button
 		   here, that I need to be able to navigate to, so string can be empty */
+		/* 2TRANS: do not translate */
 		{ CDW_WIDGET_BUTTON,    2,  log_path_button_col,  3,          1,  f_log_fp_b,            _(">"),               CDW_COLORS_DIALOG },
 		{ CDW_WIDGET_CHECKBOX,  3,  first_col + 1,   1,               1,  f_showlog_cb,          (void *) NULL,  config->showlog ? 1 : 0 },
 		/* 2TRANS: this is checkbox label: keep no longer than original */
@@ -696,6 +704,12 @@ cdw_rv_t page_other_build_fields(cdw_form_t *cdw_form, void *cdw_config, int fir
 		/* 2TRANS: this is input field label, user can enter into the field a custom value of max. size of ISO volume; "MB" is megabytes */
 		{ CDW_WIDGET_LABEL,     8,  first_col,       width_narrow,    1,  f_cust_volume_size_l,  _("Custom ISO volume size (MB):"),    0 },
 		{ CDW_WIDGET_INPUT,     8,  second_col + 5,  CDW_CVF_WIDTH,   1,  f_cust_volume_size_i,  cvf_buffer,               CDW_CVF_WIDTH },
+
+		{ CDW_WIDGET_CHECKBOX, 11,  first_col + 1,   1,               1,  f_follow_symlinks_cb,  (void *) NULL,  config->follow_symlinks ? 1 : 0 },
+		/* 2TRANS: this is checkbox label: keep no longer than original */
+		{ CDW_WIDGET_LABEL,    11,  first_col + 4,   width_wide - 3,  1,  f_follow_symlinks_l,  _("Follow added symbolic links"),      0 },
+
+
 		/* guard */
 		{ -1,                   0,  0,             0,                 0,  0,                     (void *) NULL,                        0 } };
 
@@ -776,7 +790,6 @@ void cdw_config_window_save_non_config_fields(void)
 */
 void cdw_config_window_save_config_fields(cdw_config_t *config)
 {
-	cdw_rv_t ss;
 
 	/* *** HARDWARE *** */
 	cdw_form_t *cdw_form = c_pages[CONFIG_PAGE_ID_HW].cdw_form;
@@ -799,7 +812,7 @@ void cdw_config_window_save_config_fields(cdw_config_t *config)
 
 	/* *** LOGGING AND OTHER *** */
 	cdw_form = c_pages[CONFIG_PAGE_ID_OTHER].cdw_form;
-	ss = cdw_string_set(&(config->log_full_path), cdw_ncurses_get_field_string(*(cdw_form->fields + f_log_fp_i)));
+	cdw_string_set(&(config->log_full_path), cdw_ncurses_get_field_string(*(cdw_form->fields + f_log_fp_i)));
 	config->showlog = cdw_form_get_checkbox_state(cdw_form, f_showlog_cb);
 	dropdown = cdw_form_get_dropdown(cdw_form, f_volume_size_dd);
 	config->volume_size_id = cdw_dropdown_get_current_item_id(dropdown);
@@ -816,6 +829,9 @@ void cdw_config_window_save_config_fields(cdw_config_t *config)
 	} else {
 		cdw_vdm ("INFO: volume size string \"%s\" converted to int %ld\n", cvs_buffer, config->volume_size_custom_value);
 	}
+
+	config->follow_symlinks = cdw_form_get_checkbox_state(cdw_form, f_follow_symlinks_cb);
+
 #if 0
 
 	/* TODO: not a best place to set config->volume_size_value,
@@ -999,12 +1015,7 @@ char *cdw_config_window_page_tools_message(int n_cols)
 	/* 2TRANS: this is a description in one of
 	   configuration window panels; a series of
 	   dropdown widgets is displayed below the label */
-	char *message = _("Changes made here won't be saved after closing cdw.");
-
-        int n_lines = cdw_string_wrap_for_form_get_n_lines(message, n_cols);
-	char *new_message = cdw_string_wrap_for_form(message, n_cols, n_lines);
-
-	return new_message;
+	return cdw_string_wrap(_("Changes made here won't be saved after closing cdw."), (size_t) n_cols, CDW_ALIGN_LEFT);
 }
 
 
@@ -1014,8 +1025,7 @@ char *cdw_config_window_page_tools_message(int n_cols)
 char *cdw_config_window_page_hw_message(int n_cols)
 {
 	char *message = cdw_drive_get_help_message();
-	int n_lines = cdw_string_wrap_for_form_get_n_lines(message, n_cols);
-	char *new_message = cdw_string_wrap_for_form(message, n_cols, n_lines);
+	char *new_message = cdw_string_wrap(message, (size_t) n_cols, CDW_ALIGN_LEFT);
 
 	free(message);
 	message = (char *) NULL;
