@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,9 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "cdw_task.h"
 #include "cdw_disc.h"
 #include "cdw_drive.h"
 #include "cdw_config.h"
-#include "cdw_ext_tools.h"
 #include "cdw_dvd_rw_mediainfo.h"
 #include "cdw_cdrecord.h"
 #include "cdw_xorriso.h"
@@ -36,8 +34,7 @@
 #include "cdw_processwin.h"
 #include "cdw_fs.h"
 #include "cdw_utils.h"
-#include "cdw_sys.h"
-
+#include "cdw_ofs.h"
 
 /**
    \file cdw_disc.c
@@ -119,16 +116,18 @@ Size    Time                  Mode 1        Mode 2        Form 1        Form 2
 
 
 
-/** \brief Variable storing information about disc currently in drive */
-cdw_disc_t current_disc;
 
 extern cdw_config_t global_config; /* main cdw configuration variable */
 
+static void cdw_disc_reset(cdw_disc_t *disc);
+
+static bool cdw_disc_get_make_processwin(void);
+
+static cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc);
 static cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc);
 static cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc);
 
 static cdw_rv_t cdw_disc_validate(cdw_disc_t *disc);
-static int cdw_disc_get_default_drive_speed_id(cdw_disc_t *disc);
 static void cdw_disc_resolve_type_label(cdw_disc_t *disc);
 static void cdw_disc_resolve_write_speeds(cdw_disc_t *disc);
 static void cdw_disc_resolve_capacities(cdw_disc_t *disc);
@@ -136,6 +135,9 @@ static void cdw_disc_resolve_capacities(cdw_disc_t *disc);
 static void cdw_disc_debug_print_disc(cdw_disc_t *disc);
 
 
+
+
+
 /* these labels are in the same order as in cdw_disc_type_t type
    declaration in cdw_disc.h */
 static const char *cdw_disc_type_labels[] = {
@@ -204,7 +206,7 @@ void cdw_disc_init(void)
 	   checking the disc */
 	disc_not_readable_reasons[CDW_DISC_NOT_READABLE_OTHER].label = _("cdw can't read the disc.");
 
-	cdw_disc_reset(&current_disc);
+
 
 	return;
 }
@@ -214,16 +216,229 @@ void cdw_disc_init(void)
 
 
 /**
-   \brief Simple access function
+   \brief Create new disc data structure, initialize it with default values
 
-   Function returns pointer to global variable representing current disc.
-   The pointer is always valid, but information in the variable may be
-   invalid or outdated. Use other functions to bring state of the variable
-   up to date.
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
+
+   Allocate new data structure of type cdw_disc_t, initialize all its
+   fields with default, initial values. Return pointer to the data structure.
+   The function also initializes cdw_disc_t->cdio.
+
+   You have to call cdw_disc_delete() to properly deallocate the data
+   structure.
+
+   \return pointer to disc data structure on success
+   \return NULL pointer on failure
 */
-cdw_disc_t *cdw_disc_get(void)
+cdw_disc_t *cdw_disc_new(void)
 {
-	return &current_disc;
+	cdw_disc_t *disc = (cdw_disc_t *) malloc(sizeof (cdw_disc_t));
+	if (!disc) {
+		cdw_vdm ("ERROR: malloc\n");
+		return (cdw_disc_t *) NULL;
+	}
+
+	cdw_disc_reset(disc);
+
+	disc->cdio = cdw_cdio_new();
+	if (!disc->cdio) {
+		cdw_vdm ("ERROR: cdw_cdio_new()\n");
+		free(disc);
+		disc = (cdw_disc_t *) NULL;
+	}
+
+	return disc;
+}
+
+
+
+
+
+/**
+   \brief Get metainformation about a disc
+
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
+
+   Get all available metainformation about a disc (well, at least
+   those pieces of information that are needed to fill fields of
+   cdw_disc_t data structure). Save the information in \p disc.
+
+   The function searches for disc in default optical disc drive.
+   Then it checks if there is any disc in the drive, and whether the
+   disc is unmounted. It also does few more things, so that you
+   just have to make one function call to 'get' a disc.
+
+   \p disc should be created first with cdw_disc_new().
+   Don't call the function twice with the same \p disc, as this would
+   cause memory leaks.
+
+   \p disc - data structure to which to save disc metainformation.
+
+   Function returns CDW_ERROR if there were some serious errors during
+   execution of the function. Failed call to strdup() is such an error,
+   but no disc in drive is not an error.
+
+   \return CDW_ERROR if errors occur during information retrieval
+   \return CDW_NO if information about disc can't be collected
+   \return CDW_OK if information about disc has been collected
+*/
+cdw_rv_t cdw_disc_get(cdw_disc_t *disc)
+{
+	cdw_assert (disc, "ERROR: disc is NULL\n");
+	if (!disc) {
+		/* probably cdw_disc_new() returned NULL, caller
+		   didn't check this, and called this function
+		   with NULL parameter */
+		return CDW_NO;
+	}
+
+	bool local_processwin = cdw_disc_get_make_processwin();
+
+	cdw_rv_t retval = CDW_ERROR;
+	const char *device_fullpath = cdw_drive_get_drive_fullpath();
+	if (!device_fullpath) {
+		cdw_vdm ("WARNING: no drive\n");
+		retval = CDW_NO;
+		goto cdw_get_disc_return;
+	}
+
+	disc->device_fullpath = strdup(device_fullpath);
+	if (!disc->device_fullpath) {
+		cdw_vdm ("ERROR: strdup()\n");
+		retval = CDW_ERROR;
+		goto cdw_get_disc_return;
+	}
+
+	cdw_rv_t crv = cdw_drive_disc_availability();
+	if (crv != CDW_OK) {
+		cdw_vdm ("WARNING: cdw_drive_availability()\n");
+		retval = crv;
+		goto cdw_get_disc_return;
+	}
+
+	crv = cdw_fs_check_device_mounted(disc->device_fullpath);
+	if (crv == CDW_ERROR) {
+		cdw_vdm ("WARNING: checking if disc is mounted ended with error\n");
+		/* let's try and proceed with further checks anyway */
+	} else if (crv == CDW_OK) { /* device is mounted */
+		cdw_vdm ("INFO: optical disc is mounted\n");
+		/* 2TRANS: this is title of dialog window */
+		cdw_buttons_dialog(_("Error"),
+				   /* 2TRANS: this is message in dialog window */
+				   _("Cannot check media: disc is mounted. Please unmount your optical disc."),
+				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
+		retval = CDW_NO;
+		goto cdw_get_disc_return;
+	} else { /* mounted == CDW_NO, not mounted */
+		cdw_vdm ("INFO: disc is not mounted\n");
+	}
+
+
+	crv = cdw_disc_get_meta_info(disc);
+	if (crv == CDW_ERROR) {
+		cdw_vdm ("ERROR: failed to get disc meta info\n");
+		cdw_disc_reset(disc);
+		retval = CDW_ERROR;
+		goto cdw_get_disc_return;
+	} else {
+		cdw_disc_resolve(disc);
+		retval = CDW_OK;
+		goto cdw_get_disc_return;
+	}
+
+ cdw_get_disc_return:
+	if (local_processwin) {
+		cdw_processwin_destroy("", false);
+	}
+
+	return retval;
+}
+
+
+
+
+
+/**
+   \brief Create processwin needed during execution of cdw_disc_get()
+
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
+
+   During execution of cdw_disc_get() and functions called by cdw_disc_get(),
+   some information needs to be displayed in a process window. This function
+   checks if one exists already, and if not, it creates it.
+   The process window is created with default title and message, appropriate
+   for process of getting disc metainformation.
+
+   If the process window has been created by the function, the function
+   returns true. It returns false otherwise.
+
+   \return true if the function created process window
+   \return false otherwise
+*/
+bool cdw_disc_get_make_processwin(void)
+{
+	if (cdw_processwin_is_active()) {
+		return false;
+	} else {
+		/* 2TRANS: this is title of process window; reading
+		   disc meta information */
+		cdw_processwin_create(_("Read disc information"),
+				      /* 2TRANS: this is message in
+					 process window */
+				      _("Reading disc information"),
+				      false);
+		return true;
+	}
+}
+
+
+
+
+
+/**
+   \brief Deallocate cdw_disc_t data structure
+
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
+
+   Deallocate all memory used by given \p disc.
+   Handle deallocating cdw_disc_t->cdio as well.
+   Set the disc to NULL.
+
+   Function accepts NULL disc (*disc == NULL). It does not accept
+   NULL pointer (disc == NULL).
+
+   \param disc - pointer to disc to deallocate
+*/
+void cdw_disc_delete(cdw_disc_t **disc)
+{
+	cdw_assert (disc, "ERROR: pointer to disc is NULL\n");
+	if (!disc) {
+		cdw_vdm ("ERROR: pointer to disc is NULL\n");
+		return;
+	}
+
+	if (!(*disc)) {
+		cdw_vdm ("WARNING: disc is NULL\n");
+		return;
+	}
+
+	if ((*disc)->cdio) {
+		cdw_cdio_delete(&(*disc)->cdio);
+	}
+
+	if ((*disc)->device_fullpath) {
+		free((*disc)->device_fullpath);
+		(*disc)->device_fullpath = (char *) NULL;
+	}
+
+	free(*disc);
+	*disc = (cdw_disc_t *) NULL;
+
+	return;
 }
 
 
@@ -233,23 +448,23 @@ cdw_disc_t *cdw_disc_get(void)
 /**
    \brief Reset values in a disc
 
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
+
    Set some values of \p disc to their initial state, which corresponds
-   to 'no disc in drive'.
+   to 'no disc in drive'. Don't touch cdw_disc_t->cdio nor any of fields
+   in cdw_disc_t->cdio.
 
    \param disc - disc to reset
 */
 void cdw_disc_reset(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
-	disc->simple_type = CDW_DISC_SIMPLE_TYPE_UNKNOWN;
-	disc->simple_type_label[0] = '\0';
 	disc->type = CDW_DISC_TYPE_UNKNOWN;
 	disc->type_label[0] = '\0';
 
-	disc->cdio_disc_mode = CDIO_DISC_MODE_NO_INFO;
-
-	disc->n_tracks = -1;
+	disc->device_fullpath = (char *) NULL;
 
 	disc->state_empty = CDW_UNKNOWN;
 	disc->state_writable = CDW_UNKNOWN;
@@ -259,10 +474,8 @@ void cdw_disc_reset(cdw_disc_t *disc)
 	disc->capacity.sectors_total = 0;
 	disc->capacity.sectors_used = 0;
 
-	disc->fs.type = -1;
-	disc->fs.type_label = (char *) NULL;
-	disc->fs.volume_id[0] = '\0';
-        disc->fs.n_sectors = 0;
+
+
 	disc->write_speeds.n_speeds = -1;
 	disc->write_speeds.drive_max_speed = -1;
 	disc->write_speeds.drive_default_speed = -1;
@@ -305,6 +518,7 @@ void cdw_disc_reset(cdw_disc_t *disc)
 	disc->libburn_info.formatting_status = BURN_FORMAT_IS_UNKNOWN;
 	disc->libburn_info.read_capacity = -1;
 
+	/* TODO: should we add call to cdw_cdio_reset()? */
 
 	/* take care of other things that should be recalculated based
 	   on the assignments made in this function */
@@ -318,28 +532,34 @@ void cdw_disc_reset(cdw_disc_t *disc)
 
 
 /**
-   \brief Inspect given disc and current cdw configuration to get index of initial speed value
+   \brief Get initial value of burning/erasing speed for given disc
 
-   Inspect speed range (selected by user in configuration) to calculate
-   index that can be used to initialize dropdown with writing speeds or
-   to get initial value of writing speed.
+   \date Function's top-level comment reviewed on 2012-01-29
+   \date Function's body reviewed on 2012-01-29
 
-   You need to call cdw_disc_validate() before calling this
-   function to ensure that disc->write_speeds.n_speeds > 0.
+   Check \p disc to see what should be the initial/default value
+   of burning/erasing speed for the \p disc. The value should have
+   been saved in \p disc during collecting of disc meta data.
 
-   \param disc - disc for which to get initial write speed index
+   The value is either default speed for given combination of
+   disc + drive, or (if there is no such speed saved), first value
+   on list of supported burning speeds.
 
-   \return correct index of speed in writing_speeds on success
+   'id' in function's name means 'value'.
+
+   \param disc - disc for which to get initial write speed
+
+   \return value of initial burning/erasing speed
 */
 int cdw_disc_get_initial_write_speed_id(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 	cdw_assert (disc->write_speeds.n_speeds > 0, "ERROR: you forgot to call cdw_disc_validate()\nand now n_speeds is non-positive\n");
 
-	if (disc->write_speeds.n_speeds == 1) {
+	if (disc->write_speeds.drive_default_speed != -1) {
+		return disc->write_speeds.drive_default_speed;
+	} else {
 		return disc->write_speeds.speeds[0];
-	} else { /* n_speeds > 1 */
-		return cdw_disc_get_default_drive_speed_id(disc);
 	}
 }
 
@@ -348,13 +568,19 @@ int cdw_disc_get_initial_write_speed_id(cdw_disc_t *disc)
 
 
 /**
-   \brief Get from disc all available meta information about the disc
+   \brief Get metainformation from a disc
 
-   Get as much meta information about disc as possible, using available
-   tools: libcdio calls, and calls to cdrecord or dvd+rw-mediainfo.
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
 
-   The function is smart enough to determine when to call cdrecord
-   and when to call dvd+rw-mediainfo.
+   Get meta information from disc currently in drive.
+   The function should be invoked after ensuring that an optical disc
+   exists in a drive, and that the disc is not mounted.
+
+   Meta information is collected using first with cdio calls and then
+   with external tools (cdrecord, dvd+rw-mediainfo, xorriso).
+   The high-level information is then resolved using cdw_disc_resolve(),
+   and validated with cdw_disc_validate().
 
    \param disc - disc variable into which to store the information
 
@@ -363,16 +589,16 @@ int cdw_disc_get_initial_write_speed_id(cdw_disc_t *disc)
 */
 cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
 	cdw_rv_t crv = cdw_disc_get_meta_info_with_cdio(disc);
-	if (crv != CDW_OK) {
+	if (crv != OK) {
 		cdw_vdm ("ERROR: failed to open disc with cdio calls\n");
 		return CDW_ERROR;
 	}
 
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD
-	    && disc->fs.type == CDIO_FS_AUDIO) {
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD
+	    && disc->cdio->ofs->type == CDIO_FS_AUDIO) {
 
 		/* we won't call external tools to get more meta information
 		   about a disc, so for audio CDs we have to set type here */
@@ -446,51 +672,32 @@ cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 
 
 
-/* *** local functions *** */
-
-
-
-
 /**
    \brief Get basic disc meta information using libcdio calls
 
-   Get information such as simple disc type, file system on a disc,
-   file system readable label, volume id.
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   Get information such as simple disc type, number of tracks on a disc,
+   file system on a disc, and some more information about the file system.
 
    \param disc - disc variable into which to store the information
 
    \return CDW_OK on success
-   \return CDW_ERROR on error
+   \return CDW_ERROR when function failed to get information (e.g. because of error)
 */
 cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
-	cdw_rv_t crv = cdw_cdio_open_disc();
-	if (crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to open disc with cdio calls\n");
+	cdw_rv_t crv = cdw_cdio_get(disc->cdio, disc->device_fullpath);
+	if (crv == CDW_ERROR) {
+		cdw_vdm ("ERROR: cdw_cdio_get()\n");
 		return CDW_ERROR;
 	}
 
-	disc->simple_type = cdw_cdio_get_simple_type();
-	disc->cdio_disc_mode = cdw_cdio_get_disc_mode();
-	disc->state_empty = cdw_cdio_is_blank() ? CDW_TRUE : CDW_FALSE;
-	disc->fs.type = cdw_cdio_get_fs_type();
-	disc->fs.type_label = cdw_cdio_get_fs_type_label();
-	disc->fs.n_sectors = cdw_cdio_get_fs_n_sectors();
-	disc->n_tracks = cdw_cdio_get_number_of_tracks();
-	/* cdw_cdio_get_volume_id() always returns pointer to valid C string
-	   no longer than ISO_MAX_VOLUME_ID + 1 (including ending '\0') */
-	strncpy(disc->fs.volume_id, cdw_cdio_get_volume_id(), ISO_MAX_VOLUME_ID + 1);
-
-	cdw_cdio_close_disc();
-
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
-		strcpy(disc->simple_type_label, "CD");
-	} else if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
-		strcpy(disc->simple_type_label, "DVD");
-	} else {
-		cdw_vdm ("ERROR: failed to get meta info for disc %d, which is not CD nor DVD\n", disc->simple_type);
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_UNKNOWN) {
+		cdw_vdm ("ERROR: failed to get meta info for disc %d, which is not CD nor DVD\n", disc->cdio->simple_type);
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
 				   /* 2TRANS: this is message dialog window,
@@ -500,7 +707,6 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 		cdw_main_ui_main_window_wrefresh();
 		return CDW_ERROR;
 	}
-	cdw_vdm ("INFO: disc simple type is \"%s\"\n", disc->simple_type_label);
 
 	return CDW_OK;
 }
@@ -512,6 +718,9 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 /**
    \brief Call external tools to collect meta information about a disc
 
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
    Function checks disc type, then checks which tool should check
    disc meta information, and then calls function calling the tool
    to do the job.
@@ -523,10 +732,10 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 */
 cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
 	cdw_task_t *task = cdw_task_new(CDW_TASK_MEDIA_INFO, disc);
-	if (task == (cdw_task_t *) NULL) {
+	if (!task) {
 		cdw_vdm ("ERROR: failed to create a task\n");
 		return CDW_ERROR;
 	}
@@ -535,7 +744,7 @@ cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 		    || tool_id == CDW_TOOL_XORRISO
 		    || tool_id == CDW_TOOL_DVD_RW_MEDIAINFO,
 		    "ERROR: tool selector returns unsupported tool %lld\n", tool_id);
-	cdw_assert (task->media_info.tool.label != (char *) NULL,
+	cdw_assert (task->media_info.tool.label,
 		    "ERROR: tool selector returns NULL path to tool\n");
 
 	disc->media_info_source = tool_id;
@@ -561,10 +770,7 @@ cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 	cdw_task_delete(&task);
 
 	if (tool_status != CDW_OK ||  crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to get media info with %s\n",
-			 tool_id == CDW_TOOL_CDRECORD ? "cdrecord" :
-			 tool_id == CDW_TOOL_XORRISO ? "xorriso" :
-			 tool_id == CDW_TOOL_DVD_RW_MEDIAINFO ? "dvd+rw-mediainfo" : "unknown tool");
+		cdw_vdm ("ERROR: failed to get media info with %s\n", cdw_ext_tools_get_tool_name(tool_id));
 		return CDW_ERROR;
 	} else {
 		return CDW_OK;
@@ -575,25 +781,12 @@ cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 
 
 
-int cdw_disc_get_default_drive_speed_id(cdw_disc_t *disc)
-{
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
-
-	if (disc->write_speeds.drive_default_speed == -1) {
-		/* default drive speed is unknown */
-		return disc->write_speeds.speeds[0];
-	} else {
-		return disc->write_speeds.drive_default_speed;
-	}
-}
-
-
-
-
-
 /**
    \brief Remove any repeating values from table of writing speeds, sort the speeds
 
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
    When parsing output of "cdrecord -prcap", cdw puts speeds available
    for a disc in table. The problem is that the speed values can repeat
    in the table (cdrecord prints them this way). This function removes
@@ -616,7 +809,7 @@ int cdw_disc_get_default_drive_speed_id(cdw_disc_t *disc)
 void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 {
 #ifndef NDEBUG
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 	cdw_vdm ("INFO: table of write speeds before resolving:\n");
 	for (int i = 0; i < CDW_DISC_N_SPEEDS_MAX; i++) {
 		if (disc->write_speeds.speeds[i] != 0) {
@@ -626,7 +819,7 @@ void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 #endif
 
 	/* number of speeds in table has been limited at the level of parsing
-	   cdrecord / dvd+rw-mediainfo output, so at this point we are
+	   cdrecord/dvd+rw-mediainfo/libburn output, so at this point we are
 	   sure that there is no more than CDW_DISC_N_SPEEDS_MAX
 	   speed values in write_speeds[] table
 
@@ -634,7 +827,7 @@ void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 	   example list of speeds from dvd+rw-mediainfo: 16, 16, 12, 10, 8, 4
 	   example list of speeds from libburn: 16, 12, 10, 8, 4, 16, 16, 12, 10, 8, 4, 16 */
 
-	/* algorithm of resolving speeds table:
+	/* algorithm of resolving any table of speeds:
 	   1. sort from highest to lowest (with possible zeros at the end of result table);
 	   2. remove duplicates (compress);
 	   3. count non-zero speeds to get number of valid speeds: n_speeds;
@@ -670,6 +863,9 @@ void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 /**
    \brief Check disc data structure for any obvious errors
 
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
    The function checks if fields of \p disc don't contain any
    obvious errors or inconsistencies.
 
@@ -683,10 +879,10 @@ void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 */
 cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
-	if (disc->simple_type != CDW_DISC_SIMPLE_TYPE_CD
-	    && disc->simple_type != CDW_DISC_SIMPLE_TYPE_DVD) {
+	if (disc->cdio->simple_type != CDW_DISC_SIMPLE_TYPE_CD
+	    && disc->cdio->simple_type != CDW_DISC_SIMPLE_TYPE_DVD) {
 
 		/* simple_type _has to_ be known to do anything useful */
 		cdw_vdm ("ERROR: disc is neither CD nor DVD\n");
@@ -728,10 +924,13 @@ cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
    \brief Calculate values of some fields in disc that represent
    higher-level information about disc
 
-   The function checks some fields in \p disc that were set by
-   cdrecord- or dvd+rw-mediainfo-related code (let's call them lower-level
-   data), and based on values of these fields it sets values of some fields
-   representing higher-level information.
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   The function checks some fields in \p disc that were set by external
+   tools modules (let's call the fields lower-level data), and based on
+   values of these fields it sets values of some fields representing
+   higher-level information.
 
    Simple example: "if disc type is DVD-RW then disc is erasable", or
    "if disc type is CD-RW and certain values related to session information
@@ -755,7 +954,7 @@ cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
 */
 cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
 	/* first set disc label, it may be needed later */
 	cdw_disc_resolve_type_label(disc);
@@ -768,8 +967,11 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 	/* don't modify disc->state_empty, it was already set using
 	   information about file system on the disc */
 
-	const char *drive = cdw_drive_get_drive_fullpath();
-	int status = cdw_drive_status(drive);
+	if (!disc->device_fullpath) {
+		return CDW_NO;
+	}
+
+	int status = cdw_drive_status(disc->device_fullpath);
 	if (status == CDS_NO_DISC || status == CDS_TRAY_OPEN) {
 		/* this check is to avoid further unnecessary checks
 		   and possible printing of warning messages that
@@ -844,11 +1046,18 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 	}
 
 
-	/* now easy part */
+	if (disc->state_empty == CDW_UNKNOWN) {
+		cdw_vdm ("WARNING: state_empty = UNKNOWN\n");
+		cdw_vdm ("WARNING: current ofs type = %d / %s\n", disc->cdio->ofs->type, disc->cdio->ofs->type_label);
 
-	/* disc->state_empty was set earlier */
+		if (disc->cdio->blank) {
+			disc->state_empty = CDW_TRUE;
+		} else {
+			disc->state_empty = CDW_FALSE;
+		}
+	}
 
-	/* some common tool independent settings;
+	/* some common, tool independent settings;
 	   these "state values" may have been set by tool-dependent
 	   code, but here cover some special cases, in tool independent fashion */
 	if (disc->type == CDW_DVD_RWP
@@ -904,6 +1113,9 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 /**
    \brief Function sets proper disc type label
 
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
    Function checks type of given \p disc (an integer value) and
    copies proper human-readable label to \p disc->type_label field.
    Disc type must be set before calling this function.
@@ -915,7 +1127,7 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 */
 void cdw_disc_resolve_type_label(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
 	char *l = disc->type_label;
 
@@ -939,6 +1151,9 @@ void cdw_disc_resolve_type_label(cdw_disc_t *disc)
 /**
    \brief Function checking if given disc is readable
 
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
    Disc is readable if it is not empty and is of proper (supported) type
    and has supported file system.
 
@@ -949,35 +1164,36 @@ void cdw_disc_resolve_type_label(cdw_disc_t *disc)
 */
 const char *cdw_disc_check_if_is_readable(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
 	if (disc->state_empty == CDW_TRUE) {
 		cdw_vdm ("ERROR: disc is empty, shouldn't you have checked this before calling this function?\n");
 		return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_EMPTY].label;;
 	}
 
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
 		if (disc->type == CDW_CD_AUDIO) {
 			/* reading Audio CDs with cdio calls */
 			return (char *) NULL;
 		}
-		if (disc->cdio_disc_mode == CDIO_DISC_MODE_CD_DATA /* Mode1/Mode2 Formless? */
-		    || disc->cdio_disc_mode == CDIO_DISC_MODE_CD_XA /* Mode2 Form1 / Mode2 Form2 ? */
-
-		    /* well, this is redundant, covered by "if" above */
-		    || disc->cdio_disc_mode == CDIO_DISC_MODE_CD_DA
-
+		if (disc->cdio->mode == CDIO_DISC_MODE_CD_DATA /* Mode1/Mode2 Formless? */
+		    || disc->cdio->mode == CDIO_DISC_MODE_CD_XA /* Mode2 Form1 / Mode2 Form2 ? */
 		    /* not 100% sure what this is, but cdw doesn't support it */
 		    /* ||  disc->cdio_disc_mode == CDIO_DISC_MODE_CD_MIXED */ ) {
 
-			/* reading CDs with cdio calls, so I'm not
-			   caring about file system support in operating system */
+			if (cdw_ofs_is_iso(disc->cdio->ofs->type)) {
+				return (char *) NULL;
+			}
+		} else if (disc->cdio->mode == CDIO_DISC_MODE_CD_DA) {
+			/* well, this check is redundant, covered by earlier "if" */
 			return (char *) NULL;
+		} else {
+			;
 		}
 
 		return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_OTHER].label;
-	} else if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
-		if (cdw_cdio_is_fs_iso(disc->fs.type)) {
+	} else if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
+		if (cdw_ofs_is_iso(disc->cdio->ofs->type)) {
 
 			return (char *) NULL;
 
@@ -1010,7 +1226,7 @@ const char *cdw_disc_check_if_is_readable(cdw_disc_t *disc)
 			return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_UNSUPPORTED_FS].label;;
 		}
 	} else {
-		cdw_assert (0, "ERROR: incorrect simple disc type %d\n", disc->simple_type);
+		cdw_assert (0, "ERROR: incorrect simple disc type %d\n", disc->cdio->simple_type);
 		return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_OTHER].label;
 	}
 }
@@ -1020,15 +1236,18 @@ const char *cdw_disc_check_if_is_readable(cdw_disc_t *disc)
 
 
 /**
-   \brief Debug function displaying on stderr basic information about a disc
+   \brief Debug function printing to stderr basic information about a disc
 
-   \param disc - disc to display
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   \param disc - disc to print
 */
 void cdw_disc_debug_print_disc(cdw_disc_t *disc)
 {
-	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
+	cdw_assert (disc, "ERROR: disc is NULL\n");
 
-	cdw_vdm ("CURRENT DISC:    simple type = \"%s\"\n", disc->simple_type_label);
+	cdw_vdm ("CURRENT DISC:    simple type = \"%s\"\n", disc->cdio->simple_type_label);
 	cdw_vdm ("CURRENT DISC:           type = \"%s\" (%d)\n", disc->type_label, disc->type);
 	cdw_vdm ("CURRENT DISC:    state empty = \"%s\"\n", cdw_utils_get_cdw_bool_type_label(disc->state_empty));
 	cdw_vdm ("CURRENT DISC: state writable = \"%s\"\n", cdw_utils_get_cdw_bool_type_label(disc->state_writable));
@@ -1037,14 +1256,14 @@ void cdw_disc_debug_print_disc(cdw_disc_t *disc)
 
 	float coeff = 512.0; /* (n_sectors * 2048) / (1024.0 * 1024.0) = n_sectors / 512.0 */
 	cdw_vdm ("CURRENT DISC CAPACITY:  used = %lld (%.3f MB)\n", disc->capacity.sectors_used, (double) disc->capacity.sectors_used / coeff);
-	cdw_vdm ("CURRENT DISC CAPACITY: total = %lld (%.3f MB)\n", disc->capacity.sectors_total, (double) disc->capacity.sectors_total / coeff);
+	//cdw_vdm ("CURRENT DISC CAPACITY: total = %lld (%.3f MB)\n", disc->capacity.sectors_total, (double) disc->capacity.sectors_total / coeff);
 
 	if (disc->state_empty == CDW_TRUE) {
 		return;
 	}
-	cdw_vdm ("CURRENT DISC:   file system: = \"%s\"\n", disc->fs.type_label);
-	if (cdw_cdio_is_fs_iso(disc->fs.type)) {
-		cdw_vdm ("CURRENT DISC:     volume id: = \"%s\"\n", disc->fs.volume_id);
+	cdw_vdm ("CURRENT DISC:   file system: = \"%s\"\n", disc->cdio->ofs->type_label);
+	if (cdw_ofs_is_iso(disc->cdio->ofs->type)) {
+		cdw_vdm ("CURRENT DISC:     volume id: = \"%s\"\n", disc->cdio->ofs->volume_id);
 	}
 
 	if (disc->type_writable) {
@@ -1060,6 +1279,28 @@ void cdw_disc_debug_print_disc(cdw_disc_t *disc)
 
 
 
+/**
+   \brief Resolve capacity of a disc
+
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   Each disc can be characterized by total capacity (amount of sectors
+   that can be put on a disc), and used space (amount of sectors already
+   holding data / amount of burned sectors).
+
+   Knowing the two parameters is a useful thing, and showing the
+   information to the user is important.
+
+   Unfortunately:
+    - getting the correct values is tricky,
+    - getting the values is tool dependent.
+
+   The function tries to do its best, very often using number of sectors
+   from cdw_disc_t->cdw_cdio_t->cdw_ofs_t as 'used space'.
+
+   \param disc - disc for which to resolve the values
+*/
 void cdw_disc_resolve_capacities(cdw_disc_t *disc)
 {
 	if (disc->cdrecord_info.rzone_size > 0
@@ -1070,29 +1311,29 @@ void cdw_disc_resolve_capacities(cdw_disc_t *disc)
 
 		cdw_vdm ("INFO: using cdrecord info (DVD rzone size)\n");
 		disc->capacity.sectors_total = disc->cdrecord_info.rzone_size;
-		disc->capacity.sectors_used = disc->fs.n_sectors;
+		disc->capacity.sectors_used = disc->cdio->ofs->n_sectors;
 	} else if (disc->cdrecord_info.phys_size != -1) {
 		cdw_vdm ("INFO: using cdrecord info (DVD phys size)\n");
 		disc->capacity.sectors_total = disc->cdrecord_info.phys_size;
-		disc->capacity.sectors_used = disc->fs.n_sectors;
+		disc->capacity.sectors_used = disc->cdio->ofs->n_sectors;
 	} else if (disc->cdrecord_info.start_of_lead_out != -1) {
 		cdw_vdm ("INFO: using cdrecord info (CD)\n");
 		disc->capacity.sectors_total = disc->cdrecord_info.start_of_lead_out;
-		disc->capacity.sectors_used = disc->fs.n_sectors;
+		disc->capacity.sectors_used = disc->cdio->ofs->n_sectors;
 	} else if (disc->dvd_rw_mediainfo_info.end_lba != -1) {
 		cdw_vdm ("INFO: using dvd+rw-mediainfo info\n");
 		disc->capacity.sectors_total = disc->dvd_rw_mediainfo_info.end_lba;
-		disc->capacity.sectors_used = disc->fs.n_sectors;
+		disc->capacity.sectors_used = disc->cdio->ofs->n_sectors;
 	} else if (disc->xorriso_info.sectors_total != -1) {
 		cdw_vdm ("INFO: using xorriso info\n");
 		disc->capacity.sectors_total = disc->xorriso_info.sectors_total;
-		disc->capacity.sectors_used = disc->xorriso_info.sectors_used >= 0 ? disc->xorriso_info.sectors_used : disc->fs.n_sectors;
+		disc->capacity.sectors_used = disc->xorriso_info.sectors_used >= 0 ? disc->xorriso_info.sectors_used : disc->cdio->ofs->n_sectors;
 	} else if (disc->libburn_info.unformatted_size != 0) {
 		cdw_vdm ("INFO: using libburn info\n");
 		disc->capacity.sectors_total = disc->libburn_info.unformatted_size / 2048;
 		disc->capacity.sectors_used = 0;
-		if (disc->fs.n_sectors > 0) {
-			cdw_vdm ("WARNING: assuming that disc is empty while fs size != 0 (%ld)\n", disc->fs.n_sectors);
+		if (disc->cdio->ofs->n_sectors > 0) {
+			cdw_vdm ("WARNING: assuming that disc is empty while fs size != 0 (%ld)\n", disc->cdio->ofs->n_sectors);
 		}
 	} else {
 		cdw_vdm ("WARNING: no info available\n");
@@ -1104,6 +1345,19 @@ void cdw_disc_resolve_capacities(cdw_disc_t *disc)
 
 
 
+/**
+   \brief Check if disc type allows for writing
+
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   Some disc types just can't be written to...
+
+   \param disc - disc to check
+
+   \return true if type of given disc allows writing to it
+   \return false if type of given disc doesn't allow writing to it
+*/
 bool cdw_disc_is_disc_type_writable(cdw_disc_t *disc)
 {
 	if (disc->type == CDW_DISC_TYPE_UNKNOWN
@@ -1116,3 +1370,78 @@ bool cdw_disc_is_disc_type_writable(cdw_disc_t *disc)
 		return true;
 	}
 }
+
+
+
+
+
+/**
+   \brief Get number of first track on a disc
+
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   This is just a simple accessor function.
+
+   Get number of first track on a disc. cdio gets this information
+   through dedicated function (cdio_get_first_track_num()), so maybe
+   this isn't that obvious.
+
+   \param disc - disc to check
+
+   \return number of first track on a disc
+*/
+track_t cdw_disc_get_first_track(cdw_disc_t *disc)
+{
+	cdw_assert (disc, "ERROR: disc is NULL\n");
+	return disc->cdio->first_track;
+}
+
+
+
+
+
+/**
+   \brief Get number of last track on a disc
+
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   This is just a simple accessor function.
+
+   Get number of last track on a disc.
+
+   \param disc - disc to check
+
+   \return number of last track on a disc
+*/
+track_t cdw_disc_get_last_track(cdw_disc_t *disc)
+{
+	cdw_assert (disc, "ERROR: disc is NULL\n");
+	return disc->cdio->last_track;
+}
+
+
+
+
+
+/**
+   \brief Get total number of tracks on a disc
+
+   \date Function's top-level comment reviewed on 2012-02-02
+   \date Function's body reviewed on 2012-02-02
+
+   This is just a simple accessor function.
+
+   Get total number of tracks on a disc.
+
+   \param disc - disc to check
+
+   \return total number of tracks on a disc
+*/
+track_t cdw_disc_get_number_of_tracks(cdw_disc_t *disc)
+{
+	cdw_assert (disc, "ERROR: disc is NULL\n");
+	return disc->cdio->n_tracks;
+}
+
