@@ -9,6 +9,7 @@
 
 #include "main.h"
 #include "cdw_cdio.h"
+#include "cdw_ofs.h"
 
 
 /* this value is used as size of table used to hold speed dropdown labels AND
@@ -132,24 +133,12 @@ typedef struct {
 } cdw_disc_speeds_info_t;
 
 
-
-
 /* variable storing information about disc currently in drive */
 typedef struct {
-	/* simple type of disc currently in drive:
-	   CD, DVD or UNKNOWN - sometimes knowing this is enough */
-	cdw_disc_simple_type_t simple_type;
-	/* human-readable representation of disc simple type */
-	char simple_type_label[4 + 1];
+	cdw_disc_type_t type; /* precise type of disc currently in drive */
+	char type_label[CDW_DISC_TYPE_LABEL_LEN + 1]; /* human-readable representation of disc type */
 
-	/* precise type of disc currently in drive */
-	cdw_disc_type_t type;
-	/* human-readable representation of disc type */
-	char type_label[CDW_DISC_TYPE_LABEL_LEN + 1];
-
-	discmode_t cdio_disc_mode;
-
-	int n_tracks;
+	char *device_fullpath;
 
 	int state_empty;
 	int state_writable;
@@ -161,8 +150,6 @@ typedef struct {
 		long long int sectors_total;
 	} capacity;
 
-	cdw_optical_fs_t fs;
-
 	cdw_disc_speeds_info_t write_speeds;
 
 	cdw_id_t accepted_disc_modes;
@@ -173,23 +160,31 @@ typedef struct {
 	cdw_dvd_rw_mediainfo_info_t dvd_rw_mediainfo_info;
 	cdw_xorriso_t xorriso_info;
 	cdw_libburn_t libburn_info;
+
+	cdw_cdio_t *cdio;
 } cdw_disc_t;
 
 
-/* this function only serves to initialize some labels;
-   the labels don't have to be free()d, so there is no cdw_disc_clean() */
+/* module initialization and finalization
+   (well, there is no finalization, so no cdw_disc_clean()) */
 void cdw_disc_init(void);
+/* void cdw_disc_clean(void); */
+
+cdw_disc_t *cdw_disc_new(void);
+cdw_rv_t    cdw_disc_get(cdw_disc_t *disc);
+void        cdw_disc_delete(cdw_disc_t **disc);
 
-cdw_disc_t *cdw_disc_get(void);
 cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc);
-/* reset all values in given disc */
-void cdw_disc_reset(cdw_disc_t *disc);
+
+track_t cdw_disc_get_first_track(cdw_disc_t *disc);
+track_t cdw_disc_get_last_track(cdw_disc_t *disc);
+track_t cdw_disc_get_number_of_tracks(cdw_disc_t *disc);
+
+
 void cdw_disc_debug_print_states_and_types(cdw_disc_t *disc);
 const char *cdw_disc_check_if_is_readable(cdw_disc_t *disc);
 bool cdw_disc_is_disc_type_writable(cdw_disc_t *disc);
 
-/* one and only function to read metadata from optical disc to 'cdw_disc_t' data structure */
-cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc);
 
 
 /* use when you need to get writing speed index based on config.speed_range */
