@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,7 @@ static cdw_id_clabel_t drive_requests[] = {
 	{ CDROMEJECT,         "CDROMEJECT" },
 	{ CDROMCLOSETRAY,     "CDROMCLOSETRAY" },
 	{ CDROM_DRIVE_STATUS, "CDROM_DRIVE_STATUS" },
+	{ CDROM_LOCKDOOR,     "CDROM_LOCKDOOR" },
 	{ 0,                  (char *) NULL}}; /* guard */
 
 
@@ -204,6 +205,13 @@ cdw_rv_t cdw_drive_eject_tray(const char *device_fullpath)
 			cdw_vdm ("INFO: ejected drive after %d seconds\n", i);
 			return CDW_OK;
 		} else {
+#ifdef CDROM_LOCKDOOR
+			usleep(200000);
+			/* calls ioctl with 0 as a third argument -> unlock */
+			cdw_drive_execute_request(device_fullpath, CDROM_LOCKDOOR);
+#else
+			/* some error message maybe? */
+#endif
 			sleep(1);
 		}
 	}
@@ -261,6 +269,8 @@ cdw_rv_t cdw_drive_close_tray(const char *device_fullpath)
 
 
 
+/* TODO: make the function return errno, so that caller can
+   act upon an error */
 int cdw_drive_execute_request(const char *device_fullpath, long unsigned int request)
 {
 	/* yes, I know about sloppy casting of long unsigned int to
@@ -299,19 +309,25 @@ int cdw_drive_execute_request(const char *device_fullpath, long unsigned int req
 
 
 /**
-   \brief Ensure that disc is in drive and drive tray is closed
+   \brief Check availability of optical drive and disc
 
-   This function prevents program from attempting to access CD
-   when there is no disc in drive.
-   Uses ioctl() to check drive status and ensure that disc is in drive.
-   Uses dialog box to communicate with user in case of problems.
+   This function provides information if there is a drive and a disc
+   in the drive, and informs user (with dialog window messages) about
+   potential problems with finding drive or disc.
 
-   \return true if disc is in tray and tray is closed
-   \return false if drive is empty and user decided to not put disc in drive or drive doesn't answer
+   \return CDW_CANCEL if there is no optical drive available
+   \return CDW_NO if there is a drive, but there is no disc in it
+   \return CDW_OK if there is a drive, and there is a disc in it
  */
-bool cdw_drive_is_disc_in_drive(void)
+cdw_rv_t cdw_drive_disc_availability(void)
 {
 	const char *drive = cdw_drive_get_drive_fullpath();
+	if (!drive) {
+		cdw_vdm ("WARNING: no drive\n");
+		cdw_buttons_dialog(_("Warning"), _("No drive.\n"), CDW_BUTTONS_OK, CDW_COLORS_WARNING);
+		return CDW_CANCEL;
+	}
+
 	/* 2TRANS: this is title of dialog window:
 	   some problem occurred while trying to access disc */
 	const char *title = _("CD drive error");
@@ -324,7 +340,7 @@ bool cdw_drive_is_disc_in_drive(void)
 		int rv = cdw_drive_status(drive);
 		switch (rv) {
 			case CDS_DISC_OK:
-				return true;
+				return CDW_OK;
 			case CDS_NO_DISC:
 				/* 2TRANS: this is message in dialog window: no optical disc
 				   found in drive; user can select OK or CANCEL button */
@@ -375,11 +391,11 @@ bool cdw_drive_is_disc_in_drive(void)
 			if (crv == CDW_OK) {
 				; /* continue loop */
 			} else {
-				return false;
+				return CDW_NO;
 			}
 		}
 
-		sleep(3);
+		sleep(5);
 	} /* for () */
 
 	cdw_buttons_dialog(title,
@@ -389,7 +405,7 @@ bool cdw_drive_is_disc_in_drive(void)
 			   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 	cdw_main_ui_main_window_wrefresh();
 
-	return false; /* user couldn't decide what to do or hardware settings in cdw options are invalid */
+	return CDW_NO; /* user couldn't decide what to do or hardware settings in cdw options are invalid */
 }
 
 
@@ -405,7 +421,7 @@ bool cdw_drive_is_disc_in_drive(void)
 
    Values displayed in disc info view are reset after ejecting tray and
    updated with valid values after closing tray. Valid values are
-   collected by cdw_disc_current_disc_get_meta_info(true).
+   collected by cdw_disc_get().
 
    This code would be used two or three times in main(), so
    I decided to extract it to separate function.
@@ -442,11 +458,9 @@ void cdw_drive_eject_tray_with_ui_update(const char *device_fullpath)
 	cdw_processwin_wrefresh();
 	cdw_drive_eject_tray(device_fullpath);
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
-	cdw_main_ui_disc_info_view_display_data(current_disc);
-	/* update information about disc size and usage */
-	cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+	/* update information about disc size and usage:
+	   0, 0 = sectors used, sectors total */
+	cdw_main_window_volume_info_view_update(0, 0, false, global_config.follow_symlinks);
 
 	return;
 }
@@ -466,20 +480,12 @@ void cdw_drive_close_tray_with_ui_update(const char *device_fullpath)
 	usleep(500000);
 	cdw_drive_close_tray(device_fullpath);
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
-	cdw_rv_t mi = cdw_disc_get_meta_info(current_disc);
-	if (mi == CDW_OK) {
-		/* 'current_disc' still points to one and only 'current_disc'
-		   variable, which only gets updated, not reallocated, after
-		   cdw_disc_get_meta_info() call */
-		cdw_main_ui_disc_info_view_display_data(current_disc);
-		/* update information about disc size and usage */
-		cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
-	} else {
-		; /* disc info view is not updated, but user can press
-		     'R' key to try to refresh it */
-	}
+	cdw_disc_t *disc = cdw_disc_new();
+	cdw_disc_get(disc);
+	cdw_main_ui_disc_info_view_display_data(disc);
+	/* update information about disc size and usage */
+	cdw_main_window_volume_info_view_update(disc->capacity.sectors_used, disc->capacity.sectors_total, false, global_config.follow_symlinks);
+	cdw_disc_delete(&disc);
 
 	return;
 }
@@ -592,10 +598,10 @@ void cdw_drive_toggle_tray_with_ui_update2(void)
 
 	int rv = cdw_drive_toggle_tray2(drive, true);
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
+	cdw_disc_t *disc = cdw_disc_new();
+
 	if (rv == CDS_DISC_OK) {
-		cdw_rv_t crv = cdw_read_disc_info();
+		cdw_rv_t crv = cdw_disc_get(disc);
 		if (crv != CDW_OK) {
 			/* 2TRANS: this is title of dialog window */
 			cdw_buttons_dialog(_("Error"),
@@ -612,13 +618,15 @@ void cdw_drive_toggle_tray_with_ui_update2(void)
 		cdw_vdm ("ERROR: failed to toggle drive tray\n");
 	}
 
-	cdw_main_ui_disc_info_view_display_data(current_disc);
+	cdw_main_ui_disc_info_view_display_data(disc);
 	/* update information about disc size and usage */
-	cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+	cdw_main_window_volume_info_view_update(disc->capacity.sectors_used, disc->capacity.sectors_total, false, global_config.follow_symlinks);
 	/* 2TRANS: this is message in process window,
 	    "drive" is "optical drive" - a burner */
 	cdw_processwin_destroy(_("Drive state changed"), false);
 
+	cdw_disc_delete(&disc);
+
 	return;
 }
 
@@ -708,47 +716,13 @@ const char *cdw_drive_get_drive_fullpath(void)
 
 
 
-#if 0
-cdw_rv_t cdw_drive_print_help_message(WINDOW *window)
-{
-#if 0
-	/* 2TRANS: this is message displayed as a small help in configuration
-	   window. Please keep line breaks, they are used for making message
-	   layout more readable, not to preserve certain lengths of lines.
-	   Please keep "/dev/xxx" unchanged. Remember to escape quotes. */
-	cdw_textarea_print_message(window, _("INFO:\n\n\"CD Reader/Writer device\" entry should have form \'/dev/xxx\' and is obligatory - it cannot be left empty. Don't put ending slash in the path.\n\n\"SCSI device\" entry should have \'X,Y,Z\' or \'ATAPI:X,Y,Z\' form and is optional. It should be set and used only if cdrecord has troubles using \'/dev/xxx\' device - otherwise leave empty. Consult cdrecord manual and/or output of 'cdrecord dev=help' command for details of \"SCSI device\" entry."),
-				   CDW_ALIGN_LEFT);
-#else
-	char *message = cdw_string_concat
-		/* 2TRANS: this is a help message */
-		(drives.n_drives == 0 ? _("cdw can't detect any drives, you may want to enter and use custom value.\n") : "",
-		 /* here -1 is only an initial value */
-		 drives.n_drives != 0 && drives.cdio_default_drive_ind != -1 ?
-		 /* 2TRANS: this is a help message, describing content
-		    of a dropdown */
-		 _("'*' in dropdown denotes default drive.\n") : "",
-		 /* 2TRANS: this is a help message, keep \"/dev/xxx\" unchanged */
-		 _("Custom path to device should have form \"/dev/xxx\", without ending slash.\n\"SCSI device\" field is for experts only, and if filled, cdw may not work correctly.\n"),
-		 (char *) NULL);
-
-	cdw_textarea_print_message(window, message, CDW_ALIGN_LEFT);
-	wrefresh(window);
-	free(message);
-	message = (char *) NULL;
-#endif
-	return CDW_OK;
-}
-
-#endif
-
-
 
 char *cdw_drive_get_help_message(void)
 {
 #if 1
 	char *message = cdw_string_concat
 		/* 2TRANS: this is a help message */
-		(drives.n_drives == 0 ? _("cdw can't detect any drives, you may want to enter and use custom value.\n") : "",
+		(drives.n_drives == 0 ? _("cdw can't detect any drives, you may want to enter and use custom path to a device.\n") : "",
 		 /* here -1 is only an initial value */
 		 drives.n_drives != 0 && drives.cdio_default_drive_ind != -1 ?
 		 /* 2TRANS: this is a help message, describing content
