@@ -47,8 +47,7 @@ void cdw_drive_toggle_tray_with_ui_update2(void);
 void cdw_drive_reload_tray_with_ui_update(const char *device_fullpath, bool final_eject);
 
 
-
-bool cdw_drive_is_disc_in_drive(void);
+cdw_rv_t cdw_drive_disc_availability(void);
 
 CDW_DROPDOWN *cdw_drive_make_drives_dropdown(WINDOW *window, int row, int col, int width);
 //cdw_rv_t cdw_drive_print_help_message(WINDOW *window);
