@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -653,14 +653,7 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 	if (success) {
 		return CDW_OK;
 	} else {
-		if (task->tool_status.cdrecord & CDW_TOOL_STATUS_CDRECORD_NO_DISK_WRONG_DISK) {
-			/* there may be some information in "disc"
-			   variable, but it is either incomplete,
-			   or wrong; or both; */
-			cdw_disc_reset(disc);
-		}
-
-		cdw_vdm ("ERROR: failed to correctly get meta information with cdrecord; consult error informations above\n");
+		cdw_vdm ("ERROR: failed to correctly get meta information with cdrecord\n");
 		return CDW_ERROR;
 	}
 }
@@ -753,7 +746,7 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 	cdw_vdm ("speed string is \"%s\", %zd / %zd chars\n", speed_string, strlen(speed_string), speed_len_max);
 
 	char track_format[10];
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
 		/* "-data": CD-ROM Mode 1 sector format, 2048 bytes per sector */
 		/* "-xa": CD-ROM Mode 2 form 1 sector format, 2048 bytes per sector */
 		strcpy(track_format,
