@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -151,8 +151,7 @@ struct {
 "No standard Profile"
 
 some of them are supported by cdw as well. Labels of supported discs are
-captured in handle_current_disc_type()
-*/
+captured in cdw_cdrecord_handle_disc_type_{new|old}_style() */
 
 
 
@@ -163,7 +162,6 @@ extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 extern time_t time0;
 
 extern cdw_config_t global_config;
-extern cdw_disc_t current_disc;
 extern cdw_task_t *thread_task;
 
 
@@ -180,8 +178,8 @@ static int cdw_cdrecord_handle_drive_max_speed(regex_t *regex, regmatch_t *match
 static int cdw_cdrecord_handle_drive_default_speed(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_current_write_speed(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_msinfo_session_sectors(regex_t *regex, regmatch_t *matches);
-static int cdw_cdrecord_handle_current_disc_type_old_style(regex_t *regex, regmatch_t *matches);
-static int cdw_cdrecord_handle_current_disc_type_new_style(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_disc_type_old_style(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_disc_type_new_style(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_prcap_write_speed_table(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_reload_disk_and_hit_cr(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_toc_beginning(regex_t *regex, regmatch_t *matches);
@@ -240,7 +238,7 @@ static cdw_regex_t stdout_regex[] = {
 	     sure that format of information (new/old) is correctly recognized */
 	  "Current:[ ]+0x00([0-9a-fA-F]+)",
 	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
-	  cdw_cdrecord_handle_current_disc_type_old_style },
+	  cdw_cdrecord_handle_disc_type_old_style },
 
 	{ "disc type, new style", 1006,
 	  /* "Current: CD-RW" - cdrecord reports type of disc currently in drive (for both CD and DVD) */
@@ -248,7 +246,7 @@ static cdw_regex_t stdout_regex[] = {
 	  /* ([CDVRWOMDL+/\-]+) on purpose don't include '0x' to distinguish long format from short */
 	  "Current:[ ]+([CDVRWOMDL+/-]+[a-z ]*)",
 	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
-	  cdw_cdrecord_handle_current_disc_type_new_style },
+	  cdw_cdrecord_handle_disc_type_new_style },
 
 	{ "write speeds", 1007,
 	  /* "Write speed # 0:  1764 kB/s CLV/PCAV (CD  10x, DVD  1x, BD  0x)" */
@@ -616,10 +614,10 @@ int cdw_cdrecord_handle_image_writing(regex_t *regex, regmatch_t *matches)
 	if (tracknum > 1) {
 		/* sometimes cdrecord prints tracknum = 02 when burning iso
 		   image to non-virgin DVD+RW */
-		if (current_disc.type == CDW_DVD_RWP) {
+		if (thread_task->disc->type == CDW_DVD_RWP) {
 			cdw_sdm ("WARNING: cdrecord reports tracknum = %d during burning image to DVD+RW, resetting to 1\n", tracknum);
 		} else {
-			cdw_sdm ("WARNING: cdrecord reports tracknum = %d during burning image to !DVD+RW (%d), resetting to 1\n", tracknum, current_disc.type);
+			cdw_sdm ("WARNING: cdrecord reports tracknum = %d during burning image to !DVD+RW (%d), resetting to 1\n", tracknum, thread_task->disc->type);
 		}
 		tracknum = 1;
 	}
@@ -725,7 +723,7 @@ int cdw_cdrecord_handle_direct_writing(regex_t *regex, regmatch_t *matches)
 		   can't properly get multisession info for DVD+RW disc,
 		   it is impossible to burn second (or any higher) session
 		   to DVD+RW */
-		if (current_disc.type == CDW_DVD_RWP) {
+		if (thread_task->disc->type == CDW_DVD_RWP) {
 			cdw_vdm ("WARNING: cdrecord reports tracknum = %d during burning files to DVD+RW, resetting to 1\n", tracknum);
 			tracknum = 1;
 		}
@@ -775,13 +773,13 @@ int cdw_cdrecord_handle_fixating(__attribute__((unused)) regex_t *regex, __attri
    wodim: Try again with wodim blank=all.
 
    This function is called when cdw catches "Cannot blank disk". The function
-   sets current_disc.type_erasable to CDW_FALSE, so that higher level
+   sets thread_task->disc->type_erasable to CDW_FALSE, so that higher level
    code will know that blanking failed and perhaps blanking with mode 'all' is
    required.
 */
 int cdw_cdrecord_handle_blanking_not_supported(__attribute__((unused)) regex_t *regex, __attribute__((unused)) regmatch_t *matches)
 {
-	current_disc.type_erasable = CDW_FALSE;
+	thread_task->disc->type_erasable = CDW_FALSE;
 
 	return 0;
 }
@@ -827,9 +825,9 @@ int cdw_cdrecord_handle_drive_default_speed(regex_t *regex, regmatch_t *matches)
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
 		return -1;
 	} else {
-		current_disc.write_speeds.drive_default_speed = atoi(submatch);
+		thread_task->disc->write_speeds.drive_default_speed = atoi(submatch);
 		cdw_vdm ("INFO: submatch %d, drive default speed = \"%s\" -> %d\n",
-			 sub, submatch, current_disc.write_speeds.drive_default_speed);
+			 sub, submatch, thread_task->disc->write_speeds.drive_default_speed);
 		return 0;
 	}
 }
@@ -872,17 +870,17 @@ int cdw_cdrecord_handle_current_write_speed(regex_t *regex, regmatch_t *matches)
 	}
 	/* at this point simple data type should be known,
 	   so this should be safe */
-	cdw_assert (current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_CD
-		    || current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
+	cdw_assert (thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD
+		    || thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
 
 		    "ERROR: simple type is not set to any correct value\n");
 
-	if (current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+	if (thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
 		cdw_vdm ("INFO: current write speed for CD = %d\n", cd_speed);
-		current_disc.write_speeds.drive_default_speed = cd_speed;
+		thread_task->disc->write_speeds.drive_default_speed = cd_speed;
 	} else {
 		cdw_vdm ("INFO: current write speed for DVD = %d\n", dvd_speed);
-		current_disc.write_speeds.drive_default_speed = dvd_speed;
+		thread_task->disc->write_speeds.drive_default_speed = dvd_speed;
 	}
 
 	return 0;
@@ -906,9 +904,9 @@ int cdw_cdrecord_handle_drive_max_speed(regex_t *regex, regmatch_t *matches)
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
 		return -1;
 	} else {
-		current_disc.write_speeds.drive_max_speed = atoi(submatch);
+		thread_task->disc->write_speeds.drive_max_speed = atoi(submatch);
 		cdw_vdm ("INFO: submatch %d, drive max speed = \"%s\" -> %d\n",
-			 sub, submatch, current_disc.write_speeds.drive_max_speed);
+			 sub, submatch, thread_task->disc->write_speeds.drive_max_speed);
 		return 0;
 	}
 }
@@ -921,11 +919,11 @@ int cdw_cdrecord_handle_msinfo_cannot_get_next_writable(__attribute__((unused))
 {
 	/* can't get multisession info for next
 	   session - to be interpreted by higher-level code */
-	current_disc.cdrecord_info.last_sess_start = -1;
-	current_disc.cdrecord_info.next_sess_start = -1;
+	thread_task->disc->cdrecord_info.last_sess_start = -1;
+	thread_task->disc->cdrecord_info.next_sess_start = -1;
 
 	cdw_vdm ("INFO: last session start=%ld, next session start=%ld\n",
-		 current_disc.cdrecord_info.last_sess_start, current_disc.cdrecord_info.next_sess_start);
+		 thread_task->disc->cdrecord_info.last_sess_start, thread_task->disc->cdrecord_info.next_sess_start);
 
 	return 0;
 
@@ -939,11 +937,11 @@ int cdw_cdrecord_handle_msinfo_cannot_read_session_offset(__attribute__((unused)
 {
 	/* can't get multisession info about any existing session (disc
 	   is blank?) - to be interpreted by higher-level code */
-	current_disc.cdrecord_info.last_sess_start = 0;
-	current_disc.cdrecord_info.next_sess_start = 0;
+	thread_task->disc->cdrecord_info.last_sess_start = 0;
+	thread_task->disc->cdrecord_info.next_sess_start = 0;
 
 	cdw_vdm ("INFO: last session start=%ld, next session start=%ld\n",
-		 current_disc.cdrecord_info.last_sess_start, current_disc.cdrecord_info.next_sess_start);
+		 thread_task->disc->cdrecord_info.last_sess_start, thread_task->disc->cdrecord_info.next_sess_start);
 
 	return 0;
 }
@@ -965,10 +963,10 @@ int cdw_cdrecord_handle_msinfo_cannot_read_session_offset(__attribute__((unused)
    'CD-RW' can be replaced by other disc types. List of disc types
    recognized by cdrecord is on top of this file. Some of the types are
    also supported by cdw. For discs supported by cdw proper fields in
-   current_disc will be set to corresponding values. Otherwise these
+   thread_task->disc will be set to corresponding values. Otherwise these
    fields will be set to 'unknown' values.
 */
-int cdw_cdrecord_handle_current_disc_type_new_style(regex_t *regex, regmatch_t *matches)
+int cdw_cdrecord_handle_disc_type_new_style(regex_t *regex, regmatch_t *matches)
 {
 	/* format:                  1
 	   "Current:[ ]+([CDVRWOMDL+/-]+[a-z ]*)" */
@@ -984,30 +982,30 @@ int cdw_cdrecord_handle_current_disc_type_new_style(regex_t *regex, regmatch_t *
 	cdw_vdm ("INFO: format: new / cdrecord, submatch = \"%s\"\n", submatch);
 
 	if (!strcmp(submatch, "CD-RW")) {
-		current_disc.type = CDW_CD_RW;
+		thread_task->disc->type = CDW_CD_RW;
 	} else if (!strcmp(submatch, "CD-R")) {
-		current_disc.type = CDW_CD_R;
+		thread_task->disc->type = CDW_CD_R;
 	} else if (!strcmp(submatch, "CD-ROM")) {
 		/* TODO: checking of cdio_disc_mode should be
 		   moved to higher-level code */
-		if (current_disc.cdio_disc_mode == CDIO_DISC_MODE_CD_DA) {
-			current_disc.type = CDW_CD_AUDIO;
+		if (thread_task->disc->cdio->mode == CDIO_DISC_MODE_CD_DA) {
+			thread_task->disc->type = CDW_CD_AUDIO;
 		} else {
-			current_disc.type = CDW_CD_ROM;
+			thread_task->disc->type = CDW_CD_ROM;
 		}
 	} else if (!strcmp(submatch, "DVD-R")) {
-		current_disc.type = CDW_DVD_R;
+		thread_task->disc->type = CDW_DVD_R;
 	} else if (!strcmp(submatch, "DVD-R sequential recording")) {
-		current_disc.type = CDW_DVD_R_SEQ;
+		thread_task->disc->type = CDW_DVD_R_SEQ;
 	} else if (!strcmp(submatch, "DVD+R")) {
-		current_disc.type = CDW_DVD_RP;
+		thread_task->disc->type = CDW_DVD_RP;
 	} else if (!strcmp(submatch, "DVD+RW")) {
-		current_disc.type = CDW_DVD_RWP;
+		thread_task->disc->type = CDW_DVD_RWP;
 #if 0 /* cdw 0.6.0 will not support cdrecord + DVD-RW */
 	} else if (!strcmp(submatch, "DVD-RW sequential recording")) {
-		current_disc.type = CDW_DVD_RW_SEQ;
+		thread_task->disc->type = CDW_DVD_RW_SEQ;
 	} else if (!strcmp(submatch, "DVD-RW restricted overwrite")) {
-		current_disc.type = CDW_DVD_RW_RES;
+		thread_task->disc->type = CDW_DVD_RW_RES;
 #endif
 #if 0 /* no support for DVD+R DL with cdrecord/wodim; disabling this
 	 piece of code disables the support in all higher level code,
@@ -1016,12 +1014,12 @@ int cdw_cdrecord_handle_current_disc_type_new_style(regex_t *regex, regmatch_t *
 		   && global_config.support_dvd_rp_dl) {
 
 		cdw_vdm ("INFO: disc type is DVD+R/DL\n");
-		current_disc.type = CDW_DVD_RP_DL;
+		thread_task->disc->type = CDW_DVD_RP_DL;
 #endif
 	} else if (!strcmp(submatch, "DVD-ROM")) {
-		current_disc.type = CDW_DVD_ROM;
+		thread_task->disc->type = CDW_DVD_ROM;
 	} else {
-		/* current_disc.type = CDW_DISC_TYPE_UNKNOWN */
+		/* thread_task->disc->type = CDW_DISC_TYPE_UNKNOWN */
 		cdw_vdm ("WARNING: submatch = \"%s\" => disc type = unsupported\n", submatch);
 	}
 
@@ -1032,7 +1030,7 @@ int cdw_cdrecord_handle_current_disc_type_new_style(regex_t *regex, regmatch_t *
 
 
 
-int cdw_cdrecord_handle_current_disc_type_old_style(regex_t *regex, regmatch_t *matches)
+int cdw_cdrecord_handle_disc_type_old_style(regex_t *regex, regmatch_t *matches)
 {
 	/*                      1
 	  Current:[ ]+0x00([0-9a-fA-F]+) */
@@ -1051,34 +1049,34 @@ int cdw_cdrecord_handle_current_disc_type_old_style(regex_t *regex, regmatch_t *
 	cdw_assert (l == 2, "ERROR: len is %zd, expected 2\n", l);
 	long long ind = strtol(submatch, (char **) NULL, 16);
 
-	current_disc.type = wodim_profiles[ind].cdw_id;
+	thread_task->disc->type = wodim_profiles[ind].cdw_id;
 	/* the line above sets correct type of disc_type in most cases,
 	   but there are few special cases, handled below: */
-	if (current_disc.type == CDW_CD_ROM) {
+	if (thread_task->disc->type == CDW_CD_ROM) {
 		/* TODO: checking of cdio_disc_mode should be
 		   moved to higher-level code */
-		if (current_disc.cdio_disc_mode == CDIO_DISC_MODE_CD_DA) {
-			current_disc.type = CDW_CD_AUDIO;
+		if (thread_task->disc->cdio->mode == CDIO_DISC_MODE_CD_DA) {
+			thread_task->disc->type = CDW_CD_AUDIO;
 		} else {
-			current_disc.type = CDW_CD_ROM;
+			thread_task->disc->type = CDW_CD_ROM;
 		}
-	} else if (current_disc.type == CDW_DVD_RW_SEQ
-		   || current_disc.type == CDW_DVD_RW_RES) {
+	} else if (thread_task->disc->type == CDW_DVD_RW_SEQ
+		   || thread_task->disc->type == CDW_DVD_RW_RES) {
 
 		/* cdw 0.6.0 will not support cdrecord + DVD-RW */
-		current_disc.type = CDW_DISC_TYPE_UNKNOWN;
+		thread_task->disc->type = CDW_DISC_TYPE_UNKNOWN;
 
-	} else if (current_disc.type == CDW_DVD_RP_DL) {
+	} else if (thread_task->disc->type == CDW_DVD_RP_DL) {
 		/* no support for DVD+R DL with cdrecord/wodim;
 		   this line of code disables the support in all
 		   higher level code, since in no other place a
 		   value is assigned to "type" */
-		current_disc.type = CDW_DISC_TYPE_UNKNOWN;
+		thread_task->disc->type = CDW_DISC_TYPE_UNKNOWN;
 	} else {
-		; /* current_disc.type = some value from wodim_profiles[] */
+		; /* thread_task->disc->type = some value from wodim_profiles[] */
 	}
 
-	if (current_disc.type == CDW_DISC_TYPE_UNKNOWN) {
+	if (thread_task->disc->type == CDW_DISC_TYPE_UNKNOWN) {
 		cdw_vdm ("WARNING: submatch = \"%s\" => disc type = unknown\n", submatch);
 	}
 
@@ -1099,10 +1097,10 @@ int cdw_cdrecord_handle_current_disc_type_old_style(regex_t *regex, regmatch_t *
    that are written in following format: X,Y
    Capturing such information means that this disc is appendable (because
    the second address is valid). Therefore this is the only reasonable place
-   to set current_disc.type_writable = CDW_TRUE in this file.
+   to set thread_task->disc->type_writable = CDW_TRUE in this file.
    This assignment can be made here or in higher level code that examines
-   values of current_disc.cdrecord_info.last_sess_start and
-   current_disc.cdrecord_info.next_sess_start closer.
+   values of thread_task->disc->cdrecord_info.last_sess_start and
+   thread_task->disc->cdrecord_info.next_sess_start closer.
 
    If disc was blank then we would have captured "Cannot read session offset";
    if the disc was closed, we would capture "Cannot get next writable address".
@@ -1123,13 +1121,13 @@ int cdw_cdrecord_handle_msinfo_session_sectors(regex_t *regex, regmatch_t *match
 		}
 
 		if (i == 1) {
-			current_disc.cdrecord_info.last_sess_start = atol(submatch);
+			thread_task->disc->cdrecord_info.last_sess_start = atol(submatch);
 			cdw_vdm ("INFO: submatch %d, last session start = \"%s\" ->  %ld\n",
-				 i, submatch, current_disc.cdrecord_info.last_sess_start);
+				 i, submatch, thread_task->disc->cdrecord_info.last_sess_start);
 		} else if (i == 2) {
-			current_disc.cdrecord_info.next_sess_start = atol(submatch);
+			thread_task->disc->cdrecord_info.next_sess_start = atol(submatch);
 			cdw_vdm ("INFO: submatch %d, next session start = \"%s\" ->  %ld\n",
-				 i, submatch, current_disc.cdrecord_info.next_sess_start);
+				 i, submatch, thread_task->disc->cdrecord_info.next_sess_start);
 		} else {
 			;
 		}
@@ -1147,7 +1145,7 @@ int cdw_cdrecord_handle_msinfo_session_sectors(regex_t *regex, regmatch_t *match
 	   2. disc is empty, and cdrecord may print
 	      "Cannot read session offset" */
 	cdw_vdm ("INFO: last session start=%ld, next session start=%ld\n",
-		 current_disc.cdrecord_info.last_sess_start, current_disc.cdrecord_info.next_sess_start);
+		 thread_task->disc->cdrecord_info.last_sess_start, thread_task->disc->cdrecord_info.next_sess_start);
 
 	return 0;
 }
@@ -1213,7 +1211,7 @@ int cdw_cdrecord_handle_blanking_time(regex_t *regex, regmatch_t *matches)
 
    Function parses "Write speed..." lines from example output
    below to extract speed number and value and puts them in
-   current_disc.cdrecord_info.write_speeds.speeds[] table.
+   thread_task->disc->cdrecord_info.write_speeds.speeds[] table.
    Currently only CD speeds are parsed and stored in write_speeds[].
    "Number of supported..." line is not parsed.
 
@@ -1267,31 +1265,35 @@ int cdw_cdrecord_handle_prcap_write_speed_table(regex_t *regex, regmatch_t *matc
 	}
 	/* at this point simple data type should be known,
 	   so this should be safe */
-	cdw_assert (current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_CD
-		    || current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
+	cdw_assert (thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD
+		    || thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
 
 		    "ERROR: simple type is not set to any correct value\n");
 
-	if (current_disc.simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
-		cdw_vdm ("INFO: adding speed %d for CD\n", cd_speed);
-		current_disc.write_speeds.speeds[speed_index] = cd_speed;
+	if (speed_index < CDW_DISC_N_SPEEDS_MAX) {
+		if (thread_task->disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+			cdw_vdm ("INFO: adding speed %d for CD\n", cd_speed);
+			thread_task->disc->write_speeds.speeds[speed_index] = cd_speed;
+		} else {
+			cdw_vdm ("INFO: adding speed %d for DVD\n", dvd_speed);
+			thread_task->disc->write_speeds.speeds[speed_index] = dvd_speed;
+		}
 	} else {
-		cdw_vdm ("INFO: adding speed %d for DVD\n", dvd_speed);
-		current_disc.write_speeds.speeds[speed_index] = dvd_speed;
+		cdw_vdm ("WARNING: discarding speed #%d, index larger than write speeds table\n", speed_index);
 	}
 
-	for (int i = 0; i < 20; i++) {
-		cdw_vdm ("INFO: current_disc.write_speeds.speeds[%d] = %d\n", i, current_disc.write_speeds.speeds[i]);
-	}
+	cdw_vdm ("INFO: write speed #%2d = %2d\n", speed_index, thread_task->disc->write_speeds.speeds[speed_index]);
 
 	return 0;
 }
 
 
 
+
+
 int cdw_cdrecord_handle_cannot_read_toc(__attribute__((unused)) regex_t *regex, __attribute__((unused)) regmatch_t *matches)
 {
-	current_disc.cdrecord_info.has_toc = false;
+	thread_task->disc->cdrecord_info.has_toc = false;
 	return 0;
 }
 
@@ -1351,7 +1353,7 @@ int cdw_cdrecord_handle_toc_beginning(regex_t *regex, regmatch_t *matches)
 	cdw_vdm ("INFO: first/last track on disc: %d/%d\n", first_track, last_track);
 
 	/* actually this is the most interesting part */
-	current_disc.cdrecord_info.has_toc = true;
+	thread_task->disc->cdrecord_info.has_toc = true;
 
 	return 0;
 }
@@ -1465,13 +1467,13 @@ int cdw_cdrecord_handle_supported_disc_modes(regex_t *regex, regmatch_t *matches
 	while (mode != (char *) NULL) {
 		if (!strcmp(mode, "TAO")) {
 			cdw_vdm ("INFO: drive supports TAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_TAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_TAO;
 		} else if (!strcmp(mode, "DAO")) {
 			cdw_vdm ("INFO: drive supports DAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_DAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_DAO;
 		} else if (!strcmp(mode, "SAO")) {
 			cdw_vdm ("INFO: drive supports SAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_SAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_SAO;
 		} else {
 			;
 		}
@@ -1499,8 +1501,8 @@ int cdw_cdrecord_handle_phys_size(regex_t *regex, regmatch_t *matches)
 		return -1;
 	}
 
-	current_disc.cdrecord_info.phys_size = strtoll(submatch, (char **) NULL, 10);
-	cdw_vdm ("INFO: physical size: %lld\n", current_disc.cdrecord_info.phys_size);
+	thread_task->disc->cdrecord_info.phys_size = strtoll(submatch, (char **) NULL, 10);
+	cdw_vdm ("INFO: physical size: %lld\n", thread_task->disc->cdrecord_info.phys_size);
 
 	return 0;
 }
@@ -1524,8 +1526,8 @@ int cdw_cdrecord_handle_atip_start_of_lead_out(regex_t *regex, regmatch_t *match
 		return -1;
 	}
 
-	current_disc.cdrecord_info.start_of_lead_out = strtoll(submatch, (char **) NULL, 10);
-	cdw_vdm ("INFO: start of lead out: %lld\n", current_disc.cdrecord_info.start_of_lead_out);
+	thread_task->disc->cdrecord_info.start_of_lead_out = strtoll(submatch, (char **) NULL, 10);
+	cdw_vdm ("INFO: start of lead out: %lld\n", thread_task->disc->cdrecord_info.start_of_lead_out);
 
 	return 0;
 }
@@ -1550,10 +1552,10 @@ int cdw_cdrecord_handle_rzone_size(regex_t *regex, regmatch_t *matches)
 	}
 
 	long long int rzone_size = strtoll(submatch, (char **) NULL, 10);
-	if (rzone_size > current_disc.cdrecord_info.rzone_size) {
-		current_disc.cdrecord_info.rzone_size = rzone_size;
+	if (rzone_size > thread_task->disc->cdrecord_info.rzone_size) {
+		thread_task->disc->cdrecord_info.rzone_size = rzone_size;
 	}
-	cdw_vdm ("INFO: rzone size: %lld\n", current_disc.cdrecord_info.rzone_size);
+	cdw_vdm ("INFO: rzone size: %lld\n", thread_task->disc->cdrecord_info.rzone_size);
 
 	return 0;
 }
