@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -158,6 +158,8 @@ cdw_rv_t cdw_dvd_rw_mediainfo_set_disc_states(cdw_disc_t *disc)
 			cdw_vdm ("INFO: read capacity == 0, setting disc state_empty to \"yes\"\n");
 			disc->state_empty = CDW_TRUE;
 		} else {
+			/* yes, unknown; "read capacity" of burned-and-erased
+			   DVD+RW or DVD-RW Restricted disc can be misleading */
 			cdw_vdm ("INFO: read capacity != 0, setting disc state_empty to \"unknown\"\n");
 			disc->state_empty = CDW_UNKNOWN;
 		}
