@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 
 
 extern cdw_config_t global_config;
-extern cdw_disc_t current_disc;
+extern cdw_task_t *thread_task;
 
 static int cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matches);
 static int cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches);
@@ -217,7 +217,7 @@ Write Speed #4:        4.0x1385=5540KB/s
 \endverbatim
 
    All that needs to be done is to convert #0 (#1, #2...) into number,
-   use the number to index current_disc.write_speeds.speeds[] table,
+   use the number to index thread_task->disc->write_speeds.speeds[] table,
    and store writing speed in the table.
 */
 int cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches)
@@ -244,19 +244,19 @@ int cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *m
 			/* this will work in both cases if dvd+rw-mediainfo
 			   would list speed indexes from highest to lowest and
 			   from lowest to highest */
-			if (speed_i + 1 > current_disc.write_speeds.n_speeds
+			if (speed_i + 1 > thread_task->disc->write_speeds.n_speeds
 			    && speed_i < CDW_DISC_N_SPEEDS_MAX) {
 
-				current_disc.write_speeds.n_speeds = speed_i + 1; /* number of elements in table */
+				thread_task->disc->write_speeds.n_speeds = speed_i + 1; /* number of elements in table */
 			}
 			cdw_vdm ("INFO: speed_i = %d, n_speeds = %d, CDW_DISC_N_SPEEDS_MAX = %d\n",
-				 speed_i, current_disc.write_speeds.n_speeds, CDW_DISC_N_SPEEDS_MAX);
+				 speed_i, thread_task->disc->write_speeds.n_speeds, CDW_DISC_N_SPEEDS_MAX);
 		} else if (i == 3) {
 			/* speed value; write_speeds[] has limited size */
 			if (speed_i < CDW_DISC_N_SPEEDS_MAX) {
-				current_disc.write_speeds.speeds[speed_i] = atoi(submatch);
+				thread_task->disc->write_speeds.speeds[speed_i] = atoi(submatch);
 				cdw_vdm ("INFO: submatch %d: speed %d = \"%s\" -> %d\n",
-					 i, speed_i, submatch, current_disc.write_speeds.speeds[speed_i]);
+					 i, speed_i, submatch, thread_task->disc->write_speeds.speeds[speed_i]);
 			}
 		} else if (i == 4) {
 			cdw_vdm ("INFO: submatch %d: remainder of regexp = \"%s\"\n", i, submatch);
@@ -306,10 +306,10 @@ int cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *match
 		   have distinct first letters;
 		   assigning values to disc->state_* has been moved to
 		   dvd_rw_mediainfo_interface.c */
-		current_disc.dvd_rw_mediainfo_info.disc_status = submatch[0];
+		thread_task->disc->dvd_rw_mediainfo_info.disc_status = submatch[0];
 		cdw_vdm ("INFO: disc status \"%s\" saved as '%c'\n",
 			 submatch,
-			 current_disc.dvd_rw_mediainfo_info.disc_status);
+			 thread_task->disc->dvd_rw_mediainfo_info.disc_status);
 		return 0;
 	}
 }
@@ -331,7 +331,7 @@ int cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *match
 "Mounted Media:([ ]+)([a-zA-Z0-9]+),([ ]+)([\-a-zA-Z +]*)"
 \endverbatim
 
-   This function sets current_disc.type
+   This function sets thread_task->disc->type
 */
 int cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *matches)
 {
@@ -355,38 +355,38 @@ int cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *mat
 	/* convert string value into int value so that
 	   higher level code doesn't have to compare strings */
 	if (!strncmp(submatch, "DVD-R Sequential", 16)) {
-		current_disc.type = CDW_DVD_R_SEQ;
+		thread_task->disc->type = CDW_DVD_R_SEQ;
 
 	} else if (!strncmp(submatch, "DVD-R Restricted", 16)) {
-		current_disc.type = CDW_DVD_R_RES;
+		thread_task->disc->type = CDW_DVD_R_RES;
 
 	} else if (!strcmp(submatch, "DVD-R")) {
-		current_disc.type = CDW_DVD_R;
+		thread_task->disc->type = CDW_DVD_R;
 
 	} else if (!strncmp(submatch, "DVD-RW Sequential", 17)) {
-		current_disc.type = CDW_DVD_RW_SEQ;
+		thread_task->disc->type = CDW_DVD_RW_SEQ;
 
 	} else if (!strncmp(submatch, "DVD-RW Restricted", 17)) {
-		current_disc.type = CDW_DVD_RW_RES;
+		thread_task->disc->type = CDW_DVD_RW_RES;
 
 	} else if (!strncmp(submatch, "DVD-RW", 6)) {
-		current_disc.type = CDW_DVD_RW;
+		thread_task->disc->type = CDW_DVD_RW;
 
 	} else if (!strcmp(submatch, "DVD+R")) {
-		current_disc.type = CDW_DVD_RP;
+		thread_task->disc->type = CDW_DVD_RP;
 
 	} else if (!strcmp(submatch, "DVD+RW")) {
-		current_disc.type = CDW_DVD_RWP;
+		thread_task->disc->type = CDW_DVD_RWP;
 
 	} else if (!strcmp(submatch, "DVD-ROM")) {
-		current_disc.type = CDW_DVD_ROM;
+		thread_task->disc->type = CDW_DVD_ROM;
 
 	} else if (!strcmp(submatch, "DVD+R Double Layer")
 		   && global_config.support_dvd_rp_dl) {
-		current_disc.type = CDW_DVD_RP_DL;
+		thread_task->disc->type = CDW_DVD_RP_DL;
 
 	} else {
-		current_disc.type = CDW_DISC_TYPE_UNKNOWN;
+		thread_task->disc->type = CDW_DISC_TYPE_UNKNOWN;
 		cdw_vdm ("WARNING: unknown disc type, submatch was \"%s\"\n", submatch);
 	}
 
@@ -413,9 +413,9 @@ int cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch_
 	}
 
 	/* current write speed */
-	current_disc.write_speeds.drive_default_speed = atoi(submatch);
+	thread_task->disc->write_speeds.drive_default_speed = atoi(submatch);
 	cdw_vdm ("INFO: submatch %d, current write speed = \"%s\" -> %d\n",
-		 sub, submatch, current_disc.write_speeds.drive_default_speed);
+		 sub, submatch, thread_task->disc->write_speeds.drive_default_speed);
 
 	return 0;
 }
@@ -443,15 +443,15 @@ int cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *mat
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
 		return -1;
 	}
-	current_disc.dvd_rw_mediainfo_info.read_capacity = (ssize_t) atoll(submatch);
+	thread_task->disc->dvd_rw_mediainfo_info.read_capacity = (ssize_t) atoll(submatch);
 
 	/* since regexp code matched this submatch as ([0-9]+), then value
 	   '0' possibly returned by atoi() is not a sign of incorrectly
 	   formed string in its argument, but it means that the arguments
 	   translates to zero */
 	cdw_vdm ("INFO: submatch %d, read capacity = \"%s\" -> %zd\n",
-		 sub, submatch, current_disc.dvd_rw_mediainfo_info.read_capacity);
-	cdw_vdm ("INFO: read capacity = %zd\n", current_disc.dvd_rw_mediainfo_info.read_capacity);
+		 sub, submatch, thread_task->disc->dvd_rw_mediainfo_info.read_capacity);
+	cdw_vdm ("INFO: read capacity = %zd\n", thread_task->disc->dvd_rw_mediainfo_info.read_capacity);
 
 	return 0;
 }
@@ -476,9 +476,9 @@ int cdw_dvd_rw_mediainfo_pr_handle_end_lba(regex_t *regex, regmatch_t *matches)
 	}
 
 	long long int end_lba = strtoll(submatch, (char **) NULL, 10);
-	if (end_lba > current_disc.dvd_rw_mediainfo_info.end_lba) {
+	if (end_lba > thread_task->disc->dvd_rw_mediainfo_info.end_lba) {
 		cdw_vdm ("INFO: end LBA = %lld\n", end_lba);
-		current_disc.dvd_rw_mediainfo_info.end_lba = end_lba;
+		thread_task->disc->dvd_rw_mediainfo_info.end_lba = end_lba;
 	}
 
 	return 0;
@@ -504,16 +504,16 @@ int cdw_dvd_rw_mediainfo_pr_handle_track_size(regex_t *regex, regmatch_t *matche
 		return -1;
 	}
 
-	long int track_size = strtoll(submatch, (char **) NULL, 10);
-	if (current_disc.dvd_rw_mediainfo_info.all_tracks_size == -1) {
+	long long int track_size = strtoll(submatch, (char **) NULL, 10);
+	if (thread_task->disc->dvd_rw_mediainfo_info.all_tracks_size == -1) {
 		/* init */
-		current_disc.dvd_rw_mediainfo_info.all_tracks_size = 0;
+		thread_task->disc->dvd_rw_mediainfo_info.all_tracks_size = 0;
 	}
-	current_disc.dvd_rw_mediainfo_info.all_tracks_size += track_size;
+	thread_task->disc->dvd_rw_mediainfo_info.all_tracks_size += track_size;
 	/* current track may be the last one */
-	current_disc.dvd_rw_mediainfo_info.last_track_size = track_size;
-	cdw_vdm ("INFO: all tracks size = %lld\n", current_disc.dvd_rw_mediainfo_info.all_tracks_size);
-	cdw_vdm ("INFO: last track size = %lld\n", current_disc.dvd_rw_mediainfo_info.last_track_size);
+	thread_task->disc->dvd_rw_mediainfo_info.last_track_size = track_size;
+	cdw_vdm ("INFO: all tracks size = %lld\n", thread_task->disc->dvd_rw_mediainfo_info.all_tracks_size);
+	cdw_vdm ("INFO: last track size = %lld\n", thread_task->disc->dvd_rw_mediainfo_info.last_track_size);
 
 	return 0;
 }
@@ -527,7 +527,7 @@ int cdw_dvd_rw_mediainfo_pr_handle_dvd_rom_detected(__attribute__((unused)) rege
 	/* this isn't necessary, because
 	   cdw_dvd_rw_mediainfo_pr_handle_mounted_media()
 	   can recognize DVD-ROM, but it doesn't hurt either */
-	current_disc.type = CDW_DVD_ROM;
+	thread_task->disc->type = CDW_DVD_ROM;
 
 	return 0;
 }
@@ -595,7 +595,7 @@ int cdw_dvd_rw_mediainfo_pr_handle_dvd_toc(regex_t *regex, regmatch_t *matches)
 			/* track numbers reported by dvd+rw-mediainfo are
 			   1-based, we will keep it that way; this will be also
 			compatible with cdio */
-			current_disc.dvd_rw_mediainfo_info.track_address[i_track] = atol(track_start_address);
+			thread_task->disc->dvd_rw_mediainfo_info.track_address[i_track] = atol(track_start_address);
 		} else {
 			;
 		}
@@ -615,8 +615,8 @@ int cdw_dvd_rw_mediainfo_pr_handle_dvd_toc(regex_t *regex, regmatch_t *matches)
    Track Start Address:   470672*2KB
    \endverbatim
    and it is repeated for every track. We use
-   current_disc.dvd_rw_mediainfo_info.n_tracks to index
-   current_disc.dvd_rw_mediainfo_info.track_address[] and point where to store
+   thread_task->disc->dvd_rw_mediainfo_info.n_tracks to index
+   thread_task->disc->dvd_rw_mediainfo_info.track_address[] and point where to store
    successive track start addresses.
 
    The function has similar effect to handle_dvd_toc(), but it also captures
@@ -643,7 +643,7 @@ int cdw_dvd_rw_mediainfo_pr_handle_track_start_address(regex_t *regex, regmatch_
 		if (i == 2) {
 			cdw_sdm ("submatch%d: \"%s\"\n", i, submatch);
 			int tsa = atoi(submatch); /* track start address */
-			current_disc.dvd_rw_mediainfo_info.track_address[current_disc.dvd_rw_mediainfo_info.n_tracks++] = tsa;
+			thread_task->disc->dvd_rw_mediainfo_info.track_address[thread_task->disc->dvd_rw_mediainfo_info.n_tracks++] = tsa;
 		}
 	}
 
@@ -673,13 +673,13 @@ STDOUT READ TRACK INFORMATION[#4]:
  * is processed by handle_track_start_address().
  * Therefore we can use:
  *
- * (current_disc.dvd_rw_mediainfo_info.n_tracks - 1)
+ * (thread_task->disc->dvd_rw_mediainfo_info.n_tracks - 1)
  * to get
- * last session start from current_disc.dvd_rw_mediainfo_info.track_address[]
+ * last session start from thread_task->disc->dvd_rw_mediainfo_info.track_address[]
  *
- * (current_disc.dvd_rw_mediainfo_info.n_tracks)
+ * (thread_task->disc->dvd_rw_mediainfo_info.n_tracks)
  * to get
- * next (blank) session start from current_disc.dvd_rw_mediainfo_info.track_address[]
+ * next (blank) session start from thread_task->disc->dvd_rw_mediainfo_info.track_address[]
  *
  * However "next (blank) session start" is not quite the same as "Next Writable
  * Address":
@@ -717,21 +717,21 @@ int cdw_dvd_rw_mediainfo_pr_handle_next_writable_address(regex_t *regex, regmatc
 	}
 
 
-	if (current_disc.dvd_rw_mediainfo_info.n_tracks == 0) {
+	if (thread_task->disc->dvd_rw_mediainfo_info.n_tracks == 0) {
 		/* no tracks, blank disc, so both last_sess_start
 		   and next_sess_start should be 0 */
-		current_disc.last_sess_start = 0;
-		current_disc.next_sess_start = 0;
+		thread_task->disc->last_sess_start = 0;
+		thread_task->disc->next_sess_start = 0;
 	} else {
 		/* see comment at the beginning of file, starting with plus_16 */
-		current_disc.last_sess_start = current_disc.dvd_rw_mediainfo_info.track_address[current_disc.dvd_rw_mediainfo_info.n_tracks - 1] + 16;
-		current_disc.next_sess_start = nwa;
+		thread_task->disc->last_sess_start = thread_task->disc->dvd_rw_mediainfo_info.track_address[thread_task->disc->dvd_rw_mediainfo_info.n_tracks - 1] + 16;
+		thread_task->disc->next_sess_start = nwa;
 	}
 
 
-	cdw_sdm ("current_disc.last_sess_start - 16 = %ld\n", current_disc.last_sess_start - 16);
-	cdw_sdm ("current_disc.last_sess_start = %ld\n", current_disc.last_sess_start);
-	cdw_sdm ("current_disc.next_sess_start = %ld\n", current_disc.next_sess_start);
+	cdw_sdm ("thread_task->disc->last_sess_start - 16 = %ld\n", thread_task->disc->last_sess_start - 16);
+	cdw_sdm ("thread_task->disc->last_sess_start = %ld\n", thread_task->disc->last_sess_start);
+	cdw_sdm ("thread_task->disc->next_sess_start = %ld\n", thread_task->disc->next_sess_start);
 
 	return 0;
 }
