@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -179,7 +179,7 @@ static struct {
 static void cdw_ext_tools_init_labels(void);
 static void cdw_ext_tools_print_labels(void);
 static void cdw_ext_tools_find_external_tools(void);
-static int  cdw_ext_tools_copy_tool_instances(cdw_id_t t, char **instances);
+static int  cdw_ext_tools_copy_tool_instances(cdw_id_t t, cdw_dll_t *instances);
 static void cdw_ext_tools_init_cdw_tools(void);
 
 static void     cdw_ext_tools_build_cd_handlers(void);
@@ -206,6 +206,10 @@ static CDW_DROPDOWN *cdw_ext_tools_handler_dropdown_new(WINDOW *window, int row,
 
 static bool cdw_ext_tools_dvd_rw_tools_available(void);
 
+static bool cdw_ext_tools_set_iso9660_otf_tool_cd(cdw_disc_t *disc, cdw_id_label_t *tool);
+static bool cdw_ext_tools_set_iso9660_otf_tool_dvd(cdw_disc_t *disc, cdw_id_label_t *tool);
+
+
 
 /* \brief Label in tools dropdown indicating that a tool is not available */
 static char label_not_available[20];
@@ -351,13 +355,25 @@ void cdw_ext_tools_init_cdw_tools(void)
 void cdw_ext_tools_find_external_tools(void)
 {
 	for (int t = 0; t < CDW_EXT_TOOLS_N_TOOLS; t++) {
-		/* instances = table with paths to every executable named
+		/* instances = list with paths to every executable named
 		   tools[t].name, that is available in paths specified in
 		   environment variable PATH */
-		char **instances = cdw_which(cdw_ext_tools.tools[t].name);
-		if (instances == (char **) NULL) {
+		cdw_dll_t *instances = (cdw_dll_t *) NULL;
+		if (!cdw_which(cdw_ext_tools.tools[t].name, &instances)) {
 			cdw_vdm ("WARNING: can't get instances table for tool \"%s\"\n",
 				 cdw_ext_tools.tools[t].name);
+		}
+
+		if (t == CDW_TOOL_MKISOFS) {
+			cdw_which("genisoimage", &instances);
+		} else if (t == CDW_TOOL_CDRECORD) {
+			cdw_which("wodim", &instances);
+		} else {
+			;
+		}
+
+		if (!instances) {
+			/* no paths for given tool */
 			continue;
 		}
 
@@ -369,14 +385,7 @@ void cdw_ext_tools_find_external_tools(void)
 
 		/* paths to instances are strdup()ed by copy_tool_instances(),
 		   so instances are no longer needed */
-		for (int i = 0; i < cdw_ext_tools.tools[t].n_instances; i++) {
-			cdw_assert (instances[i] != (char *) NULL, "ERROR: instance #%d is NULL (n_instances = %d)\n",
-				    i, cdw_ext_tools.tools[t].n_instances);
-			free(instances[i]);
-			instances[i] = (char *) NULL;
-		}
-		free(instances);
-		instances = (char **) NULL;
+		cdw_dll_delete(&instances, free);
 
 		if (cdw_ext_tools.tools[t].n_instances == 0) {
 			cdw_ext_tools.tools[t].available = false;
@@ -402,18 +411,26 @@ void cdw_ext_tools_find_external_tools(void)
    returned by cdw_which (table of instances), and put the copies
    into cdw_ext_tools.tools[t].instances table.
 
+   The function does not check if number of instances is less than
+   CDW_EXT_TOOLS_N_INSTANCES_MAX. It is assumed that the limit is
+   not exceeded.
+
    \param t - tool id for which \p instances were collected
-   \param instances - table of fullpaths to tool \p t
+   \param instances - list of fullpaths to tool \p t
 
    \return number of instances copied from \p instances
 */
-int cdw_ext_tools_copy_tool_instances(cdw_id_t t, char **instances)
+int cdw_ext_tools_copy_tool_instances(cdw_id_t t, cdw_dll_t *instances)
 {
 	int i = 0;
-	while (instances[i] != (char *) NULL) { /* while (!guard) */
-		cdw_ext_tools.tools[t].instances[i] = strdup(instances[i]);
-		cdw_vdm ("INFO: fullpath to executable = \"%s\"\n",
-			 cdw_ext_tools.tools[t].instances[i]);
+	for (; instances; instances = instances->next) {
+		cdw_ext_tools.tools[t].instances[i] = strdup(instances->data);
+		if (!cdw_ext_tools.tools[t].instances[i]) {
+			cdw_vdm ("ERROR: failed to strdup item #%d (%s)\n", i, (char *) instances->data);
+		} else {
+			cdw_vdm ("INFO: fullpath to executable = \"%s\"\n",
+				 cdw_ext_tools.tools[t].instances[i]);
+		}
 		i++;
 	}
 
@@ -618,10 +635,11 @@ void cdw_ext_tools_print_labels(void)
 {
 	for (int t = 0; t < CDW_EXT_TOOLS_N_TOOLS; t++) {
 		for (int i = 0; i < cdw_ext_tools.tools[t].n_instances; i++) {
-			cdw_assert (cdw_ext_tools.tools[t].instances[i] != (char *) NULL,
+			cdw_assert (cdw_ext_tools.tools[t].instances[i],
 				    "ERROR: fullpath to instance #%d of tool %d is NULL\n", i, t);
 			fprintf(stderr, " + %s_labels[%d] = \"%s\"\n",
-				cdw_ext_tools.tools[t].name, i, cdw_ext_tools.config.cdrecord.items[i].label);
+				cdw_ext_tools.tools[t].name, i,
+				cdw_ext_tools.tools[t].instances[i]);
 		}
 	}
 
@@ -855,6 +873,8 @@ void cdw_ext_tools_build_dvd_handlers(void)
 	bool mkisofs = cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS);
 	cdw_id_clabel_t tools[] = {{ CDW_TOOL_GROWISOFS, mkisofs ? clabel_dvd_rw_tools_mkisofs : clabel_dvd_rw_tools },
 				   { CDW_TOOL_CDRECORD,  mkisofs ? clabel_cdrecord_mkisofs : clabel_cdrecord },
+				   /* 2TRANS: this is label in dropdown list; it allows selecting xorriso,
+				      with information that you can handle "only" DVD+R and DVD-R discs with it */
 				   { CDW_TOOL_XORRISO,   _("xorriso (only DVD+/-R)")     }};
 	int n_items = 3;
 
@@ -1164,10 +1184,12 @@ int cdw_ext_tools_get_manual_selection_ind(cdw_id_t tool_id)
 
 bool cdw_ext_tools_disc_type_supported_by_xorriso(cdw_disc_type_t disc_type)
 {
-	if (disc_type == CDW_DVD_R
-	    || disc_type == CDW_DVD_RP
-	    || disc_type == CDW_CD_R
-	    || disc_type == CDW_CD_RW) {
+	if (disc_type == CDW_CD_R
+	    || disc_type == CDW_CD_RW
+	    || disc_type == CDW_DVD_R
+	    || disc_type == CDW_DVD_R_SEQ
+	    || disc_type == CDW_DVD_R_RES
+	    || disc_type == CDW_DVD_RP) {
 
 		return true;
 	} else {
@@ -1237,18 +1259,20 @@ bool cdw_ext_tools_set_burn_tool(cdw_disc_t *disc, cdw_id_label_t *burn_tool, cd
 	/* either the function was called for burning image file to
 	   disc (and thus iso9660_tool->id == NONE), or burning
 	   tool is available and is MKISOFS */
-	bool mkisofs_ok = iso9660_tool->id == CDW_TOOL_NONE || iso9660_tool->id == CDW_TOOL_MKISOFS;
+	bool mkisofs_iso_ok = iso9660_tool->id == CDW_TOOL_NONE || iso9660_tool->id == CDW_TOOL_MKISOFS;
+	bool xorriso_iso_ok = iso9660_tool->id == CDW_TOOL_NONE || iso9660_tool->id == CDW_TOOL_XORRISO;
 
 	int impl_ind = 0;
 	if (cdw_ext_tools.config.manual_selection) {
-		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->simple_type);
-		if (id == CDW_TOOL_CDRECORD && mkisofs_ok) {
+		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->cdio->simple_type);
+		if (id == CDW_TOOL_CDRECORD && mkisofs_iso_ok) {
 			burn_tool->id = CDW_TOOL_CDRECORD;
-		} else if (id == CDW_TOOL_GROWISOFS && mkisofs_ok) {
+		} else if (id == CDW_TOOL_GROWISOFS && mkisofs_iso_ok) {
 			burn_tool->id = CDW_TOOL_GROWISOFS;
-		} else if (id == CDW_TOOL_XORRISO) {
+		} else if (id == CDW_TOOL_XORRISO && xorriso_iso_ok) {
 			burn_tool->id = CDW_TOOL_XORRISO;
 		} else {
+			cdw_vdm ("ERROR: returning false 1\n");
 			return false;
 		}
 
@@ -1259,6 +1283,7 @@ bool cdw_ext_tools_set_burn_tool(cdw_disc_t *disc, cdw_id_label_t *burn_tool, cd
 						   /* 2TRANS: this is message in dialog window */
 						   _("cdw can't handle this type of disc using xorriso, please select different tool in Configuration -> Tools."),
 						   CDW_BUTTONS_OK, CDW_COLORS_DIALOG);
+				cdw_vdm ("ERROR: returning false 2\n");
 				return false;
 			}
 		}
@@ -1266,20 +1291,23 @@ bool cdw_ext_tools_set_burn_tool(cdw_disc_t *disc, cdw_id_label_t *burn_tool, cd
 	} else {
 		if (cdw_ext_tools_disc_type_supported_by_dvd_rw_tools(disc->type)
 		    && cdw_ext_tools_is_tool_available(CDW_TOOL_GROWISOFS)
-		    && mkisofs_ok) {
+		    && mkisofs_iso_ok) {
 
 			burn_tool->id = CDW_TOOL_GROWISOFS;
+
 		} else if (cdw_ext_tools_disc_type_supported_by_cdrecord(disc->type)
 			   && cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)
-			   && mkisofs_ok) {
+			   && mkisofs_iso_ok) {
 
 			burn_tool->id = CDW_TOOL_CDRECORD;
+
 		} else if (cdw_ext_tools_disc_type_supported_by_xorriso(disc->type)
 			   && cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)
-			   && iso9660_tool->id == CDW_TOOL_XORRISO) {
+			   && xorriso_iso_ok) {
 
 			burn_tool->id = CDW_TOOL_XORRISO;
 		} else {
+			cdw_vdm ("ERROR: returning false 3\n");
 			return false;
 		}
 		impl_ind = CDW_EXT_TOOLS_SYSTEM_DEFAULT_INSTANCE_IND;
@@ -1288,6 +1316,7 @@ bool cdw_ext_tools_set_burn_tool(cdw_disc_t *disc, cdw_id_label_t *burn_tool, cd
 	burn_tool->label = cdw_ext_tools.tools[burn_tool->id].instances[impl_ind];
 	if (burn_tool->label == (char *) NULL) {
 		cdw_vdm ("ERROR: burn tool label is NULL (tool id = %lld, impl ind = %d)\n", burn_tool->id, impl_ind);
+		cdw_vdm ("ERROR: returning false 4\n");
 		return false;
 	} else {
 		return true;
@@ -1350,7 +1379,7 @@ bool cdw_ext_tools_set_iso9660_otf_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 		   mean that mkisofs is available; do necessary check for
 		   availability of mkisofs before picking it */
 
-		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->simple_type);
+		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->cdio->simple_type);
 		if ((id == CDW_TOOL_CDRECORD || id == CDW_TOOL_GROWISOFS)
 		    && cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)) {
 			tool->id = CDW_TOOL_MKISOFS;
@@ -1370,26 +1399,15 @@ bool cdw_ext_tools_set_iso9660_otf_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 
 		   use system default implementation of selected ISO9660 creator */
 
-		if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
-			if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
-			    && cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)) {
-				tool->id = CDW_TOOL_MKISOFS;
-			} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)) {
-				tool->id = CDW_TOOL_XORRISO;
-			} else {
+		if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+			bool have_tool = cdw_ext_tools_set_iso9660_otf_tool_cd(disc, tool);
+			if (!have_tool) {
 				return false;
 			}
-		} else if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
-			if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
-			    && cdw_ext_tools_is_tool_available(CDW_TOOL_GROWISOFS)) {
-				tool->id = CDW_TOOL_MKISOFS;
 
-			} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
-			    && cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)) {
-				tool->id = CDW_TOOL_MKISOFS;
-			} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)) {
-				tool->id = CDW_TOOL_XORRISO;
-			} else {
+		} else if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
+			bool have_tool = cdw_ext_tools_set_iso9660_otf_tool_dvd(disc, tool);
+			if (!have_tool) {
 				return false;
 			}
 		} else {
@@ -1411,6 +1429,167 @@ bool cdw_ext_tools_set_iso9660_otf_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 
 
 
+/**
+   \brief Select ISO9660 fs creator suitable for given CD disc
+
+   \date Function's top-level comment reviewed on 2012-02-11
+   \date Function's body reviewed on 2012-02-11
+
+   Examine type and state of a CD \p disc and select appropriate
+   tool for creating ISO9660 file system on-the-fly (otf).
+
+   The function should be called only when an automatic selection
+   of a tool should be performed, because it uses some logic and
+   guessing to get the tool.
+
+   Function sets 'id' field of \p tool.
+
+   \param disc - disc to which to examine
+   \param tool - variable, in which to set 'id' field
+
+   \return true if tool id has been set successfully
+   \return false if tool id has not been set successfully
+*/
+bool cdw_ext_tools_set_iso9660_otf_tool_cd(cdw_disc_t *disc, cdw_id_label_t *tool)
+{
+	tool->id = CDW_TOOL_NONE;
+
+	/* first check if there is already some data burned
+	   by a known tool */
+	if (disc->state_empty == CDW_FALSE) {
+		if (disc->cdio->ofs->cdw_tool == CDW_TOOL_XORRISO) {
+			if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)) {
+
+				tool->id = CDW_TOOL_XORRISO;
+				cdw_vdm ("INFO: selected xorriso as ISO9660 otf tool, based on existing fs\n\n\n");
+				return true;
+			}
+		} else if (disc->cdio->ofs->cdw_tool == CDW_TOOL_MKISOFS) {
+			if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
+			    && cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)) {
+
+				tool->id = CDW_TOOL_MKISOFS;
+				cdw_vdm ("INFO: selected mkisofs as ISO9660 otf tool, based on existing fs\n\n\n");
+				return true;
+			}
+		} else {
+			/* there is an optical file system,
+			   but created by unknown tool;
+			   fall through to generic algorithm; */
+			cdw_vdm ("INFO: disc is non-empty, but original tool is unknown; going to generic ISO9660 ofs tool selection algorithm\n\n\n");
+		}
+	} else {
+		cdw_vdm ("INFO: disc is empty, going to generic ISO9660 ofs tool selection algorithm\n\n\n");
+	}
+
+
+	/* generic algorithm; the code below works in two cases:
+	   - when disc is empty;
+	   - when disc is not empty, but no tool has been selected; */
+
+	if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
+	    && cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)) {
+
+		tool->id = CDW_TOOL_MKISOFS;
+		cdw_vdm ("INFO: selected mkisofs as ISO9660 otf tool, based on generic algorithm\n\n\n");
+		return true;
+
+	} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)) {
+
+		tool->id = CDW_TOOL_XORRISO;
+		cdw_vdm ("INFO: selected xorriso as ISO9660 otf tool, based on generic algorithm\n\n\n");
+		return true;
+
+	} else {
+		return false;
+	}
+}
+
+
+
+
+
+/**
+   \brief Select ISO9660 fs creator suitable for given DVD disc
+
+   \date Function's top-level comment reviewed on 2012-02-11
+   \date Function's body reviewed on 2012-02-11
+
+   Examine type and state of a DVD \p disc and select appropriate
+   tool for creating ISO9660 file system on-the-fly (otf).
+
+   The function should be called only when an automatic selection
+   of a tool should be performed, because it uses some logic and
+   guessing to get the tool.
+
+   Function sets 'id' field of \p tool.
+
+   \param disc - disc to which to examine
+   \param tool - variable, in which to set 'id' field
+
+   \return true if tool id has been set successfully
+   \return false if tool id has not been set successfully
+*/
+bool cdw_ext_tools_set_iso9660_otf_tool_dvd(cdw_disc_t *disc, cdw_id_label_t *tool)
+{
+	tool->id = CDW_TOOL_NONE;
+
+	/* first check if there is already some data burned
+	   by a known tool */
+	if (disc->state_empty == CDW_FALSE) {
+
+		if (disc->cdio->ofs->cdw_tool == CDW_TOOL_XORRISO
+		    && (cdw_ext_tools_disc_type_supported_by_xorriso(disc->type))) {
+			if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)) {
+
+				tool->id = CDW_TOOL_XORRISO;
+				cdw_vdm ("INFO: selected xorriso as ISO9660 otf tool, based on existing fs\n\n\n");
+				return true;
+			}
+		} else if (disc->cdio->ofs->cdw_tool == CDW_TOOL_MKISOFS) {
+			if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
+			    && (cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)
+				|| cdw_ext_tools_is_tool_available(CDW_TOOL_GROWISOFS))) {
+
+				tool->id = CDW_TOOL_MKISOFS;
+				cdw_vdm ("INFO: selected mkisofs as ISO9660 otf tool, based on existing fs\n\n\n");
+				return true;
+			}
+		} else {
+			/* there is an optical file system,
+			   but created by unknown tool;
+			   fall through to generic algorithm; */
+			cdw_vdm ("INFO: disc is non-empty, but original tool is unknown; going to generic ISO9660 ofs tool selection algorithm\n\n\n");
+		}
+	} else {
+		cdw_vdm ("INFO: disc is empty, going to generic ISO9660 ofs tool selection algorithm\n\n\n");
+	}
+
+
+	/* generic algorithm; the code below works in two cases:
+	   - when disc is empty;
+	   - when disc is not empty, but no tool has been selected; */
+	if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)
+	    && (cdw_ext_tools_is_tool_available(CDW_TOOL_GROWISOFS)
+		||cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD))) {
+
+		tool->id = CDW_TOOL_MKISOFS;
+		cdw_vdm ("INFO: selected mkisofs as ISO9660 otf tool, based on generic algorithm\n\n\n");
+		return true;
+
+	} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_XORRISO)
+		   && (cdw_ext_tools_disc_type_supported_by_xorriso(disc->type))) {
+
+		tool->id = CDW_TOOL_XORRISO;
+		cdw_vdm ("INFO: selected xorriso as ISO9660 otf tool, based on generic algorithm\n\n\n");
+		return true;
+	} else {
+		return false;
+	}
+}
+
+
+
 bool cdw_ext_tools_set_iso9660_sa_tool(cdw_id_label_t *tool)
 {
 	int impl_ind = 0;
@@ -1450,7 +1629,7 @@ bool cdw_ext_tools_set_erase_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 {
 	int impl_ind = 0;
 	if (cdw_ext_tools.config.manual_selection) {
-		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->simple_type);
+		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->cdio->simple_type);
 		if (id == CDW_TOOL_CDRECORD) {
 			tool->id = CDW_TOOL_CDRECORD;
 		} else if (id == CDW_TOOL_GROWISOFS) {
@@ -1495,7 +1674,7 @@ bool cdw_ext_tools_set_media_info_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 {
 	int impl_ind = 0;
 	if (cdw_ext_tools.config.manual_selection) {
-		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->simple_type);
+		cdw_id_t id = cdw_ext_tools_get_manual_disc_handler_family(disc->cdio->simple_type);
 		if (id == CDW_TOOL_CDRECORD) {
 			tool->id = CDW_TOOL_CDRECORD;
 		} else if (id == CDW_TOOL_GROWISOFS) {
@@ -1508,7 +1687,7 @@ bool cdw_ext_tools_set_media_info_tool(cdw_disc_t *disc, cdw_id_label_t *tool)
 		cdw_assert (cdw_ext_tools_is_tool_available(tool->id), "ERROR: tool was in dd, but is not available\n");
 		impl_ind = cdw_ext_tools_get_manual_selection_ind(tool->id);
 	} else {
-		if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD
+		if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD
 		    && cdw_ext_tools_dvd_rw_tools_available()) {
 			tool->id = CDW_TOOL_DVD_RW_MEDIAINFO;
 		} else if (cdw_ext_tools_is_tool_available(CDW_TOOL_CDRECORD)) {
