@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -193,14 +193,7 @@ char *cdw_mkisofs_create_command(cdw_task_t *task, cdw_disc_t *disc)
 
 	char *meta_information = cdw_mkisofs_create_meta_information_string(task);
 
-	bool root = false;
-	char *root_string = task->create_image.mkisofs.root_dir;
-	/* TODO: move trimming to code getting the string from options form */
-	cdw_string_rtrim(root_string);
-	root_string = cdw_string_ltrim(root_string);
-	if (root_string != (char *) NULL && !strlen(root_string)) {
-		root = true;
-	}
+	bool has_root_dir = !cdw_string_is_all_spaces(task->create_image.mkisofs.root_dir);
 
 	char *command = cdw_string_concat(
 
@@ -213,15 +206,15 @@ char *cdw_mkisofs_create_command(cdw_task_t *task, cdw_disc_t *disc)
 
 			append_multi ? multi_string : " ",
 
-			meta_information, /* already surrounded by spaces */
+			meta_information ? meta_information : "", /* already surrounded by spaces */
 
 			/* options entered by user in Configuration window
 			   in 'Other mkisofs options' field */
 			" ", task->create_image.mkisofs.other_mkisofs_options, " ",
 
-			root ? " -root \"" : "",
-			root ? task->create_image.mkisofs.root_dir : "",
-			root ? "\" " : "",
+			has_root_dir ? " -root \"" : "",
+			has_root_dir ? task->create_image.mkisofs.root_dir : "",
+			has_root_dir ? "\" " : "",
 
 			/* 'graft points' means text file with
 			   list of files to burn AND root
@@ -239,7 +232,7 @@ char *cdw_mkisofs_create_command(cdw_task_t *task, cdw_disc_t *disc)
 			/* NULL is obligatory last concat() argument */
 			(char *) NULL);
 
-	if (meta_information != (char *) NULL) {
+	if (meta_information) {
 		free(meta_information);
 		meta_information = (char *) NULL;
 	}
@@ -336,15 +329,15 @@ cdw_rv_t cdw_mkisofs_run_task(cdw_task_t *task, cdw_disc_t *disc)
 */
 ssize_t cdw_mkisofs_create_multi_string(const cdw_disc_t *disc, /* out */ char **multi_string)
 {
-	cdw_assert (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD
-		    || disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
+	cdw_assert (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD
+		    || disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD,
 
 		    "ERROR: simple disc type is neither CD nor DVD: %d\n",
-		    disc->simple_type);
+		    disc->cdio->simple_type);
 	cdw_assert (*multi_string == (char *) NULL, "ERROR: *multi_string is not NULL\n");
 
 	cdw_sdm ("INFO disc simple type is %s\n",
-		 disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD ? "CD" : "DVD");
+		 disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD ? "CD" : "DVD");
 	cdw_vdm ("INFO: last_sess_start=%ld, next_sess_start=%ld\n",
 		 disc->cdrecord_info.last_sess_start, disc->cdrecord_info.next_sess_start);
 
@@ -389,15 +382,36 @@ ssize_t cdw_mkisofs_create_multi_string(const cdw_disc_t *disc, /* out */ char *
 
 
 
+
 char *cdw_mkisofs_create_meta_information_string(cdw_task_t *task)
 {
+	/* there will always be a volume ID, either from
+	   config file, or from wizard's form field */
 	size_t volume_id = strlen(task->create_image.volume_id);
-	size_t preparer = strlen(task->create_image.preparer);
-	size_t publisher = strlen(task->create_image.publisher);
-	size_t system_id = strlen(task->create_image.system_id);
-	size_t copyright = strlen(task->create_image.copyright);
-	size_t volume_set_id = strlen(task->create_image.volume_set_id);
-	size_t abstract = strlen(task->create_image.abstract);
+
+	size_t preparer = 0;
+	size_t publisher = 0;
+	size_t system_id = 0;
+	size_t copyright = 0;
+	size_t volume_set_id = 0;
+	size_t abstract = 0;
+
+	if (task->id == CDW_TASK_BURN_FROM_FILES) {
+		/* this is to avoid adding (possibly outdated/invalid) meta
+		   information; the information may come from previous
+		   'create image' task, but now it may be invalid for
+		   purposes of 'burn from files' task;
+		   TODO: this should be fixed by adding 'meta' tab to
+		   configuration panel in write wizard */
+		;
+	} else {
+		preparer = strlen(task->create_image.preparer);
+		publisher = strlen(task->create_image.publisher);
+		system_id = strlen(task->create_image.system_id);
+		copyright = strlen(task->create_image.copyright);
+		volume_set_id = strlen(task->create_image.volume_set_id);
+		abstract = strlen(task->create_image.abstract);
+	}
 
 	char *s = cdw_string_concat(" ",
 				    volume_id ? " -V \"" : "",
