@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -580,9 +580,13 @@ int cdw_mkisofs_handle_implementation_botch(regex_t *regex, regmatch_t *matches)
 	} else {
 		thread_task->tool_status.mkisofs |= CDW_TOOL_STATUS_MKISOFS_IMPLEMENTATION_BOTCH;
 		cdw_vdm ("ERROR: thread_task->tool_status.mkisofs |= CDW_TOOL_STATUS_MKISOFS_IMPLEMENTATION_BOTCH\n");
+		/* 2TRANS: this is message printed to log file */
 		cdw_logging_write(_("ERROR: problem with creating file system on optical disc.\n"));
+		/* 2TRANS: this is message printed to log file */
 		cdw_logging_write(_("ERROR: Perhaps this information will help resolve the problem:\n"));
-		cdw_logging_write("ERROR: %s\n", submatch);
+		/* 2TRANS: this is message printed to log file;
+		   '%s' is an error message printed by mkisofs */
+		cdw_logging_write(_("ERROR: %s\n"), submatch);
 
 		return 0;
 	}
