@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include "cdw_config.h"
 #include "cdw_drive.h"
 #include "cdw_utils.h"
-
+#include "cdw_logging.h"
 
 extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];
@@ -43,7 +43,6 @@ extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 extern time_t time0;
 
 extern cdw_config_t global_config;
-extern cdw_disc_t current_disc;
 extern cdw_task_t *thread_task;
 
 
@@ -52,6 +51,9 @@ extern cdw_task_t *thread_task;
 static int cdw_xorriso_handle_atip_speeds(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_atip_supported_modes(regex_t *regex, regmatch_t *matches);
 #endif
+static int cdw_xorriso_handle_media_product(regex_t *regex, regmatch_t *matches);
+static int cdw_xorriso_handle_media_summary(regex_t *regex, regmatch_t *matches);
+
 
 /* stderr handlers */
 static int cdw_xorriso_handle_writing(regex_t *regex, regmatch_t *matches);
@@ -59,14 +61,12 @@ static int cdw_xorriso_handle_writing_cdrecord(regex_t *regex, regmatch_t *match
 static int cdw_xorriso_handle_writing_completed(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_media_current(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_media_status(regex_t *regex, regmatch_t *matches);
-static int cdw_xorriso_handle_media_product(regex_t *regex, regmatch_t *matches);
-static int cdw_xorriso_handle_media_summary(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_blanking(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_blanking_done(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_simulation_not_supported(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_exceeds_free_space(regex_t *regex, regmatch_t *matches);
 static int cdw_xorriso_handle_cannot_read_iso_image_tree(regex_t *regex, regmatch_t *matches);
-
+static int cdw_xorriso_handle_not_a_known_option(regex_t *regex, regmatch_t *matches);
 
 static int previous_percent = 0;
 
@@ -201,6 +201,12 @@ static cdw_regex_t stderr_regex[] = {
 	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
 	  cdw_xorriso_handle_cannot_read_iso_image_tree },
 
+	{ "FAILURE : Not a known option", 1010,
+	  /* Unknown option passed to xorriso */
+	  "FAILURE : Not a known option: (.+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_xorriso_handle_not_a_known_option },
+
 
 	{ "table guard", -1,
 	  /* guard: debug_id = -1 */
@@ -401,18 +407,18 @@ int cdw_xorriso_handle_media_current(regex_t *regex, regmatch_t *matches)
 	}
 
 	if (!strcmp(submatch, "CD-RW")) {
-		current_disc.type = CDW_CD_RW;
+		thread_task->disc->type = CDW_CD_RW;
 	} else if (!strcmp(submatch, "CD-R")) {
-		current_disc.type = CDW_CD_R;
+		thread_task->disc->type = CDW_CD_R;
 	} else if (!strcmp(submatch, "DVD-R")) {
-		current_disc.type = CDW_DVD_R;
+		thread_task->disc->type = CDW_DVD_R;
 	} else if (!strcmp(submatch, "DVD+R")) {
-		current_disc.type = CDW_DVD_RP;
+		thread_task->disc->type = CDW_DVD_RP;
 	} else {
-		/* current_disc.type = CDW_DISC_TYPE_UNKNOWN */
+		/* thread_task->disc->type = CDW_DISC_TYPE_UNKNOWN */
 		cdw_vdm ("WARNING: submatch = \"%s\" => disc type = unsupported\n", submatch);
 	}
-	cdw_vdm ("INFO: disc type string \"%s\" -> disc type %d\n", submatch, current_disc.type);
+	cdw_vdm ("INFO: disc type string \"%s\" -> disc type %d\n", submatch, thread_task->disc->type);
 
 	return 0;
 }
@@ -423,6 +429,18 @@ int cdw_xorriso_handle_media_current(regex_t *regex, regmatch_t *matches)
 
 int cdw_xorriso_handle_media_status(regex_t *regex, regmatch_t *matches)
 {
+	if (thread_task->id == CDW_TASK_CREATE_IMAGE) {
+		/* xorriso output for ISO files (no disc):
+		   "Drive current: -outdev '/tmp/image.iso'
+		   Media current: stdio file, overwriteable
+		   Media status : is blank"
+
+		   Obviously we shouldn't proceed with handling media
+		   status when there is no media involved (i.e. when
+		   creating standalone image file) */
+		return 0;
+	}
+
 	/* regex for capturing information about current disc status:
                             1
 	  "Media status : (.+)" */
@@ -438,20 +456,20 @@ int cdw_xorriso_handle_media_status(regex_t *regex, regmatch_t *matches)
 
 	if (strstr(submatch, "is written")) {
 		cdw_vdm ("INFO: disc is written\n");
-		current_disc.xorriso_info.is_written = CDW_TRUE;
+		thread_task->disc->xorriso_info.is_written = CDW_TRUE;
 	}
 	if (strstr(submatch, "is blank")) {
 		cdw_vdm ("INFO: disc is blank\n");
-		current_disc.xorriso_info.is_blank = CDW_TRUE;
+		thread_task->disc->xorriso_info.is_blank = CDW_TRUE;
 	}
 
 	if (strstr(submatch, "is appendable")) {
 		cdw_vdm ("INFO: disc is appendable\n");
-		current_disc.xorriso_info.is_appendable = CDW_TRUE;
+		thread_task->disc->xorriso_info.is_appendable = CDW_TRUE;
 	}
 	if (strstr(submatch, "is closed")) {
 		cdw_vdm ("INFO: disc is closed\n");
-		current_disc.xorriso_info.is_closed = CDW_TRUE;
+		thread_task->disc->xorriso_info.is_closed = CDW_TRUE;
 	}
 
 	return 0;
@@ -480,8 +498,8 @@ int cdw_xorriso_handle_atip_speeds(regex_t *regex, regmatch_t *matches)
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
 		return -1;
 	} else {
-		current_disc.write_speeds.speeds[0] = atoi(submatch);
-		cdw_vdm ("INFO: lower speed = %d\n", current_disc.write_speeds.speeds[0]);
+		thread_task->disc->write_speeds.speeds[0] = atoi(submatch);
+		cdw_vdm ("INFO: lower speed = %d\n", thread_task->disc->write_speeds.speeds[0]);
 	}
 
 	sub = 5;
@@ -490,8 +508,8 @@ int cdw_xorriso_handle_atip_speeds(regex_t *regex, regmatch_t *matches)
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
 		return -1;
 	} else {
-		current_disc.write_speeds.speeds[1] = atoi(submatch);
-		cdw_vdm ("INFO: upper speed = %d\n", current_disc.write_speeds.speeds[1]);
+		thread_task->disc->write_speeds.speeds[1] = atoi(submatch);
+		cdw_vdm ("INFO: upper speed = %d\n", thread_task->disc->write_speeds.speeds[1]);
 	}
 
 	return 0;
@@ -521,13 +539,13 @@ int cdw_xorriso_handle_atip_supported_modes(regex_t *regex, regmatch_t *matches)
 	while (mode != (char *) NULL) {
 		if (!strcmp(mode, "TAO")) {
 			cdw_vdm ("INFO: drive supports TAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_TAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_TAO;
 		} else if (!strcmp(mode, "DAO")) {
 			cdw_vdm ("INFO: drive supports DAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_DAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_DAO;
 		} else if (!strcmp(mode, "SAO")) {
 			cdw_vdm ("INFO: drive supports SAO\n");
-			current_disc.accepted_disc_modes |= CDW_DISC_MODE_SAO;
+			thread_task->disc->accepted_disc_modes |= CDW_DISC_MODE_SAO;
 		} else {
 			;
 		}
@@ -581,7 +599,7 @@ int cdw_xorriso_handle_media_product(regex_t *regex, regmatch_t *matches)
 		cdw_vdm ("INFO: %d frames\n", frames);
 	}
 
-	current_disc.xorriso_info.sectors_total = frames + 75 * seconds + 60 * 75 * minutes;
+	thread_task->disc->xorriso_info.sectors_total = frames + 75 * seconds + 60 * 75 * minutes;
 
 	return 0;
 
@@ -829,14 +847,14 @@ int cdw_xorriso_handle_media_summary(regex_t *regex, regmatch_t *matches)
 
 	const int sector_size_b = 2048;
 
-	current_disc.xorriso_info.sectors_used =
+	thread_task->disc->xorriso_info.sectors_used =
 		(data_size * data_coeff) / sector_size_b;
 
-	current_disc.xorriso_info.sectors_total =
-		current_disc.xorriso_info.sectors_used
+	thread_task->disc->xorriso_info.sectors_total =
+		thread_task->disc->xorriso_info.sectors_used
 		+ (free_area_size * free_area_coeff) / sector_size_b;
 
-	cdw_vdm ("INFO: ooo used %ldd, free %ldd\n", current_disc.xorriso_info.sectors_used, (free_area_size * free_area_coeff) / sector_size_b);
+	cdw_vdm ("INFO: ooo used %ldd, free %ldd\n", thread_task->disc->xorriso_info.sectors_used, (free_area_size * free_area_coeff) / sector_size_b);
 
 	return 0;
 }
@@ -878,3 +896,37 @@ int cdw_xorriso_handle_cannot_read_iso_image_tree(__attribute__((unused)) regex_
 	thread_task->tool_status.xorriso |= CDW_TOOL_STATUS_XORRISO_CANNOT_READ_ISO_IMAGE_TREE;
 	return 0;
 }
+
+
+
+
+
+int cdw_xorriso_handle_not_a_known_option(regex_t *regex, regmatch_t *matches)
+{
+	/* regexp for catching unknown option
+	                                    1
+	   "FAILURE : Not a known option: (.+)" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	char submatch[PIPE_BUFFER_SIZE];
+	unsigned int i = 1;
+	int len = cdw_regex_get_submatch(matches, i, stderr_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", i, len);
+		return -1;
+	}
+	char *option = cdw_string_trim(submatch);
+	if (option) {
+		/* 2TRANS: this is a message printed to log file;
+		   '%s' is a string with unknown option passed to xorriso */
+		cdw_logging_write(_("ERROR: unknown xorriso option \"%s\"\n"), option);
+		free(option);
+		option = (char *) NULL;
+	} else {
+		/* 2TRANS: this is a message printed to log file */
+		cdw_logging_write(_("ERROR: unknown xorriso option\n"));
+	}
+
+	thread_task->tool_status.xorriso |= CDW_TOOL_STATUS_XORRISO_NOT_A_KNOWN_OPTION;
+	return 0;
+}
