@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <mcheck.h>
+#include <signal.h> /* sig_atomic_t */
 
 #include <dirent.h> /* PATH_MAX */
 
@@ -77,6 +78,11 @@
 #include "cdw_sys.h"
 #include "cdw_task.h"
 #include "cdw_tabs_window.h"
+#include "cdw_ofs.h"
+
+#ifdef CDW_MAIN_WINDOW_CAN_RESIZE
+sig_atomic_t cdw_main_window_resize = 0;
+#endif
 
 extern cdw_config_t global_config;
 extern cdw_arguments_t cdw_commandline_arguments;
@@ -111,7 +117,7 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	curses_colors_init_phase1();
+	cdw_colors_init_phase1();
 	/* there is no curses_colors_clean() to register with atexit() */
 
 	crv = cdw_fs_init();
@@ -138,7 +144,7 @@ int main(int argc, char *argv[])
 		global_config.support_dvd_rp_dl = cdw_commandline_arguments.support_dvd_rp_dl;
 
 		/* now we have a config dir path, we can get colors config file */
-		curses_colors_init_phase2();
+		cdw_colors_init_phase2();
 		/* there is no curses_colors_clean() to register with atexit() */
 
 		/* cdw_cdio_drives module detects drives in user's system;
@@ -258,7 +264,23 @@ int cdw_main_loop(void)
 	WINDOW *window = cdw_main_ui_get_main_window();
 	int c = 'a';
 
-	while ((c = wgetch(window))) {
+	while ((c = wgetch(window))
+#ifdef CDW_MAIN_WINDOW_CAN_RESIZE
+	        || cdw_main_window_resize
+#endif
+		  ) {
+#ifdef CDW_MAIN_WINDOW_CAN_RESIZE
+		/* yay! this works :)
+		   I only need to add function that resizes all
+		   widgets embedded in main window :/ */
+		if (cdw_main_window_resize) {
+			cdw_main_window_resize = 0;
+			int lines, columns;
+			getmaxyx(stdscr, lines, columns);
+			resizeterm(lines, columns);
+			continue;
+		}
+#endif
 		switch (c) {
 
 		/* ****************************** *
@@ -288,9 +310,14 @@ int cdw_main_loop(void)
 		case 'R':
 			/* read disc meta information and display basic
 			   information in 'Disc info' area */
-			cdw_read_disc_info();
-			/* update information about disc size and usage */
-			cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
+			{
+				cdw_disc_t *disc = cdw_disc_new();
+				cdw_disc_get(disc);
+				cdw_main_ui_disc_info_view_display_data(disc);
+				/* update information about disc size and usage */
+				cdw_main_window_volume_info_view_update(disc->capacity.sectors_used, disc->capacity.sectors_total, true, global_config.follow_symlinks);
+				cdw_disc_delete(&disc);
+			}
 			break;
 		case 'v':
 		case 'V':
@@ -322,7 +349,10 @@ int cdw_main_loop(void)
 	        case 'D':
 			cdw_read_and_display_disc_info();
 		        break;
+		case KEY_F(9):
 		case KEY_F(10):
+			/* on some terminals F10 may be used/broken, so
+			   use F9 as well */
 			cdw_main_ui_show_license();
 			break;
 
@@ -509,14 +539,10 @@ void cdw_conditional_exit(void)
 #include "cdw_utils.h"
 #include "cdw_config.h"
 #include "cdw_debug.h"
-#include "cdw_which.h"
 #include "gettext.h"
 #include "cdw_ncurses.h"
 
 
-static WINDOW *cdw_main_test_window = (WINDOW *) NULL;
-WINDOW *cdw_main_test_subwin = (WINDOW *) NULL;
-
 
 
 int main(int argc, char **argv)
@@ -552,7 +578,6 @@ int main(int argc, char **argv)
 		cdw_fs_run_tests();
 		cdw_utils_run_tests();
 		cdw_file_run_tests();
-		cdw_which_test();
 
 		/* "make check" facility requires this message to be
 		   printed on stdout; don't localize it */
