@@ -49,5 +49,12 @@ typedef struct {
 } cdw_id_label_t;
 
 
+/* ugly, this belong to user interface */
+struct cdw_form_type;
+typedef struct cdw_form_type cdw_form_t;
+typedef int (* cdw_form_widget_function_t)(cdw_form_t *, void *);
+
+typedef void (* cdw_deallocator_t)(void *ptr);
+
 
 #endif /* H_CDW_MAIN */
