@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,8 +60,9 @@
 #include "cdw_file_manager.h"
 #include "cdw_string.h"
 #include "cdw_list_display.h"
-#include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_widgets.h"
+#include "cdw_window.h"
 #include "cdw_debug.h"
 #include "cdw_fs_browser.h"
 #include "cdw_graftpoints.h"
@@ -328,7 +329,7 @@ cdw_rv_t cdw_file_manager_handle_deleting_from_selected_files(void)
 		if (key == KEY_DC) {
 			crv = cdw_selected_files_remove_file(selected_files.display->current_item_ind);
 			if (crv == CDW_OK) {
-				crv = cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+				crv = cdw_main_window_volume_info_view_update(-1, -1, false, global_config.follow_symlinks);
 				if (crv == CDW_OK) {
 					crv = CDW_OK;
 				} else {
@@ -352,7 +353,7 @@ cdw_rv_t cdw_file_manager_handle_deleting_from_selected_files(void)
 	}
 	/* FIXME: this function does not inform user when some errors occur */
 
-	crv = cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
+	crv = cdw_main_window_volume_info_view_update(-1, -1, true, global_config.follow_symlinks);
 	if (crv == CDW_OK) {
 		return CDW_OK;
 	} else {
@@ -426,7 +427,7 @@ cdw_rv_t cdw_file_selector(void)
 		cdw_vdm ("ERROR: failed to create window for file system browser\n");
 		return CDW_ERROR;
 	}
-	cdw_ncurses_window_add_help_marker(window);
+	cdw_window_add_help(window);
 
 	/* fs browser will occupy whole window, without leaving any margins or borders */
 	cdw_fs_browser_t *fs_browser = cdw_fs_browser_new(window, file_selector.initial_fullpath);
@@ -511,7 +512,7 @@ cdw_rv_t cdw_file_selector(void)
 	if (selected_files.display->n_items > 0) {
 		/* true - upon finishing adding files recount size of
 		   selected files */
-		cdw_rv_t crv = cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
+		cdw_rv_t crv = cdw_main_window_volume_info_view_update(-1, -1, true, global_config.follow_symlinks);
 		if (crv == CDW_OK) {
 			return CDW_OK;
 		} else {
@@ -560,9 +561,7 @@ cdw_rv_t cdw_file_manager_create_selected_files_view(WINDOW *parent)
 		return CDW_ERROR;
 	} else {
 		selected_files.display->display_item = cdw_file_display_file;
-		cdw_list_display_add_return_char(selected_files.display, KEY_DC);
-		cdw_list_display_add_return_char(selected_files.display, 'q');
-		cdw_list_display_add_return_char(selected_files.display, 'Q');
+		cdw_list_display_add_return_key(selected_files.display, KEY_DC);
 		selected_files.display->display_format = CDW_LIST_DISPLAY_FORMAT_LONG;
 
 		cdw_list_display_refresh(selected_files.display);
@@ -830,8 +829,8 @@ void cdw_file_selector_display_current_path(cdw_fs_browser_t *browser, WINDOW *w
 	char *dirpath = cdw_fs_browser_get_current_printable_dirpath(browser);
 	cdw_assert (dirpath != (char *) NULL, "ERROR: failed to get current dirpath\n");
 	/* 2TRANS: this is title of file selector window */
-	cdw_ncurses_nice_box(window, _("Add files"), dirpath);
-	cdw_ncurses_window_add_help_marker(window);
+	cdw_window_add_strings(window, _("Add files"), dirpath);
+	cdw_window_add_help(window);
 	wrefresh(window);
 	free(dirpath);
 	dirpath = (char *) NULL;
@@ -939,7 +938,7 @@ cdw_rv_t cdw_file_selector_add_file_to_selected(cdw_fs_browser_t *fs_browser)
 			   size of selected files was updated in
 			   cdw_selected_files_clone_and_append_file(), and
 			   will be recounted again on file selector exit */
-			crv = cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+			crv = cdw_main_window_volume_info_view_update(-1, -1, false, global_config.follow_symlinks);
 			/* updating files info view may disrupt part of file
 			   selector window; redraw file selector */
 			cdw_list_display_refresh(fs_browser->display);
