@@ -5,11 +5,10 @@
 #include <stdbool.h>
 
 #include <dirent.h>
-#include <ncursesw/ncurses.h>
 
 #include "main.h"
 #include "cdw_dll.h"
-
+#include "cdw_ncurses.h"
 
 
 cdw_rv_t cdw_file_manager_init(void);
@@ -28,7 +27,7 @@ void cdw_file_manager_scan_selected_files(void);
 
 cdw_rv_t cdw_file_manager_create_graftpoints_file(void);
 void cdw_file_manager_delete_graftpoints_file(void);
-	
+
 cdw_dll_item_t *cdw_file_manager_get_list_of_selected_files(void);
 
 
