@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,12 @@
 #include "cdw_fs.h"
 #include "cdw_file_picker.h"
 #include "cdw_string.h"
+#include "cdw_ncurses.h"
 #include "cdw_widgets.h"
+#include "cdw_window.h"
 #include "cdw_debug.h"
 #include "cdw_sys.h"
 #include "cdw_fs_browser.h"
-#include "cdw_ncurses.h"
 #include "gettext.h"
 #include "canonicalize.h"
 
@@ -188,26 +189,26 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 	}
 	/* *** main window + fs browser window *** */
 
-	picker->main_window.win = cdw_ncurses_window_new((WINDOW *) NULL,
-							 picker->main_window.n_lines, picker->main_window.n_cols,
-							 picker->main_window.begin_y, picker->main_window.begin_x,
-							 CDW_COLORS_DIALOG, title, (char *) NULL);
+	picker->main_window.win = cdw_window_new((WINDOW *) NULL,
+						 picker->main_window.n_lines, picker->main_window.n_cols,
+						 picker->main_window.begin_y, picker->main_window.begin_x,
+						 CDW_COLORS_DIALOG, title, (char *) NULL);
 	if (picker->main_window.win == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create main window for file picker\n");
 		return CDW_ERROR;
 	}
 
-	picker->browser_window.win = cdw_ncurses_window_new(picker->main_window.win,
-							    picker->browser_window.n_lines, picker->browser_window.n_cols,
-							    picker->browser_window.begin_y, picker->browser_window.begin_x,
-							    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
+	picker->browser_window.win = cdw_window_new(picker->main_window.win,
+						    picker->browser_window.n_lines, picker->browser_window.n_cols,
+						    picker->browser_window.begin_y, picker->browser_window.begin_x,
+						    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
 	if (picker->browser_window.win == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create fs browser window for file picker\n");
 		return CDW_ERROR;
 	}
 
-	cdw_ncurses_nice_box(picker->browser_window.win, "", "");
-	cdw_ncurses_window_add_help_marker(picker->browser_window.win);
+	cdw_window_add_strings(picker->browser_window.win, "", "");
+	cdw_window_add_help(picker->browser_window.win);
 
 	picker->fullpath = strdup(initial_fullpath);
 	if (picker->fullpath == (char *) NULL) {
@@ -222,13 +223,10 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 		cdw_vdm ("ERROR: failed to create fs browser\n");
 		return CDW_ERROR;
 	}
-	cdw_list_display_add_return_char(picker->fs_browser->display, CDW_KEY_TAB);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_DOWN);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_UP);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_NPAGE);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_PPAGE);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_HOME);
-	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_END);
+	cdw_list_display_add_return_keys(picker->fs_browser->display, CDW_KEY_TAB,
+					 KEY_DOWN, KEY_UP,
+					 KEY_NPAGE, KEY_PPAGE,
+					 KEY_HOME, KEY_END, 0);
 
 	/* *** buttons *** */
 
@@ -256,7 +254,7 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 	}
 	/* content of input field buffer will be stored to provided data buffer
 	   every time user presses TAB key to move away from input line */
-	picker->input.input_line->save_data_on_tab = true;
+	picker->input.input_line->save_on_tab = true;
 
 	/* *** message on top of window *** */
 	mvwprintw(picker->main_window.win, 2, 2, message);
@@ -358,11 +356,11 @@ cdw_rv_t cdw_fs_ui_file_picker_driver(struct cdw_file_picker *picker)
 			cdw_sdm ("INFO: focus on input\n");
 
 			/* 5 attempts to enter safe string */
-			key = cdw_input_line_safe_driver(picker->input.input_line, &(picker->fullpath), 5);
+			key = cdw_input_line_driver(picker->input.input_line, &(picker->fullpath), 5);
 			/* here synchronization point is implicit:
 			   picker->fullpath and input_line->buffer are always
 			   in sync, because they are basically the same variable
-			   (second argument to cdw_input_line_safe_driver()), so now
+			   (second argument to cdw_input_line_driver()), so now
 			   we need to sync it with fs_browser; this is done in
 			   cdw_fs_ui_file_picker_driver_on_enter() */
 			if (key == CDW_KEY_TAB) {
