@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include "cdw_string.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_debug.h"
 #include "canonicalize.h"
 #include "cdw_config.h"
@@ -511,7 +512,7 @@ int cdw_fs_browser_driver(cdw_fs_browser_t *browser)
 			const char *dir = cdw_fs_get_home_dir_fullpath();
 			cdw_fs_browser_browse_into_dir(browser, dir);
 			return key; /* allow client code to update current path */
-		} else if (cdw_list_display_is_return_char(browser->display, key)) {
+		} else if (cdw_list_display_is_return_key(browser->display, key)) {
 			/* some other return key that client code
 			   has to act upon */
 			return key;
@@ -530,7 +531,7 @@ int cdw_fs_browser_driver(cdw_fs_browser_t *browser)
 		}
 #endif
 
-		if (cdw_list_display_is_return_char(browser->display, key)) {
+		if (cdw_list_display_is_return_key(browser->display, key)) {
 			break;
 		}
 	}
@@ -847,10 +848,14 @@ void cdw_fs_browser_help_window(cdw_fs_browser_t *browser)
 	int begin_y = browser->display->begin_y + (browser->display->n_lines / 2) - (n_lines / 2);
 	int begin_x = browser->display->begin_x + (browser->display->n_cols / 2) - (n_cols / 2);
 
-	WINDOW *window = cdw_ncurses_window_new((WINDOW *) NULL,
-						n_lines, n_cols,
-						begin_y, begin_x,
-						CDW_COLORS_DIALOG, "Help", "Press any key to close");
+	WINDOW *window = cdw_window_new((WINDOW *) NULL,
+					n_lines, n_cols,
+					begin_y, begin_x,
+					CDW_COLORS_DIALOG,
+					/* 2TRANS: this is title of help window */
+					_("Help"),
+					/* 2TRANS: this is helper text at the bottom of help window */
+					_("Press any key to close"));
 	if (window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create new window\n");
 		return;
