@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ extern cdw_config_t global_config;      /* variable holding cdw configuration */
   Size of the file system is counted in 2048-byte sectors.
   Size of file system may be different than size of file on a disc
 */
-long cdw_iso_image_get_n_sectors(void)
+long long cdw_iso_image_get_n_sectors(void)
 {
 	cdw_vdm ("INFO: inspecting file \"%s\"\n", global_config.iso_image_full_path);
 	int f = open(global_config.iso_image_full_path, O_RDONLY);
@@ -68,7 +68,7 @@ long cdw_iso_image_get_n_sectors(void)
 	}
 
 	long long size = iso_size;
-	long sectors = size / 2048;
+	long long sectors = size / 2048;
 
 	/* WARNING: sector size is set as constant (2048), but:
 	   - 2048 is at the moment the only sector size supported by cdw
@@ -79,7 +79,7 @@ long cdw_iso_image_get_n_sectors(void)
 	   cdrecord or caller of cdrecord? */
 
 	/* 524288 * 2048-byte sectors = 1GB */
-	cdw_vdm ("INFO: returning %ld sectors (%.2f%s)\n",
+	cdw_vdm ("INFO: returning %lld sectors (%.2f%s)\n",
 		 sectors,
 		 sectors >= 524288 ? (double) size / (1024 * 1024 * 1024) : (double) size / (1024 * 1024),
 		 sectors >= 524288 ? "GB" : "MB");
