@@ -3,7 +3,7 @@
 
 
 
-long cdw_iso_image_get_n_sectors(void);
+long long cdw_iso_image_get_n_sectors(void);
 double cdw_iso_image_get_size_mb(const char *fullpath);
 
 enum {
