@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ static cdw_id_t digest_tool_id = CDW_TOOL_NONE;
 static cdw_rv_t cdw_calculate_digest_disc(cdw_task_t *task);
 static cdw_rv_t cdw_calculate_digest_set_up_reading_disc(cdw_task_t *task);
 static void    *cdw_calculate_digest_read_and_write_disc(void *thread_task);
-static void     cdw_calculate_digest_clean_up_reading_disc(void);
+static void     cdw_calculate_digest_clean_up_reading_disc(cdw_task_t *task);
 
 static cdw_rv_t cdw_calculate_digest_file(cdw_task_t *task);
 static cdw_rv_t cdw_calculate_digest_set_up_reading_file(cdw_task_t *task, const char *file_fullpath);
@@ -136,7 +136,6 @@ cdw_rv_t cdw_calculate_digest(cdw_id_t mode, const char *file_fullpath)
 
 
 	if (mode == CDW_CALCULATE_DIGEST_MODE_DISC || mode == CDW_CALCULATE_DIGEST_MODE_DISC_FILE) {
-
 		cdw_rv_t crv = cdw_calculate_digest_disc(task);
 
 		/* when cdw is run in login console and there are some
@@ -219,7 +218,7 @@ cdw_rv_t cdw_calculate_digest_disc(cdw_task_t *task)
 
 	task->calculate_digest.read_and_write = cdw_calculate_digest_read_and_write_disc;
 	crv = cdw_digest_run_task(task);
-	cdw_calculate_digest_clean_up_reading_disc();
+	cdw_calculate_digest_clean_up_reading_disc(task);
 
 	cdw_rv_t tool_status = cdw_task_check_tool_status(task);
 
@@ -254,7 +253,11 @@ void cdw_calculate_digest_summary(cdw_id_t mode, cdw_id_t tool_id)
 
 	if (mode == CDW_CALCULATE_DIGEST_MODE_FILE || mode == CDW_CALCULATE_DIGEST_MODE_DISC) {
 		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Message"), _("Finished. Digest value is in log file ('L' hotkey in main window)."),
+		cdw_buttons_dialog(_("Message"),
+				   /* 2TRANS: this is message in dialog window;
+				      "digest" is a value returned by tools such
+				      as md5sum or sha512sum */
+				   _("Finished. Digest value is in log file ('L' hotkey in main window)."),
 				   CDW_BUTTONS_OK, CDW_COLORS_DIALOG);
 		/* 2TRANS: this is text written to log file: a header printed before
 		   a digest for data file is printed;
@@ -336,22 +339,11 @@ void cdw_calculate_digest_summary(cdw_id_t mode, cdw_id_t tool_id)
 */
 cdw_rv_t cdw_calculate_digest_set_up_reading_disc(cdw_task_t *task)
 {
-	cdw_rv_t crv = cdw_read_disc_check_preconditions();
+	cdw_assert (!task->disc, "ERROR: task->disc is set\n");
+	task->disc = cdw_disc_new();
+	cdw_rv_t crv = cdw_disc_get(task->disc);
 	if (crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to get disc\n");
-		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Error"),
-				   /* 2TRANS: this is message in dialog window:
-				      some preconditions of reading a CD were not met */
-				   _("Can't open disc."),
-				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_ERROR;
-	}
-
-	crv = cdw_cdio_open_disc();
-
-	if (crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to open cdio disc\n");
+		cdw_vdm ("ERROR: cdw_disc_get()\n");
 		task->tool_status.libcdio |= CDW_TOOL_STATUS_LIBCDIO_SETUP_FAIL;
 		cdw_vdm ("ERROR: task->tool_status |= CDW_TOOL_STATUS_LIBCDIO_SETUP_FAIL\n");
 		/* 2TRANS: this is title of dialog window */
@@ -361,17 +353,30 @@ cdw_rv_t cdw_calculate_digest_set_up_reading_disc(cdw_task_t *task)
 				      library API */
 				   _("Cannot open disc."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
+
+		return CDW_ERROR;
+	}
+
+	if (!cdw_read_disc_is_proper(task->disc)) {
+		cdw_vdm ("ERROR: can't read from the disc - preconditions failed\n");
+		/* 2TRANS: this is title of dialog window */
+		cdw_buttons_dialog(_("Error"),
+				   /* 2TRANS: this is message in dialog window:
+				      some preconditions of reading a CD were not met */
+				   _("Can't read from the disc."),
+				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
+		cdw_disc_delete(&(task->disc));
 		return CDW_ERROR;
 	}
 
 	/* we can verify only first track of type supported by cdw */
-	track_t n_tracks = cdw_cdio_get_number_of_tracks();
-	track_t first_track = cdw_cdio_get_first_track();
+	track_t n_tracks = cdw_disc_get_number_of_tracks(task->disc);
+	track_t first_track = cdw_disc_get_first_track(task->disc);
 
 	if (n_tracks > 1) {
 		cdw_vdm ("WARNING: the disc has more than one track, cdw will check only first track\n");
 	}
-	if (cdw_cdio_is_data_track(first_track)) {
+	if (cdw_cdio_is_data_track(task->disc->cdio, first_track)) {
 		return CDW_OK;
 	} else {
 		cdw_vdm ("ERROR: failed to set up reading optical disc\n");
@@ -402,9 +407,10 @@ void *cdw_calculate_digest_read_and_write_disc(void *thread_task)
 
 	//cdw_vdm ("INFO: copying %ld sectors from disc\n", task->calculate_digest.data_size_sectors);
 
-	int rv = cdw_cdio_copy_sectors_to_file(cdw_cdio_get_first_track_number(),
-					       task->calculate_digest.target_fd,
-					       task->calculate_digest.data_size_sectors);
+	int rv = cdw_cdio_copy_track(task->disc->cdio,
+				     cdw_disc_get_first_track(task->disc),
+				     task->calculate_digest.target_fd,
+				     task->calculate_digest.data_size_sectors);
 
 	/* we need to close digest input, and we need to do this right now,
 	   in read/write function, i.e. still in thread;
@@ -432,10 +438,10 @@ void *cdw_calculate_digest_read_and_write_disc(void *thread_task)
 
    Function closes disc using proper cdio call.
 */
-void cdw_calculate_digest_clean_up_reading_disc(void)
+void cdw_calculate_digest_clean_up_reading_disc(cdw_task_t *task)
 {
 	cdw_vdm ("INFO: closing data source\n");
-	cdw_cdio_close_disc();
+	cdw_disc_delete(&task->disc);
 
 	return;
 }
@@ -549,16 +555,16 @@ void *cdw_calculate_digest_read_and_write_file(void *thread_task)
 void cdw_calculate_digest_processwin_update(long long i, long long n, long long size)
 {
 	/* dividing by 4s to avoid overflows */
-	long current = ((i / 4) * (size / 4)) / (256 * 256);
-	long total = ((n / 4) * (size / 4)) / (256 * 256);
+	long long current = ((i / 4) * (size / 4)) / (256 * 256);
+	long long total = ((n / 4) * (size / 4)) / (256 * 256);
 	char current_value_string[PROCESSWIN_MAX_RTEXT_LEN + 1];
 
 	/* 2TRANS: this string will be displayed as message in progress window;
 	   first %ld is amount of data already read from CD,
 	   second %ld is total amount of data to be read */
-	snprintf(current_value_string, PROCESSWIN_MAX_RTEXT_LEN + 1, _("%ld/%ld MB"), current, total);
+	snprintf(current_value_string, PROCESSWIN_MAX_RTEXT_LEN + 1, _("%lld/%lld MB"), current, total);
 	cdw_vdm ("INFO: current value string = \"%s\"\n", current_value_string);
-	cdw_processwin_display_progress_conditional(current, total, current_value_string);
+	cdw_processwin_display_progress_conditional((long) current, (long) total, current_value_string);
 
 	return;
 }
@@ -588,10 +594,13 @@ void cdw_calculate_digest_processwin_create(cdw_id_t mode)
 
 		const char *message = (char *) NULL;
 		if (mode == CDW_CALCULATE_DIGEST_MODE_DISC) {
+			/* 2TRANS: this is message printed in progress window */
 			message = _("Verifying data on optical disc");
 		} else if (mode == CDW_CALCULATE_DIGEST_MODE_FILE) {
+			/* 2TRANS: this is message printed in progress window */
 			message = _("Verifying data in given file");
 		} else { /* CDW_CALCULATE_DIGEST_MODE_DISC_FILE */
+			/* 2TRANS: this is message printed in progress window */
 			message = _("Comparing data on disc and in given file");
 		}
 
@@ -647,7 +656,9 @@ cdw_rv_t cdw_verify(void)
 	}
 
 	if (mode == CDW_CALCULATE_DIGEST_MODE_FILE || mode == CDW_CALCULATE_DIGEST_MODE_DISC_FILE) {
+		/* 2TRANS: this is a title of file selector dialog window */
 		cdw_rv_t crv = cdw_fs_ui_file_picker(_("Select data file"),
+						     /* 2TRANS: this is additional message in file selector dialog window */
 						     _("Select existing file to verify"),
 						     &fullpath, CDW_FS_FILE, R_OK, CDW_FS_EXISTING);
 
