@@ -87,11 +87,11 @@ cdw_rv_t poor_man_cdda2wav(void)
 	cdw_rv_t op = cdw_cdio_disc_open(config.cdrw_device);
 	if (op != CDW_OK) {
 		/* 2TRANS: this is title of dialog window */
-		dialogbox(_("Error"),
+		dialogbox("Error",
 			  /* 2TRANS: this is message in dialog window: disc
 			     is in drive, but cdw cannot open it using a
 			     library API */
-			  _("Cannot open disc."), DIALOG_OK, CDW_COLORS_ERROR);
+			  "Cannot open disc.", DIALOG_OK, CDW_COLORS_ERROR);
 		return CDW_GEN_ERROR;
 	} else {
 		disc = &(current_disc.cdio_disc);
@@ -101,10 +101,10 @@ cdw_rv_t poor_man_cdda2wav(void)
 	cdw_rv_t crv = get_valid_path(
 			/* 2TRANS: this is title of dialog window in which
 			user has to enter path to valid dir */
-			_("Output directory"),
+			"Output directory",
 			/* 2TRANS: this is label in dialog window, below
 			it is input field */
-			_("Enter path to existing output directory"),
+			"Enter path to existing output directory",
 			&(config.audiodir),
 			S_IRUSR|S_IWUSR|S_IXUSR, CDW_DIR, 6); /* 6 tries */
 	if (crv != CDW_OK) { /* user failed to provide correct path when
@@ -115,7 +115,7 @@ cdw_rv_t poor_man_cdda2wav(void)
 
 	/* 2TRANS: this is main part of name of audio tracks ripped from
 	   audio cd, no more than TRACK_CORE_NAME_LEN = 30 chars */
-	char *filename = make_n_string(_("track"), TRACK_CORE_NAME_LEN);
+	char *filename = make_n_string("track", TRACK_CORE_NAME_LEN);
 	if (filename == NULL) {
 		cdw_cdio_disc_close();
 		return CDW_MEM_ERROR;
@@ -123,26 +123,26 @@ cdw_rv_t poor_man_cdda2wav(void)
 
 	/* 2TRANS: this is title of dialog window where user have to
 	   enter core of track file names */
-	crv = inputbox(_("File name"),
+	crv = inputbox("File name",
 			/* 2TRANS: this is label in dialog window,
 			   below it is an input field */
-			_("Enter core of track file names"), &filename,
+			"Enter core of track file names", &filename,
 			0, TRACK_CORE_NAME_LEN); /* mode = 0 */
 	if (crv == CDW_OK) {
 		current_task.id = TASK_RIP_AUDIO_CD;
 		/* 2TRANS: this is title of dialog window, it is
 		   related to ripping audio CD */
-		cdw_processwin_create(_("Copy audio CD"),
+		cdw_processwin_create("Copy audio CD",
 				      /* 2TRANS: this is message in dialog
 					 window: ripping audio cd is in
 					 progress */
-				      _("Copying audio CD..."), true);
+				      "Copying audio CD...", true);
 		track_t i = 0;
 		for (i = disc->first_track; i <= disc->last_track; i++) {
 			char track_info[PROCESSWIN_MAX_RTEXT_LEN];
 			/* 2TRANS: this is message in dialog window:
 			   currently track number %d is ripped */
-			sprintf(track_info, _("Copying audio track %d"), i);
+			sprintf(track_info, "Copying audio track %d", i);
 			cdw_processwin_display_text_info(1, track_info);
 
 			/* 7 is length of _%2.2d.raw; there can be
@@ -179,14 +179,14 @@ cdw_rv_t poor_man_cdda2wav(void)
 		if (success) {
 			/* 2TRANS: this is message in dialog window:
 				ripping audio cd finished successfully */
-			cdw_processwin_destroy(_("Finished successfully"), true);
+			cdw_processwin_destroy("Finished successfully", true);
 
 			return CDW_OK;
 		} else {
 			/* 2TRANS: this is message in dialog window:
 				ripping audio cd finished, but not entirely
 				successfully */
-			cdw_processwin_destroy(_("Finished. Some errors occurred."), true);
+			cdw_processwin_destroy("Finished. Some errors occurred.", true);
 
 			return CDW_NO;
 		}
@@ -220,13 +220,13 @@ int cdda2wav(void)
 	FORM *form;
 
 	sprintf(cdinfo, "%s -D%s -J -H -g", CDDA2WAV, config.cdrom);
-	cdw_processwin_create(_("Load CD info..."), (char *) NULL, false);
+	cdw_processwin_create("Load CD info...", (char *) NULL, false);
 	run_command(cdinfo);
 	cdw_processwin_destroy((char *) NULL, false);
 
 	if ((fp = fopen(config.log_full_path, "r")) == NULL) {
 		clean_before_cdw_exit();
-		fprintf(stderr, _("Cannot open log file..."));
+		fprintf(stderr, "Cannot open log file...");
 		exit(-1);
 	}
 	line = (char *) malloc(1024);
@@ -243,14 +243,14 @@ int cdda2wav(void)
 	wbkgd(cddawin, COLOR_PAIR(CDW_COLORS_DIALOG));
 	werase(cddawin);
 
-	nice_box(cddawin, _("Grab audio CD"), "");
+	nice_box(cddawin, "Grab audio CD", "");
 
-	mvwprintw(cddawin, 14, 2, "( %s )", _("Press G to start or ESC to exit"));
+	mvwprintw(cddawin, 14, 2, "( %s )", "Press G to start or ESC to exit");
 	if (end_track != 0)
-		mvwprintw(cddawin, 2, 2, "%s %d", _("Number of tracks:"), max_track);
+		mvwprintw(cddawin, 2, 2, "%s %d", "Number of tracks:", max_track);
 	else {
 		wattrset(cddawin, COLOR_PAIR(CDW_COLORS_WARNING));
-		mvwprintw(cddawin, 2, 2, _("No audio CD!"), max_track);
+		mvwprintw(cddawin, 2, 2, "No audio CD!", max_track);
 	}
 	wattrset(cddawin, COLOR_PAIR(CDW_COLORS_DIALOG));
 
@@ -266,47 +266,47 @@ int cdda2wav(void)
 
 
 	field[0] = new_field(1, 20, 1, 1, 0, 0);
-	set_field_buffer(field[0], 0, _("Start track"));
+	set_field_buffer(field[0], 0, "Start track");
 	field[1] = new_field(1, 3, 1, 22, 0, 0);
 	set_field_buffer(field[1], 0, strk);
 
 	field[2] = new_field(1, 20, 2, 1, 0, 0);
-	set_field_buffer(field[2], 0, _("End track"));
+	set_field_buffer(field[2], 0, "End track");
 	field[3] = new_field(1, 3, 2, 22, 0, 0);
 	set_field_buffer(field[3], 0, etrk);
 
 	field[4] = new_field(1, 20, 4, 1, 0, 0);
-	set_field_buffer(field[4], 0, _("Stereo"));
+	set_field_buffer(field[4], 0, "Stereo");
 	field[5] = new_field(1, 1, 4, 23, 0, 0);
 	set_field_buffer(field[5], 0, (!strncmp(stereo, "1", 1) ? "X" : "_"));
 
 	field[6] = new_field(1, 24, 4, 29, 0, 0);
-	set_field_buffer(field[6], 0, _("Bits per channel"));
+	set_field_buffer(field[6], 0, "Bits per channel");
 	field[7] = new_field(1, 3, 4, 54, 0, 0);
 	set_field_buffer(field[7], 0, bpc);
 
 	field[8] = new_field(1, 20, 5, 1, 0, 0);
-	set_field_buffer(field[8], 0, _("Echo to sound dev."));
+	set_field_buffer(field[8], 0, "Echo to sound dev.");
 	field[9] = new_field(1, 1, 5, 23, 0, 0);
 	set_field_buffer(field[9], 0, (!strncmp(echo, "1", 1) ? "X" : "_"));
 
 	field[10] = new_field(1, 20, 6, 1, 0, 0);
-	set_field_buffer(field[10], 0, _("Encode to Ogg Vorbis"));
+	set_field_buffer(field[10], 0, "Encode to Ogg Vorbis");
 	field[11] = new_field(1, 1, 6, 23, 0, 0);
 	set_field_buffer(field[11], 0, (!strncmp(encode, "1", 1) ? "X" : "_"));
 
 	field[12] = new_field(1, 24, 6, 29, 0, 0);
-	set_field_buffer(field[12], 0, _("High Quality"));
+	set_field_buffer(field[12], 0, "High Quality");
 	field[13] = new_field(1, 1, 6, 55, 0, 0);
 	set_field_buffer(field[13], 0, (!strncmp(highq, "1", 1) ? "X" : "_"));
 
 	field[14] = new_field(1, 20, 7, 1, 0, 0);
-	set_field_buffer(field[14], 0, _("Encode to Mp3"));
+	set_field_buffer(field[14], 0, "Encode to Mp3");
 	field[15] = new_field(1, 1, 7, 23, 0, 0);
 	set_field_buffer(field[15], 0, (!strncmp(lame, "1", 1) ? "X" : "_"));
 
 	field[16] = new_field(1, 24, 7, 29, 0, 0);
-	set_field_buffer(field[16], 0, _("Bitrate"));
+	set_field_buffer(field[16], 0, "Bitrate");
 	field[17] = new_field(1, 4, 7, 54, 0, 0);
 	set_field_buffer(field[17], 0, brate);
 	field[18] = NULL;
@@ -431,9 +431,9 @@ int cdda2wav(void)
 		case 'G':
 			/* mkdir(config.audiodir, S_IRWXU); */
 			if ((directory = opendir(config.audiodir)) == NULL) {
-				if (dialogbox(_("Directory not exist"), _("Audio directory not exist!\nDo you wish to create?"), 3) == CDW_OK) {
+				if (dialogbox("Directory not exist", "Audio directory not exist!\nDo you wish to create?", 3) == CDW_OK) {
 					if (mkdir(config.audiodir, S_IRWXU) != 0) {
-						dialogbox(_("Error"), _("Cannot create directory!"), 0);
+						dialogbox("Error", "Cannot create directory!", 0);
 						break;
 					}
 				} else {
@@ -473,8 +473,8 @@ int cdda2wav(void)
 						} else
 							sprintf(cdinfo, "%s -D%s -t%d -b%s -g -H - | %s -b%s -o %s/audio_%.2d.ogg -", cmd, config.cdrom, tracknum, config.bitsperchn, OGGENC,
 								config.bitrate, config.audiodir, tracknum);
-						sprintf(msg, _("Track: %d"), tracknum);
-						run_command(cdinfo, _("Grab audio CD"), 0, 0, 1, msg);
+						sprintf(msg, "Track: %d", tracknum);
+						run_command(cdinfo, "Grab audio CD", 0, 0, 1, msg);
 					}
 #endif
 				} else if (strncmp(config.lame, "1", strlen(config.lame)) == 0) {
@@ -489,8 +489,8 @@ int cdda2wav(void)
 						} else
 							sprintf(cdinfo, "%s -D%s -t%d -b%s -g -H - | %s -b%s - %s/audio_%.2d.mp3 -", cmd, config.cdrom, tracknum, config.bitsperchn, LAME,
 								config.bitrate, config.audiodir, tracknum);
-						sprintf(msg, _("Track: %d"), tracknum);
-						run_command(cdinfo, _("Grab audio CD"), 0, 0, 1, msg);
+						sprintf(msg, "Track: %d", tracknum);
+						run_command(cdinfo, "Grab audio CD", 0, 0, 1, msg);
 					}
 #endif
 				} else {
@@ -499,8 +499,8 @@ int cdda2wav(void)
 					sprintf(cmd, "%s", cdinfo);
 					for (tracknum = start_track; tracknum <= end_track; tracknum++) {
 						sprintf(cdinfo, "%s -D%s -t%d -b%s -g -H %s/audio_%.2d.wav", cmd, config.cdrom, tracknum, bpc, config.audiodir, tracknum);
-						sprintf(msg, _("Track: %d"), tracknum);
-						cdw_processwin_create(_("Grab Audio CD"), msg, false);
+						sprintf(msg, "Track: %d", tracknum);
+						cdw_processwin_create("Grab Audio CD", msg, false);
 						run_command(cdinfo);
 						cdw_processwin_destroy((char *) NULL, false);
 					}
