@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,10 +53,20 @@ cdw_rv_t cdw_create_image(void)
 		return CDW_ERROR;
 	}
 
-	cdw_rv_t crv = cdw_iso9660_wizard(task);
+	/* create graftpoints file for mkisofs; first checks if there
+	   are any files that will be used as source data */
+	cdw_rv_t crv = cdw_file_manager_create_graftpoints_file();
+	if (crv != CDW_OK) {
+		cdw_task_delete(&task);
+		return CDW_NO;
+	}
+
+
+	crv = cdw_iso9660_wizard(task);
 	if (crv != CDW_OK) {
 		cdw_vdm ("INFO: cancelled in wizard\n");
 		cdw_task_delete(&task);
+		cdw_file_manager_delete_graftpoints_file();
 		return CDW_NO;
 	}
 
@@ -68,13 +78,6 @@ cdw_rv_t cdw_create_image(void)
 		return CDW_NO;
 	}
 #endif
-	/* create graftpoints file for mkisofs; first checks if there
-	   are any files that will be used as source data */
-	crv = cdw_file_manager_create_graftpoints_file();
-	if (crv != CDW_OK) {
-		cdw_task_delete(&task);
-		return CDW_NO;
-	}
 
 	/* 2TRANS: this is title of dialog window */
 	cdw_processwin_create(_("Create image"),
