@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 
 
 
-static cdw_rv_t cdw_erase_disc_check_preconditions(cdw_disc_t *disc);
+static bool     cdw_erase_disc_is_proper(cdw_disc_t *disc);
 static cdw_rv_t cdw_erase_disc_dispatcher(cdw_task_t *task, cdw_disc_t *disc);
 static cdw_rv_t cdw_erase_disc_perform_erase(cdw_task_t *task, cdw_disc_t *disc);
 
@@ -51,11 +51,10 @@ static cdw_rv_t cdw_erase_disc_perform_erase(cdw_task_t *task, cdw_disc_t *disc)
    \return CDW_OK if command finished successfully
    \return CDW_NO if some preconditions were not met
    \return CDW_CANCEL if user cancelled blanking
-   \return CDW_GEN_ERROR if some problems occurred
+   \return CDW_ERROR if some problems occurred
 */
 cdw_rv_t cdw_erase_disc(void)
 {
-	cdw_rv_t crv = cdw_read_disc_info();
 	/* FIXME: if disc is erasable, but due to some problems with disc
 	   file system cdio module can't read full meta information from
 	   the disc, cdw will be unable to erase the disc: no meta info
@@ -63,37 +62,36 @@ cdw_rv_t cdw_erase_disc(void)
 	   implement getting some basic information needed by erase module:
 	   knowing disc type + simple disc type (+ if disc is not empty)
 	   should be enough */
-	if (crv != CDW_OK) {
-		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Error"),
-				   /* 2TRANS: this is message in dialog window */
-				   _("Cannot get metainformation from disc."),
-				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		cdw_main_ui_main_window_wrefresh();
-		return CDW_NO;
-	}
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_main_ui_disc_info_view_display_data(current_disc);
 
-	/* check if disc is in drive and if is erasable */
-	crv = cdw_erase_disc_check_preconditions(current_disc);
+	cdw_disc_t *disc = cdw_disc_new();
+	cdw_rv_t crv = cdw_disc_get(disc);
 	if (crv != CDW_OK) {
+		cdw_vdm ("WARNING: cdw_disc_get() returns %s\n", cdw_utils_get_crv_label(crv));
+		cdw_disc_delete(&disc);
 		return crv;
 	}
 
-	cdw_task_t *task = cdw_task_new(CDW_TASK_ERASE_DISC, current_disc);
+	cdw_main_ui_disc_info_view_display_data(disc);
+
+	/* check if the disc in drive is of proper type */
+	if (!cdw_erase_disc_is_proper(disc)) {
+		return CDW_NO;
+	}
+
+	cdw_task_t *task = cdw_task_new(CDW_TASK_ERASE_DISC, disc);
 	if (task == (cdw_task_t *) NULL) {
 		cdw_vdm ("ERROR: failed to create a task\n");
+		cdw_disc_delete(&disc);
 		return CDW_ERROR;
 	}
 
 	/* the function will set task->blanking.format_mode for DVD-RW */
-	cdw_rv_t decision = cdw_erase_wizard(task, current_disc);
+	cdw_rv_t decision = cdw_erase_wizard(task, disc);
 
 	cdw_rv_t retval = CDW_OK;
 	if (decision == CDW_OK) {
-		retval = cdw_erase_disc_perform_erase(task, current_disc);
+		retval = cdw_erase_disc_perform_erase(task, disc);
 	} else if (decision == CDW_CANCEL) { /* selection of blanking method was cancelled by user */
 		retval = CDW_CANCEL;
 	} else {
@@ -109,6 +107,7 @@ cdw_rv_t cdw_erase_disc(void)
 		cdw_task_save(task);
 	}
 	cdw_task_delete(&task);
+	cdw_disc_delete(&disc);
 	/* 2TRANS: this is title of dialog window, displaying
 	   messages from program blanking disc */
 	after_event(_("\"Erase\" log"), 1);
@@ -249,9 +248,9 @@ cdw_rv_t cdw_erase_disc_dispatcher(cdw_task_t *task, cdw_disc_t *disc)
 
 
 
-cdw_rv_t cdw_erase_disc_check_preconditions(cdw_disc_t *current_disc)
+bool cdw_erase_disc_is_proper(cdw_disc_t *disc)
 {
-	if (current_disc->type == CDW_DISC_TYPE_UNKNOWN) {
+	if (disc->type == CDW_DISC_TYPE_UNKNOWN) {
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
 				   /* 2TRANS: this is message in dialog window:
@@ -260,13 +259,13 @@ cdw_rv_t cdw_erase_disc_check_preconditions(cdw_disc_t *current_disc)
 				   _("Can't recognize disc in drive. Try changing tools family in Configuration -> Tools or use different disc type."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 
-		return CDW_NO;
+		return false;
 	}
 
-	if (current_disc->type_erasable == CDW_TRUE) {
-		return CDW_OK;
+	if (disc->type_erasable == CDW_TRUE) {
+		return true;
 
-	} else if (current_disc->type_erasable == CDW_FALSE) {
+	} else if (disc->type_erasable == CDW_FALSE) {
 
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
@@ -274,7 +273,7 @@ cdw_rv_t cdw_erase_disc_check_preconditions(cdw_disc_t *current_disc)
 				      tries to erase disc that is not erasable */
 				   _("Disc in drive is not erasable. Cannot erase it."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_NO;
+		return false;
 	} else { /* CDW_DISC_ERASABLE_UNKNOWN || CDW_DISC_ERASABLE_ERROR */
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Warning"),
@@ -282,8 +281,7 @@ cdw_rv_t cdw_erase_disc_check_preconditions(cdw_disc_t *current_disc)
 				      tries to erase disc that may be not erasable */
 				   _("Type of disc in drive is unknown. Cannot erase it."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_NO;
-
+		return true;
 	}
 }
 
