@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@
 #include "cdw_logging.h"
 #include "cdw_main_window.h"
 #include "cdw_read_disc_info.h"
+#include "cdw_drive.h"
+
 
 /* variable holding cdw configuration */
 extern cdw_config_t global_config;
@@ -84,14 +86,14 @@ static struct strings {
 
 
 
-static char *cdw_read_disc_get_audio_file_name(void);
-void cdw_read_disc_update_track_in_processwin(track_t t, track_t n_tracks);
-char *cdw_read_disc_get_audio_track_fullpath(track_t t, const char *filename);
-cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath);
-cdw_rv_t cdw_read_disc_read_audio_disc_prepare(char **audio_file_name);
-cdw_rv_t cdw_read_disc_read_data_disc_prepare(void);
-cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename);
-cdw_rv_t cdw_read_disc_read_data_track(const char *fullpath);
+static char    *cdw_read_disc_get_audio_file_name(void);
+static void     cdw_read_disc_update_track_in_processwin(track_t t, track_t n_tracks);
+static char    *cdw_read_disc_get_audio_track_fullpath(track_t t, const char *filename);
+static cdw_rv_t cdw_read_disc_actual_reading(cdw_disc_t *disc, track_t t, const char *fullpath);
+static cdw_rv_t cdw_read_disc_read_audio_disc_prepare(char **audio_file_name);
+static cdw_rv_t cdw_read_disc_read_data_disc_prepare(cdw_disc_t *disc);
+static cdw_rv_t cdw_read_disc_read_audio_tracks(cdw_disc_t *disc, const char *filename);
+static cdw_rv_t cdw_read_disc_read_data_track(cdw_disc_t *disc, const char *fullpath);
 
 /**
    \brief Copy tracks from optical disc to separate files
@@ -108,74 +110,76 @@ cdw_rv_t cdw_read_disc_read_data_track(const char *fullpath);
    \return CDW_OK when operation finished, most probably successfully
    \return CDW_NO when some preconditions were not met
    \return CDW_CANCEL if user cancelled operation
-   \return CDW_MEM_ERROR if malloc failed
-   \return CDW_GEN_ERROR if some other error occurred
+   \return CDW_ERROR if some other error occurred
 */
 cdw_rv_t cdw_read_disc(void)
 {
-	cdw_rv_t crv = cdw_read_disc_check_preconditions();
-	if (crv != CDW_OK) { /* conditions for reading disc are not met */
+	cdw_disc_t *disc = cdw_disc_new();
+	cdw_rv_t crv = cdw_disc_get(disc);
+	if (crv != CDW_OK) {
+		cdw_main_ui_main_window_wrefresh();
+		cdw_disc_delete(&disc);
+		return crv;
+	}
+
+	if (!cdw_read_disc_is_proper(disc)) { /* conditions for reading disc are not met */
+		cdw_disc_delete(&disc);
 		return CDW_NO;
 	}
 
 	char *audio_file_name = (char *) NULL;
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_vdm ("INFO: disc type = \"%s\"\n", current_disc->type_label);
 	int ind = 0;
 	/* this opens cdio disc for getting some more meta info and for reading */
-	crv = cdw_cdio_open_disc();
-	if (current_disc->type == CDW_CD_AUDIO) {
+	if (disc->type == CDW_CD_AUDIO) {
 		ind = 0;
 		crv = cdw_read_disc_read_audio_disc_prepare(&audio_file_name);
 	} else {
-		if (current_disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
+		if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
 			ind = 1;
 		} else {
 			ind = 2;
 		}
-		crv = cdw_read_disc_read_data_disc_prepare();
+		crv = cdw_read_disc_read_data_disc_prepare(disc);
 	}
 	/* this refreshes UI after file picker and before processwin */
 	cdw_main_ui_main_window_wrefresh();
 
 	if (crv != CDW_OK) {
-		cdw_cdio_close_disc();
-		return crv;
-	}
-
-	cdw_logging_write(_(strings_table[ind].log_string));
-	cdw_processwin_create(_(strings_table[ind].window_title),
-			      _(strings_table[ind].window_message), true);
-
-
-	if (crv != CDW_OK) {
-		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Error"),
-				   /* 2TRANS: this is message in dialog window: disc
-				      is in drive, but cdw cannot open it using a
-				      library API */
-				   _("Cannot open disc."),
-				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
+		if (crv == CDW_ERROR) {
+			/* 2TRANS: this is title of dialog window */
+			cdw_buttons_dialog(_("Error"),
+					   /* 2TRANS: this is message in dialog window: disc
+					      is in drive, but cdw cannot open it using a
+					      library API */
+					   _("Cannot open disc."),
+					   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
+		}
 		cdw_main_ui_main_window_wrefresh();
+
 		if (audio_file_name != (char *) NULL) {
 			free(audio_file_name);
 			audio_file_name = (char *) NULL;
 		}
-		cdw_cdio_close_disc();
+		cdw_disc_delete(&disc);
 		return CDW_ERROR;
 	}
 
+	cdw_logging_write(_(strings_table[ind].log_string));
+	cdw_processwin_create(_(strings_table[ind].window_title),
+			      _(_(strings_table[ind].window_message)), true);
+
+	/* TODO: cdw_processwin_set_title(_(strings_table[ind].window_title)); */
 
 	/* here we finally read tracks */
-	if (current_disc->type == CDW_CD_AUDIO) {
-		crv = cdw_read_disc_read_audio_tracks(audio_file_name);
+	if (disc->type == CDW_CD_AUDIO) {
+		crv = cdw_read_disc_read_audio_tracks(disc, audio_file_name);
 		free(audio_file_name);
 		audio_file_name = (char *) NULL;
 	} else {
-		crv = cdw_read_disc_read_data_track(global_config.iso_image_full_path);
+		crv = cdw_read_disc_read_data_track(disc, global_config.iso_image_full_path);
 	}
 
-	cdw_cdio_close_disc();
+	cdw_disc_delete(&disc);
 
 	/* when cdw is run in login console and there are some disc read
 	   errors, OS prints some error messages to the console that
@@ -235,9 +239,9 @@ cdw_rv_t cdw_read_disc_read_audio_disc_prepare(char **audio_file_name)
 
 
 
-cdw_rv_t cdw_read_disc_read_data_disc_prepare(void)
+cdw_rv_t cdw_read_disc_read_data_disc_prepare(cdw_disc_t *disc)
 {
-	track_t n_tracks = cdw_cdio_get_number_of_tracks();
+	track_t n_tracks = cdw_disc_get_number_of_tracks(disc);
 	if (n_tracks > 1) {
 		/* 2TRANS: this is title of dialog window */
 	        cdw_rv_t crv = cdw_buttons_dialog(_("Warning"),
@@ -269,6 +273,7 @@ cdw_rv_t cdw_read_disc_read_data_disc_prepare(void)
 }
 
 
+
 /**
    \brief Function that encapsulates reading tracks in loop
 
@@ -285,11 +290,11 @@ cdw_rv_t cdw_read_disc_read_data_disc_prepare(void)
    \return CDW_OK on success (all tracks were read correctly)
    \return CDW_ERROR if at least one track was not read correctly
 */
-cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename)
+cdw_rv_t cdw_read_disc_read_audio_tracks(cdw_disc_t *disc, const char *filename)
 {
-	track_t first_track = cdw_cdio_get_first_track();
-	track_t last_track = cdw_cdio_get_last_track();
-	track_t n_tracks = cdw_cdio_get_number_of_tracks();
+	track_t first_track = cdw_disc_get_first_track(disc);
+	track_t last_track = cdw_disc_get_last_track(disc);
+	track_t n_tracks = cdw_disc_get_number_of_tracks(disc);
 
 	int status = 0;
 	for (track_t t = first_track; t <= last_track; t++) {
@@ -302,7 +307,7 @@ cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename)
 			continue;
 		}
 
-		cdw_rv_t crv = cdw_read_disc_actual_reading(t, fullpath);
+		cdw_rv_t crv = cdw_read_disc_actual_reading(disc, t, fullpath);
 		if (crv != CDW_OK) {
 			status--;
 		}
@@ -335,15 +340,15 @@ cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename)
 
 
 
-cdw_rv_t cdw_read_disc_read_data_track(const char *fullpath)
+cdw_rv_t cdw_read_disc_read_data_track(cdw_disc_t *disc, const char *fullpath)
 {
-	track_t first_track = cdw_cdio_get_first_track();
-	track_t n_tracks = cdw_cdio_get_number_of_tracks();
+	track_t first_track = cdw_disc_get_first_track(disc);
+	track_t n_tracks = cdw_disc_get_number_of_tracks(disc);
 
 	cdw_vdm ("INFO: copying first track %d (out of %d tracks)\n", first_track, n_tracks);
 	cdw_read_disc_update_track_in_processwin(first_track, n_tracks);
 
-	cdw_rv_t retval = cdw_read_disc_actual_reading(first_track, fullpath);
+	cdw_rv_t retval = cdw_read_disc_actual_reading(disc, first_track, fullpath);
 
 	/* under these circumstances:
 	   - cdw works on real console
@@ -395,7 +400,7 @@ char *cdw_read_disc_get_audio_track_fullpath(track_t t, const char *filename)
 
 
 
-cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
+cdw_rv_t cdw_read_disc_actual_reading(cdw_disc_t *disc, track_t t, const char *fullpath)
 {
 	cdw_rv_t retval = CDW_OK;
 	int fd = open(fullpath, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
@@ -414,7 +419,7 @@ cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
 		   program starts to read one track from CD disc;
 		   "%d" is number of track */
 		cdw_logging_write(_("Track %d: "), t);
-		int rv = cdw_cdio_copy_sectors_to_file(t, fd, -1);
+		int rv = cdw_cdio_copy_track(disc->cdio, t, fd, -1);
 
 		/* can't write "success"/"failure" in these branches
 		   of conditions, since success of cdw_cdio_copy_sectors_to_file()
@@ -456,45 +461,30 @@ cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
 
 
 
+
 /**
-   \brief Check if process of reading tracks of CD can be performed
+   \brief Check if given disc can be read
 
-   Do all necessary checks needed before performing operation of
-   copying tracks from CD to hard disc. These checks include:
-   \li checking for disc in drive
-   \li checking if disc is not blank
-   \li checking if cdw can read tracks from disc
-   \li checking if disc type and action selected by user match (e.g. if
-       user pressed 'copy audio cd' and there is audio cd in drive
+   \date Function's top-level comment reviewed on 2012-01-21
+   \date Function's body reviewed on 2012-01-21
 
-   Task specification:
-   task.id must be set to CDW_TASK_READ_DISC or CDW_TASK_CHECK_MD5SUM.
-   task.id can't be set to CDW_TASK_CHECK_MD5SUM
+   Check if a \p disc can be read - not only if it is non-empty,
+   and of supported type, but also if reading an optical file system
+   present on the \p disc is supported by operating system.
 
-   \return CDW_OK if all necessary preconditions are met
-   \return CDW_NO if any of preconditions is not met
+   \param disc - disc to be checked
+
+   \return true if disc type and state allows it to be read
+   \return false otherwise
 */
-cdw_rv_t cdw_read_disc_check_preconditions(void)
+bool cdw_read_disc_is_proper(cdw_disc_t *disc)
 {
-	cdw_rv_t crv = cdw_read_disc_info();
-	if (crv != CDW_OK) {
-		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Error"),
-				   /* 2TRANS: this is message in dialog window */
-				   _("Cannot get metainformation from disc."),
-				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		cdw_main_ui_main_window_wrefresh();
-		return CDW_NO;
-	}
-
-	cdw_disc_t *current_disc = cdw_disc_get();
-
 	/* obviously we can't read any user data from blank disc;
 	   check if disc is empty _before_ calling cdw_disc_supported_for_ripping(),
 	   because the function will return false for empty disc, even "rippable";
 	   TODO: on the other hand this code informs that "disc is empty" even for
 	   non-rippable discs, which should also be avoided */
-	if (current_disc->state_empty == CDW_TRUE) {
+	if (disc->state_empty == CDW_TRUE) {
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Information"),
 				   /* 2TRANS: this message in dialog window */
@@ -502,32 +492,32 @@ cdw_rv_t cdw_read_disc_check_preconditions(void)
 				   CDW_BUTTONS_OK, CDW_COLORS_DIALOG);
 		cdw_main_ui_main_window_wrefresh();
 
-		return CDW_NO;
-	} else if (current_disc->state_empty == CDW_UNKNOWN) {
+		return false;
+	} else if (disc->state_empty == CDW_UNKNOWN) {
 		/* 2TRANS: this is title of dialog window */
-		crv = cdw_buttons_dialog(_("Information"),
-					 /* 2TRANS: this message in dialog window */
-					 _("Disc in drive may be empty and reading from disc may fail. Continue?"),
-					 CDW_BUTTONS_OK_CANCEL, CDW_COLORS_DIALOG);
+		cdw_rv_t crv = cdw_buttons_dialog(_("Information"),
+						  /* 2TRANS: this message in dialog window */
+						  _("Disc in drive may be empty and reading from disc may fail. Continue?"),
+						  CDW_BUTTONS_OK_CANCEL, CDW_COLORS_DIALOG);
 		cdw_main_ui_main_window_wrefresh();
 		if (crv != CDW_OK) {
-			return CDW_NO;
+			return false;
 		}
 	} else {
 		;
 	}
 
-	const char *message = cdw_disc_check_if_is_readable(current_disc);
-	if (message != (char *) NULL) {
+	const char *message = cdw_disc_check_if_is_readable(disc);
+	if (message) {
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Information"),
 				   message,
 				   CDW_BUTTONS_OK, CDW_COLORS_DIALOG);
 		cdw_main_ui_main_window_wrefresh();
-		return CDW_NO;
+		return false;
 	}
 
-	return CDW_OK;
+	return true;
 }
 
 
@@ -542,7 +532,7 @@ char *cdw_read_disc_get_audio_file_name(void)
 		return (char *) NULL;
 	}
 
-	cdw_safe_input_data_t data;
+	cdw_input_data_t data;
 
 	data.attempts_max = 3; /* how many attempts? */
 	/* 2TRANS: this is title of dialog window where user have to
@@ -558,7 +548,7 @@ char *cdw_read_disc_get_audio_file_name(void)
 	   when getting a file name from user failed */
 	data.error_message = _("Can't get valid file name, aborting.");
 
-	cdw_rv_t crv = cdw_safe_input_dialog(&data);
+	cdw_rv_t crv = cdw_input_dialog(&data);
 	cdw_main_ui_main_window_wrefresh();
 	if (crv == CDW_OK) {
 		return filename;
