@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,13 +19,14 @@
 
 #include <string.h>
 
-#include "cdw_processwin.h"
+// #include "cdw_processwin.h"
 #include "cdw_read_disc_info.h"
 #include "cdw_disc.h"
+#include "cdw_window.h"
 #include "cdw_main_window.h"
 #include "cdw_widgets.h"
-#include "cdw_drive.h"
-#include "cdw_fs.h"
+// #include "cdw_drive.h"
+// #include "cdw_fs.h"
 #include "gettext.h"
 #include "cdw_debug.h"
 
@@ -33,6 +34,7 @@
 /* use to check if there is a disc in drive and if we can operate on it */
 //static bool cdw_disc_is_disc_ready(void);
 
+#if 0
 static bool using_local_processwin = false;
 
 
@@ -73,16 +75,13 @@ cdw_rv_t cdw_read_disc_info(void)
 				      false);
 	}
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
-	cdw_main_ui_disc_info_view_display_data(current_disc);
+	cdw_disc_t *disc = cdw_disc_get__();
+	cdw_main_ui_disc_info_view_display_data(disc);
 
 	cdw_rv_t retval = CDW_OK;
 	if (!cdw_drive_is_disc_in_drive()) {
 		cdw_vdm ("INFO: no disc in drive\n");
-		/* no disc in drive */
-		cdw_disc_reset(current_disc);
-		cdw_main_ui_disc_info_view_display_data(current_disc);
+		cdw_main_ui_disc_info_view_display_data(disc);
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
 				   /* 2TRANS: this is message in dialog window */
@@ -111,10 +110,9 @@ cdw_rv_t cdw_read_disc_info(void)
 	}
 
 
-	cdw_rv_t mediainfo = cdw_disc_get_meta_info(current_disc);
 	/* get_meta_info() may fail only partially; display what you can */
-	cdw_main_ui_disc_info_view_display_data(current_disc);
-	if (mediainfo != CDW_OK) {
+	cdw_main_ui_disc_info_view_display_data(disc);
+	if (!disc->has_data) {
 		cdw_vdm ("ERROR: can't get disc meta info\n");
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
@@ -139,6 +137,7 @@ cdw_read_disc_info_return:
 	return retval;
 }
 
+#endif
 
 
 
@@ -225,30 +224,40 @@ bool cdw_disc_is_disc_ready(void)
 
 
 
+
 void cdw_read_and_display_disc_info(void)
 {
-	cdw_rv_t crv = cdw_read_disc_info();
+	cdw_rv_t crv = CDW_OK;
+
+	cdw_disc_t *disc = cdw_disc_new();
+	if (disc) {
+		crv = cdw_disc_get(disc);
+	} else {
+		crv = CDW_ERROR;
+	}
+
+	cdw_main_ui_disc_info_view_display_data(disc);
 
 	int n_lines = 20;
 	int n_cols = 70;
 	int begin_y = (LINES - n_lines) / 2;
 	int begin_x = (COLS - n_cols) / 2;
-	WINDOW *window = cdw_ncurses_window_new((WINDOW *) NULL,
-						n_lines, n_cols,
-						begin_y, begin_x,
-						CDW_COLORS_DIALOG,
-						/* 2TRANS: this is a title of a window */
-						_("Disc info"),
-						/* 2TRANS: this is message at the bottom of a window */
-						_("Press any key to close window"));
+	WINDOW *window = cdw_window_new((WINDOW *) NULL,
+					n_lines, n_cols,
+					begin_y, begin_x,
+					CDW_COLORS_DIALOG,
+					/* 2TRANS: this is a title of a window */
+					_("Disc info"),
+					/* 2TRANS: this is message at the bottom of a window */
+					_("Press any key to close window"));
 	if (window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: can't create window\n");
 		return;
 	}
-	WINDOW *subwindow = cdw_ncurses_window_new(window,
-						   n_lines - 2, n_cols - 2, 1, 1,
-						   CDW_COLORS_DIALOG,
-						   (char *) NULL, (char *) NULL);
+	WINDOW *subwindow = cdw_window_new(window,
+					   n_lines - 2, n_cols - 2, 1, 1,
+					   CDW_COLORS_DIALOG,
+					   (char *) NULL, (char *) NULL);
 	if (subwindow == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: can't create window\n");
 		delwin(window);
@@ -263,7 +272,6 @@ void cdw_read_and_display_disc_info(void)
 		/* 2TRANS: this is a message in dialog window */
 		mvwprintw(subwindow, 1, col, _("Can't get disc information"));
 	} else {
-		cdw_disc_t *disc = cdw_disc_get();
 		/* 2TRANS: this is a message/label in dialog window: "disc information"
 		   meta information read from/about an optical disc */
 		mvwprintw(subwindow, 1, col, "Disc information:");
@@ -276,10 +284,10 @@ void cdw_read_and_display_disc_info(void)
 		} else {
 			/* 2TRANS: this is a label in dialog window, "%s" is
 			   for label like "ISO 9660" */
-			mvwprintw(subwindow, 4, col, "File system: %s", disc->fs.type_label);
+			mvwprintw(subwindow, 4, col, "File system: %s", disc->cdio->ofs->type_label);
 			/* 2TRANS: this is a label in dialog window, "%zd" is
 			   for number of tracks on optical disc */
-			mvwprintw(subwindow, 5, col, "Number of tracks: %zd", disc->n_tracks);
+			mvwprintw(subwindow, 5, col, "Number of tracks: %zd", disc->cdio->n_tracks);
 
 			/* 2TRANS: this is a message in dialog window,
 			   "write to the disc" = "burn data to the disc";
@@ -353,10 +361,12 @@ void cdw_read_and_display_disc_info(void)
 			   be followed by string representing volume ID
 			   (a disc label) */
 			mvwprintw(subwindow, 9, col, _("Volume id:"));
-			mvwprintw(subwindow, 10, col, "\"%s\"", disc->fs.volume_id);
+			mvwprintw(subwindow, 10, col, "\"%s\"", disc->cdio->ofs->volume_id);
 		}
 	}
 
+	cdw_disc_delete(&disc);
+
 	wrefresh(subwindow);
 	wgetch(subwindow);
 
