@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -112,6 +112,7 @@ void cdw_task_reset_state(cdw_task_t *task)
 	task->create_image.xorriso.other_xorriso_iso_options = (char *) NULL;
 	task->create_image.pad = true;
 
+	task->disc = (cdw_disc_t *) NULL;
 
 	cdw_task_reset_tools(task);
 
@@ -553,6 +554,13 @@ cdw_rv_t cdw_task_check_tool_status(cdw_task_t *task)
 			   _("cdw can't read any information from the disc using xorriso. Please adjust your settings in Configuration -> Tools."));
 	}
 
+	if (task->tool_status.xorriso & CDW_TOOL_STATUS_XORRISO_NOT_A_KNOWN_OPTION) {
+		cdw_vdm ("ERROR: masked tool_status.xorriso = 0x%08x / CDW_TOOL_STATUS_XORRISO_NOT_A_KNOWN_OPTION\n",
+			 (unsigned int) task->tool_status.xorriso & CDW_TOOL_STATUS_XORRISO_NOT_A_KNOWN_OPTION);
+		cdw_task_e((char *) NULL,
+			   /* 2TRANS: this is message in dialog window */
+			   _("Invalid option passed to xorriso. Check options in text fields in Configuration window, and under wizard's \"Other options\". See log file ('L' hotkey) for details."));
+	}
 
 	return CDW_ERROR;
 }
@@ -1087,12 +1095,28 @@ bool cdw_task_select_tool(cdw_task_t *task, cdw_disc_t *disc)
 		return true;
 
 	} else if (task->id == CDW_TASK_BURN_FROM_FILES) {
+		if (disc->state_empty != CDW_TRUE
+		    && disc->cdio->ofs->cdw_tool == CDW_TOOL_NONE
+		    && task->burn.session_mode == CDW_SESSION_MODE_CONTINUE_MULTI) {
+
+			/* file system present on disc has been created using
+			   tool other than xorriso or mkisofs; this can lead
+			   to problems with adding new data */
+
+			/* 2TRANS: this is a dialog window title */
+			cdw_buttons_dialog(_("Warning"),
+					   /* 2TRANS: this is a message in dialog window */
+					   _("cdw can't recognize tool previously used to burn to this disc. You may experience serious trouble (including data loss) if you decide to continue."),
+					   CDW_BUTTONS_OK, CDW_COLORS_WARNING);
+		}
 		success = cdw_ext_tools_set_iso9660_otf_tool(disc, &(task->create_image.tool));
 		if (!success) {
+			cdw_vdm ("ERROR: failed to get ISO9660 otf tool\n");
 			return false;
 		}
 		success = cdw_ext_tools_set_burn_tool(disc, &(task->burn.tool), &(task->create_image.tool));
 		if (!success) {
+			cdw_vdm ("ERROR: failed to get burn tool (ISO9660 otf tool = \"%s\")\n", cdw_ext_tools_get_tool_name(task->create_image.tool.id));
 			return false;
 		}
 		return true;
@@ -1298,7 +1322,7 @@ cdw_rv_t cdw_task_get_initial_options(cdw_task_t *task, cdw_disc_t *disc)
 
 	if (task->id == CDW_TASK_ERASE_DISC) {
 		task->erase.eject = global_config.eject;
-		task->erase.erase_mode = CDW_ERASE_MODE_FAST;
+		task->erase.erase_mode = global_config.erase_mode;
 		task->erase.format_mode = CDW_ERASE_DVD_RW_FORMAT_NONE;
 		task->erase.speed_id = cdw_disc_get_initial_write_speed_id(disc);
 	} else {
@@ -1352,6 +1376,11 @@ cdw_task_t *cdw_task_new(cdw_id_t id, cdw_disc_t *disc)
 		}
 	}
 
+	if (disc) {
+		cdw_assert (!task->disc, "ERROR: disc is already set\n");
+		task->disc = disc;
+	}
+
 	return task;
 }
 
@@ -1361,7 +1390,8 @@ cdw_task_t *cdw_task_new(cdw_id_t id, cdw_disc_t *disc)
 
 void cdw_task_delete(cdw_task_t **task)
 {
-	cdw_assert (task != (cdw_task_t **) NULL, "ERROR: passing pointer to NULL task to the function\n");
+	cdw_assert (task, "ERROR: passing pointer to NULL task to the function\n");
+
 	if (*task == (cdw_task_t *) NULL) {
 		cdw_vdm ("WARNING passing NULL pointer to task\n");
 		return;
