@@ -88,6 +88,8 @@ void cdw_task_debug_print_burn_options(cdw_task_t *task);
 #define CDW_TOOL_STATUS_XORRISO_SIMULATION_NOT_SUPPORTED         0x00000001
 #define CDW_TOOL_STATUS_XORRISO_NOT_ENOUGH_SPACE                 0x00000002
 #define CDW_TOOL_STATUS_XORRISO_CANNOT_READ_ISO_IMAGE_TREE       0x00000004
+#define CDW_TOOL_STATUS_XORRISO_NOT_A_KNOWN_OPTION               0x00000008
+
 
 /* mode of writing files to a CD/DVD disc */
 typedef enum {
@@ -284,6 +286,9 @@ struct cdw_task {
 		void * (*read_and_write)(void *task);
 
 	} calculate_digest;
+
+	cdw_disc_t *disc;
+
 }; /** \brief Data structure describing task that is currently
        performed or is about to be performed */
 
