@@ -24,7 +24,7 @@ typedef enum {
 
 
 
-void curses_colors_init_phase1(void);
-cdw_rv_t curses_colors_init_phase2(void);
+void     cdw_colors_init_phase1(void);
+cdw_rv_t cdw_colors_init_phase2(void);
 
 #endif /* #ifndef H_CDW_COLORS */
