@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_main_window.h"
 #include "cdw_string.h"
 #include "cdw_erase_wizard.h"
@@ -81,8 +82,8 @@ static cdw_form_dropdown_maker_t dropdown_makers[] = {
 
 
 
-static cdw_disc_t *local_disc;
-static cdw_task_t *local_task;
+static cdw_disc_t *local_disc = (cdw_disc_t *) NULL;
+static cdw_task_t *local_task = (cdw_task_t *) NULL;
 
 
 /* constants for layout of elements in wizard window */
@@ -141,7 +142,7 @@ enum {
    of DVD can quickly render it unusable.
 
    \param task - variable describing current task
-   \param current_disc - variable describing current disc
+   \param disc - variable describing current disc
 
    \return CDW_OK if user pressed accepts erasing
    \return CDW_CANCEL if user cancels erasing
@@ -327,7 +328,7 @@ cdw_rv_t cdw_erase_wizard_initial_warning(cdw_disc_type_t disc_type)
 		cdw_rv_t w = cdw_buttons_dialog(_("Attempting to erase a DVD+/-RW disc"),
 						/* 2TRANS: this is warning message in dialog window.
 						   User can press OK or Cancel button. */
-						_("This will (partially) erase information from disc making it unreadable.\nThe process may take some time.\n\nYou don't have to erase DVD+RW or DVD-RW Restricted Overwrite disc to create new compilation - just choose \"Start new, appendable disc\" in Write wizard.\n\nDo you want to continue with erasing?"),
+						_("This will (partially) erase information from disc making it unreadable. The process may take some time. You don't have to erase DVD+RW or DVD-RW Restricted Overwrite disc to create new compilation - just choose \"Start new, appendable disc\" in Write wizard. Do you want to continue with erasing?"),
 						CDW_BUTTONS_OK_CANCEL, CDW_COLORS_WARNING);
 
 		if (w != CDW_OK) {
@@ -339,7 +340,7 @@ cdw_rv_t cdw_erase_wizard_initial_warning(cdw_disc_type_t disc_type)
 		cdw_rv_t w = cdw_buttons_dialog(_("Attempting to erase a DVD-RW disc"),
 						/* 2TRANS: this is warning message in dialog window.
 						   User can press OK or Cancel button. */
-						_("This will (partially) erase information from disc making it unreadable.\nThe process may take some time.\n\nExcessive erasing of DVD may limit lifetime of media.\n\nDo you want to continue with erasing?"),
+						_("This will (partially) erase information from disc making it unreadable. The process may take some time. Excessive erasing of DVD may limit lifetime of media. Do you want to continue with erasing?"),
 						CDW_BUTTONS_OK_CANCEL, CDW_COLORS_WARNING);
 		if (w != CDW_OK) {
 			return CDW_CANCEL;
@@ -383,25 +384,25 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 
 	int begin_y = ((LINES - window_n_lines) / 2) - 2;
 	int begin_x = (COLS - window_n_cols) / 2;
-	wizard.cdw_form->window = cdw_ncurses_window_new((WINDOW *) NULL,
-							 window_n_lines, window_n_cols,
-							 begin_y, begin_x,
-							 CDW_COLORS_DIALOG,
-							 /* 2TRANS: this is title of wizard window; "erase" as in
-							    "Erasing an optical disc" */
-							 _("Erase wizard"),
-							 /* 2TRANS: this is tip at the bottom of window - user can
-							    switch between window elements using tab key */
-							 _("Use 'Tab' key to move"));
+	wizard.cdw_form->window = cdw_window_new((WINDOW *) NULL,
+						 window_n_lines, window_n_cols,
+						 begin_y, begin_x,
+						 CDW_COLORS_DIALOG,
+						 /* 2TRANS: this is title of wizard window; "erase" as in
+						    "Erasing an optical disc" */
+						 _("Erase wizard"),
+						 /* 2TRANS: this is tip at the bottom of window - user can
+						    switch between window elements using tab key */
+						 _("Use 'Tab' key to move"));
 	if (wizard.cdw_form->window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create window\n");
 		return CDW_ERROR;
 	}
 
-	wizard.cdw_form->subwindow = cdw_ncurses_window_new(wizard.cdw_form->window,
-							    subwindow_n_lines, subwindow_n_cols,
-							    3, 1,
-							    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
+	wizard.cdw_form->subwindow = cdw_window_new(wizard.cdw_form->window,
+						    subwindow_n_lines, subwindow_n_cols,
+						    3, 1,
+						    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
 
 	if (wizard.cdw_form->subwindow == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create subwindow\n");
@@ -412,7 +413,7 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 	mvwprintw(wizard.cdw_form->window, top_label_row, first_col,
 		  /* 2TRANS: this is message to user: current action will
 		     be erasing "%s" disc, where "%s" is DVD or CD */
-		  _("Select parameters for erasing %s disc"), disc->simple_type_label);
+		  _("Select parameters for erasing %s disc"), disc->cdio->simple_type_label);
 
 	cdw_erase_wizard_build_fields(task);
 
@@ -470,6 +471,7 @@ cdw_rv_t cdw_erase_wizard_build_fields(cdw_task_t *task)
 		{ CDW_WIDGET_DROPDOWN,   format_mode_row + 1,  first_col + 1,  wide_dropdown_n_cols - 2,    1,    f_format_mode_dd,    dropdown_makers,                2 },
 
 		{ CDW_WIDGET_CHECKBOX,   eject_row,            first_col + 2,  1,                           1,    f_eject_cb,         (void *) NULL,  task->erase.eject ? 1 : 0 },
+		/* 2TRANS: this is label of checkbox; "tray" is a tray of CD burner; "erasing" refers to blanking of optical disc */
 		{ CDW_WIDGET_LABEL,      eject_row,            first_col + 5,  label_n_cols - 5,            1,    f_eject_l,          _("Eject tray after erasing") ,  0 },
 
 		/* 2TRANS: button label, it refers to erasing optical disc */
