@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -354,7 +354,7 @@ cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], cdw_form_t *cd
 			cdw_assert (rv == E_OK, "ERROR: can't set userptr for field #%d\n", i);
 		} else if (widget_type == CDW_WIDGET_INPUT) {
 			cdw_vdm ("INFO: creating field #%d: input with initial string = \"%s\"\n", i, (char *) descr[i].data1);
-			fields[i] = cdw_ncurses_new_input_field((size_t) descr[i].n_lines,
+			fields[i] = cdw_ncurses_new_input_field(descr[i].n_lines,
 								descr[i].n_cols,
 								descr[i].begin_y,
 								descr[i].begin_x,
@@ -364,7 +364,7 @@ cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], cdw_form_t *cd
 								CDW_COLORS_INPUT);
 		} else if (widget_type == CDW_WIDGET_BUTTON) {
 			cdw_vdm ("INFO: creating field #%d: button\n", i);
-			fields[i] = cdw_ncurses_new_input_field((size_t) descr[i].n_lines,
+			fields[i] = cdw_ncurses_new_input_field(descr[i].n_lines,
 								descr[i].n_cols,
 								descr[i].begin_y,
 								descr[i].begin_x,
@@ -385,7 +385,7 @@ cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], cdw_form_t *cd
 			cdw_assert (rv == E_OK, "ERROR: can't set userptr for field #%d\n", i);
 		} else if (widget_type == CDW_WIDGET_DROPDOWN) {
 			cdw_vdm ("INFO: creating field #%d: dropdown (new)\n", i);
-			fields[i] = cdw_ncurses_new_input_field((size_t) descr[i].n_lines,
+			fields[i] = cdw_ncurses_new_input_field(descr[i].n_lines,
 								descr[i].n_cols,
 								descr[i].begin_y,
 								descr[i].begin_x,
@@ -830,8 +830,8 @@ int cdw_form_handle_enter(cdw_form_t *cdw_form, int fi)
 	} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
 		CDW_DROPDOWN *dropdown = cdw_form_get_dropdown(cdw_form, fi);
 		cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: dropdown is NULL\n");
-		cdw_rv_t crv = cdw_dropdown_driver(dropdown);
-		if (crv == CDW_OK) {
+		int key = cdw_dropdown_driver(dropdown);
+		if (key == CDW_KEY_ENTER) {
 			const char *label = cdw_dropdown_get_current_item_label(dropdown);
 			FIELD **fields = form_fields(cdw_form->form);
 			/* setting buffer makes field to display
