@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "cdw_string.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_main_window.h"
 #include "cdw_debug.h"
 #include "cdw_ext_tools.h"
@@ -297,18 +298,18 @@ cdw_rv_t cdw_iso9660_wizard_build(cdw_task_t *task)
 	int begin_y = ((LINES - window_n_lines) / 2) - 2;
 	int begin_x = (COLS - window_n_cols) / 2;
 	wizard.cdw_form->window =
-		cdw_ncurses_window_new((WINDOW *) NULL,
-				       window_n_lines, window_n_cols,
-				       begin_y, begin_x,
-				       CDW_COLORS_DIALOG,
-				       /* 2TRANS: this is title of wizard
-					  window; "write" as in
-					  'writing to optical disc" */
-				       _("ISO9660 wizard"),
-				       /* 2TRANS: this is tip at the bottom
-					  of window - user can switch between
-					  window elements using tab key */
-				       _("Use 'Tab' key to move"));
+		cdw_window_new((WINDOW *) NULL,
+			       window_n_lines, window_n_cols,
+			       begin_y, begin_x,
+			       CDW_COLORS_DIALOG,
+			       /* 2TRANS: this is title of wizard
+				  window; "write" as in
+				  'writing to optical disc" */
+			       _("ISO9660 wizard"),
+			       /* 2TRANS: this is tip at the bottom
+				  of window - user can switch between
+				  window elements using tab key */
+			       _("Use 'Tab' key to move"));
 
 	if (wizard.cdw_form->window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create window\n");
@@ -316,11 +317,11 @@ cdw_rv_t cdw_iso9660_wizard_build(cdw_task_t *task)
 	}
 
 	wizard.cdw_form->subwindow =
-		cdw_ncurses_window_new(wizard.cdw_form->window,
-				       subwindow_n_lines, subwindow_n_cols,
-				       3, 1,
-				       CDW_COLORS_DIALOG,
-				       (char *) NULL, (char *) NULL);
+		cdw_window_new(wizard.cdw_form->window,
+			       subwindow_n_lines, subwindow_n_cols,
+			       3, 1,
+			       CDW_COLORS_DIALOG,
+			       (char *) NULL, (char *) NULL);
 
 	if (wizard.cdw_form->subwindow == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create subwindow\n");
@@ -332,13 +333,17 @@ cdw_rv_t cdw_iso9660_wizard_build(cdw_task_t *task)
 
 	cdw_form_descr_t descr[] = {
 		/*     type             begin_y                   begin_x         n_cols                n_lines    field enum      data1                  data2 */
+		/* 2TRANS: this is label next to input field, where user can enter a label describing created ISO9660 volume */
 		{ CDW_WIDGET_LABEL,   volume_id_label_row,      first_col,      wide_label_n_cols,       1,  f_volume_id_l,  _("Volume ID:"),                                  0 },
 		{ CDW_WIDGET_INPUT,   volume_id_label_row + 1,  first_col,      CDW_ISO9660_VOLI_LEN,    1,  f_volume_id_i,  task->create_image.volume_id,  CDW_ISO9660_VOLI_LEN },
 
+		/* 2TRANS: this is label next to input field, where user can specify path to ISO9660 file */
 		{ CDW_WIDGET_LABEL,   iso_file_label_row,       first_col,      wide_label_n_cols - 2,   1,  f_iso_file_l,   _("ISO9660 image file:"),                         0 },
 		{ CDW_WIDGET_INPUT,   iso_file_label_row + 1,   first_col,      wide_label_n_cols - 8,   1,  f_iso_file_i,   task->create_image.iso9660_file_fullpath,         0 },
+		/* 2TRANS: do not translate */
 		{ CDW_WIDGET_BUTTON,  iso_file_label_row + 1,   iso_button_col, 1,                       1,  f_iso_file_b,   _(">"),                           CDW_COLORS_DIALOG },
 
+		/* 2TRANS: this is label of a button that opens configuration window; "more options" related to creating ISO9660 file */
 		{ CDW_WIDGET_BUTTON,  options_button_row,       first_col,      1,                       1,  f_options_b,    _("More options"),                CDW_COLORS_DIALOG },
 
 		/* 2TRANS: button label, it refers to writing to optical disc */
@@ -508,9 +513,13 @@ static char *cdw_wizard_window_tip = (char *) NULL;
    aggregating data and functions that prepare the data; the data
    represents widgets displayed in pages of options window */
 static cdw_config_page_t c_pages[] = {
+	/* 2TRANS: label of a tab in configuration window with options related to ISO9660; probably you don't want to translate this */
 	{ false, CDW_MKISOFS_OPTIONS_N_FIELDS,       cdw_mkisofs_options_form,       gettext_noop("ISO9660"),   (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related to ISO9660; probably you don't want to translate this */
 	{ false, CDW_XORRISO_ISO_OPTIONS_N_FIELDS,   cdw_xorriso_iso_options_form,   gettext_noop("ISO9660"),   (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related to meta information of ISO9660 file system */
 	{ false, CDW_MKISOFSRC_OPTIONS_N_FIELDS,     cdw_mkisofsrc_options_form,     gettext_noop("Meta data"), (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related to meta information of ISO9660 file system */
 	{ false, CDW_XORRISORC_N_FIELDS,             cdw_xorrisorc_form,             gettext_noop("Meta data"), (cdw_form_t *) NULL }};
 
 
@@ -526,7 +535,9 @@ cdw_rv_t cdw_iso9660_wizard_options(cdw_task_t *task)
 	/* 2TRANS: this is message at the bottom of options window;
 	   cancel means: quit without saving; '%d' is an integer used to
 	   create label of function key, e.g. "F10" */
-	int rv = asprintf(&cdw_wizard_window_tip, _("Press F%d key to save changes or ESC to cancel"), CDW_CONFIG_UI_SnC_KEY);
+	int rv = asprintf(&cdw_wizard_window_tip,
+			  _("Press F%d / F%d key to save changes or ESC to cancel"),
+			  CDW_CONFIG_UI_SnC_KEY_B, CDW_CONFIG_UI_SnC_KEY_A);
 	if (cdw_wizard_window_tip == (char *) NULL || rv == -1) {
 		cdw_vdm ("ERROR: failed to create window tip\n");
 		return CDW_ERROR;
@@ -559,10 +570,9 @@ cdw_rv_t cdw_iso9660_wizard_options(cdw_task_t *task)
 		ind++;
 	}
 
-	cdw_tabs_window_add_return_key(c_twindow, KEY_F(CDW_CONFIG_UI_SnC_KEY));
-	cdw_tabs_window_add_return_key(c_twindow, CDW_KEY_ESCAPE);
-	cdw_tabs_window_add_return_key(c_twindow, 'q');
-	cdw_tabs_window_add_return_key(c_twindow, 'Q');
+	cdw_tabs_window_add_return_keys(c_twindow,
+					KEY_F(CDW_CONFIG_UI_SnC_KEY_A), KEY_F(CDW_CONFIG_UI_SnC_KEY_B),
+					CDW_KEY_ESCAPE, 'q', 'Q', 0);
 	c_twindow->driver_reader = cdw_config_ui_tabs_window_form_reader;
 	cdw_tabs_window_finalize(c_twindow);
 
@@ -590,7 +600,8 @@ cdw_rv_t cdw_iso9660_wizard_options(cdw_task_t *task)
 		for (int j = 0; j < n_tabs; j++) {
 			cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(j + CDW_CONFIG_UI_FX_START_KEY));
 		}
-		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_A));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_B));
 		cdw_form_add_return_char(c_pages[id].cdw_form, 'q'); /* will work only for non-text fields */
 	}
 	c_twindow->user_data = (void *) c_pages;
@@ -668,7 +679,8 @@ cdw_rv_t cdw_iso9660_wizard_options_window_driver(cdw_task_t *task)
 		int key = cdw_tabs_window_driver(c_twindow);
 		if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
 			break;
-		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY)) { /* SnC = Save and Close */
+		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY_A)
+			   || key == KEY_F(CDW_CONFIG_UI_SnC_KEY_B)) { /* SnC = Save and Close */
 
 			/* flush */
 			for (int i = 0; i < c_twindow->n_tabs; i++) {
