@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@
 
    New display by default doesn't react to any keys other than movement keys
    and Escape key. This can be changed later by using
-   cdw_list_display_add_return_char()
+   cdw_list_display_add_return_key()
 
    TODO: decouple list and display - list should be separate and independent
    data structure (perhaps coupled with two pointers to functions that can:
@@ -76,7 +76,7 @@
 
 
 static void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offset, bool highlight);
-
+static int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset);
 
 
 /**
@@ -168,6 +168,8 @@ CDW_LIST_DISPLAY *cdw_list_display_new(WINDOW *parent, int n_lines, int n_cols,
 	wbkgd(display->subwindow, A_NORMAL | COLOR_PAIR(display->colors));
 	(void) wattrset(display->subwindow, A_NORMAL | COLOR_PAIR(display->colors));
 
+	cdw_list_display_add_return_keys(display, CDW_KEY_ESCAPE, 'q', 'Q', 0);
+
 	return display;
 }
 
@@ -242,7 +244,7 @@ void cdw_list_display_delete(CDW_LIST_DISPLAY **display)
    then the driver reacts to Left/Right arrows as well.
 
    Supporting of other keys (characters) can be enabled by adding these
-   characters with cdw_list_display_add_return_char() function. The only
+   characters with cdw_list_display_add_return_key() function. The only
    "reaction" to enabled key is to return with value corresponding to key
    pressed. It is up to caller of the driver function to act upon such
    event, including calling the driver function again to resume reacting
@@ -259,7 +261,7 @@ void cdw_list_display_delete(CDW_LIST_DISPLAY **display)
 
    \param display - display that currently has keyboard focus
 
-   \return CDW_ESCAPE if user pressed escape key
+   \return CDW_KEY_ESCAPE if user pressed escape key
    \return other specific key, if display was configured to react to other keys
 */
 int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
@@ -277,22 +279,42 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 	}
 	wrefresh(display->subwindow);
 
+	int key = cdw_list_display_driver_loop(display, &h_offset);
+
+	if (display->n_items != 0) {
+		/* control (and focus) is leaving display area, and to indicate this,
+		   we un-highlight current item */
+		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
+		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
+		display->display_item(display, item->data, subwindow_row, h_offset, false);
+	}
+
+	wrefresh(display->subwindow);
+
+	return key;
+}
+
+
+
+
+
+int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
+{
 	int key = 'a'; /* safe (?) initial value */
-	while (key != CDW_KEY_ESCAPE) {
-		key = wgetch(display->window);
+	while ((key = wgetch(display->window)) != ERR) {
 		switch (key) {
 		        case KEY_HOME:
 				/* first Home key should move content of
 				   window to beginning of lines, leaving
 				   vertical position unchanged */
-				if (h_offset > 0) {
-					h_offset = 0;
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+				if (*h_offset > 0) {
+					*h_offset = 0;
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				} else {
 					if (display->current_item_ind > 0) {
 						display->current_item_ind = 0;
-						h_offset = 0;
-						cdw_list_display_scroll_to_current(display, h_offset, true);
+						*h_offset = 0;
+						cdw_list_display_scroll_to_current(display, *h_offset, true);
 					}
 				}
 				break;
@@ -301,7 +323,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 				/* go to last item */
 				if (display->current_item_ind < display->n_items - 1) {
 					display->current_item_ind = display->n_items - 1;
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 				break;
 
@@ -309,7 +331,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 				/* go to next item */
 				if (display->current_item_ind < display->n_items - 1) {
 					display->current_item_ind++;
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 				break;
 
@@ -317,7 +339,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 				/* go to previous item */
 				if (display->current_item_ind > 0) {
 					display->current_item_ind--;
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 
 				break;
@@ -327,14 +349,14 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 				   "item display function" that does not
 				   support non-zero h_offset, it will be
 				   ignored by such "item display function" */
-				h_offset++;
-				cdw_list_display_scroll_to_current(display, h_offset, true);
+				(*h_offset)++;
+				cdw_list_display_scroll_to_current(display, *h_offset, true);
 				break;
 			case KEY_LEFT:
 				/* see comment for KEY_RIGHT */
-				if (h_offset > 0) {
-					h_offset--;
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+				if (*h_offset > 0) {
+					(*h_offset)--;
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 				break;
 
@@ -351,7 +373,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 						   screen size up */
 						display->current_item_ind -= (size_t) display->n_lines;
 					}
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 				break;
 
@@ -362,31 +384,25 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 					if (display->current_item_ind > display->n_items - 1) {
 						display->current_item_ind = display->n_items - 1;
 					}
-					cdw_list_display_scroll_to_current(display, h_offset, true);
+					cdw_list_display_scroll_to_current(display, *h_offset, true);
 				}
 				break;
 
 			default:
-				break;
+				; // break;
 
 		} /* switch (c) */
 
-		if (cdw_list_display_is_return_char(display, key)) {
+		if (cdw_list_display_is_return_key(display, key)) {
 			break;
 		}
-	} /* while (c != CDW_ESCAPE) */
+	} /* while () */
 
-	if (display->n_items != 0) {
-		/* control (and focus) is leaving display area, and to indicate this,
-		   we un-highlight current item */
-		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
-		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
-		display->display_item(display, item->data, subwindow_row, h_offset, false);
+	if (key == 'q' || key == 'Q' || key == CDW_KEY_ESCAPE) {
+		return CDW_KEY_ESCAPE;
+	} else {
+		return key;
 	}
-
-	wrefresh(display->subwindow);
-
-	return key;
 }
 
 
@@ -513,7 +529,7 @@ void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offs
    \param display - display that you want to configure
    \param c - character that you want to add to given \p display
 */
-void cdw_list_display_add_return_char(CDW_LIST_DISPLAY *display, int c)
+void cdw_list_display_add_return_key(CDW_LIST_DISPLAY *display, int c)
 {
 	cdw_assert (display != (CDW_LIST_DISPLAY *) NULL, "ERROR: \"display\" argument is NULL\n");
 	cdw_assert (display->n_return_chars < N_RETURN_KEYS_MAX,
@@ -521,7 +537,7 @@ void cdw_list_display_add_return_char(CDW_LIST_DISPLAY *display, int c)
 		    display->n_return_chars, N_RETURN_KEYS_MAX);
 	cdw_assert (c != 0, "ERROR: trying to add char == 0, but 0 is an initializer value\n");
 
-	if (cdw_list_display_is_return_char(display, c)) {
+	if (cdw_list_display_is_return_key(display, c)) {
 		cdw_vdm ("WARNING: attempting to add char %d / \"%s\", but it is already on the list of return chars\n",
 			 c, cdw_ncurses_key_label(c));
 	} else {
@@ -560,7 +576,7 @@ void cdw_list_display_add_return_keys(CDW_LIST_DISPLAY *display, ...)
 	va_start(ap, display);
 	int key = 'a';
 	while ((key = va_arg(ap, int)) != 0) {
-		cdw_list_display_add_return_char(display, key);
+		cdw_list_display_add_return_key(display, key);
 	}
 	va_end(ap);
 
@@ -584,7 +600,7 @@ void cdw_list_display_add_return_keys(CDW_LIST_DISPLAY *display, ...)
   \return true if given character was added to given \p display
   \return false if given character was not added to given \p display
 */
-bool cdw_list_display_is_return_char(CDW_LIST_DISPLAY *display, int c)
+bool cdw_list_display_is_return_key(CDW_LIST_DISPLAY *display, int c)
 {
 	cdw_assert (display != (CDW_LIST_DISPLAY *) NULL, "ERROR: \"display\" argument is NULL\n");
 	cdw_assert (c != 0, "ERROR: asking for 0, which is an initialization value\n");
