@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "cdw_debug.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_text_file_viewer.h"
 #include "cdw_drive.h"
 #include "cdw_cdio_drives.h"
@@ -159,7 +160,7 @@ cdw_rv_t cdw_main_ui_init(void)
 		return CDW_ERROR;
 	}
 
-	cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
+	cdw_main_window_volume_info_view_update(-1, -1, true, global_config.follow_symlinks);
 	//selected_files_size_mb = 0.0;
 	//cdw_ui_volume_info_view_display_data();
 
@@ -219,15 +220,15 @@ void cdw_main_ui_clean(void)
 	   and then you press 'q' again and then select 'ok'
 	   2. if you run cdw in 81x26 terminal */
 
-	cdw_ncurses_window_delete(&cdw_ui.files_list_view.subwindow);
-	cdw_ncurses_window_delete(&cdw_ui.main_menu.menu_subwindow);
+	cdw_window_delete(&cdw_ui.files_list_view.subwindow);
+	cdw_window_delete(&cdw_ui.main_menu.menu_subwindow);
 	cdw_assert (cdw_ui.main_menu.menu_subwindow == (WINDOW *) NULL,
 		    "ERROR: delete() function didn't set a window to NULL\n");
-	cdw_ncurses_window_delete(&cdw_ui.main_menu.subwindow);
-	cdw_ncurses_window_delete(&cdw_ui.volume_info_view.subwindow);
-	cdw_ncurses_window_delete(&cdw_ui.disc_info_view.subwindow);
-	cdw_ncurses_window_delete(&cdw_ui.tooltips_view.subwindow);
-	cdw_ncurses_window_delete(&cdw_ui.window);
+	cdw_window_delete(&cdw_ui.main_menu.subwindow);
+	cdw_window_delete(&cdw_ui.volume_info_view.subwindow);
+	cdw_window_delete(&cdw_ui.disc_info_view.subwindow);
+	cdw_window_delete(&cdw_ui.tooltips_view.subwindow);
+	cdw_window_delete(&cdw_ui.window);
 
 	return;
 }
@@ -388,7 +389,7 @@ cdw_rv_t cdw_main_ui_main_menu_view_create(void)
 
 	/* no title for this subarea - it would be overwritten with
 	   app name + version */
-	cdw_ncurses_nice_box(cdw_ui.main_menu.subwindow, (char *) NULL, (char *) NULL);
+	cdw_window_add_strings(cdw_ui.main_menu.subwindow, (char *) NULL, (char *) NULL);
 
 	cdw_ui.main_menu.items = (ITEM **) calloc(MENU_ITEMS + 1, sizeof(ITEM *));
 	if (!cdw_ui.main_menu.items) {
@@ -471,11 +472,11 @@ cdw_rv_t cdw_main_ui_files_list_view_create(void)
 	}
 
 	keypad(cdw_ui.files_list_view.subwindow, TRUE);
-	cdw_ncurses_nice_box(cdw_ui.files_list_view.subwindow,
-			     /* 2TRANS: this is title of biggest area in
-				cdw UI that shows files selected for
-				writing to cd or image file */
-			     _("Selected files"), (char *) NULL);
+	cdw_window_add_strings(cdw_ui.files_list_view.subwindow,
+			       /* 2TRANS: this is title of biggest area in
+				  cdw UI that shows files selected for
+				  writing to cd or image file */
+			       _("Selected files"), (char *) NULL);
 	wattrset(cdw_ui.files_list_view.subwindow, A_NORMAL | COLOR_PAIR(CDW_COLORS_MAIN));
 
 	cdw_rv_t crv = cdw_file_manager_create_selected_files_view(cdw_ui.files_list_view.subwindow);
@@ -515,14 +516,22 @@ cdw_rv_t cdw_main_ui_disc_info_view_create(void)
 		return CDW_ERROR;
 	}
 
-	cdw_ncurses_nice_box(cdw_ui.disc_info_view.subwindow,
-			     /* 2TRANS: this is title of area displaying basic
-				information about disc currently in drive */
-			     _("Disc info"), (char *) NULL);
+	cdw_window_add_strings(cdw_ui.disc_info_view.subwindow,
+			       /* 2TRANS: this is title of area displaying basic
+				  information about disc currently in drive */
+			       _("Disc info"), (char *) NULL);
 
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
-	cdw_main_ui_disc_info_view_display_data(current_disc);
+	cdw_disc_t *disc = cdw_disc_new();
+
+	/* no call to cdw_disc_get() when UI is created;
+	   if there is no disc in drive, the user would be nagged to
+	   insert disc into drive - that nagging wouldn't be welcomed
+	   by the user
+
+	   cdw_disc_get(disc); */
+
+	cdw_main_ui_disc_info_view_display_data(disc);
+	cdw_disc_delete(&disc);
 
 	return CDW_OK;
 }
@@ -571,7 +580,7 @@ void cdw_main_ui_disc_info_view_display_data(const cdw_disc_t *disc)
 	   if no disc in drive then "No disc"
 	   if disc is empty then "<disc type>"
 	   if disc is not empty then "<disc type>/<file system type>" */
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_UNKNOWN) {
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_UNKNOWN) {
 		/* 2TRANS: "No disc" means "currently there is no disc in
 		   drive or no information about disc has been collected
 		   yet"; keep short */
@@ -597,7 +606,7 @@ void cdw_main_ui_disc_info_view_display_data(const cdw_disc_t *disc)
 				     about disc: 1st %s is disc type (e.g. CD-RW),
 				     2nd %s is file system type on a disc; resulting
 				     string may look like "CD-RW / ISO9660"; keep short */
-				  _("%s / %s"), disc->type_label, disc->fs.type_label);
+				  _("%s / %s"), disc->type_label, disc->cdio->ofs->type_label);
 		}
 	}
 
@@ -674,7 +683,7 @@ void cdw_main_ui_main_window_wrefresh_part(int n_lines, int n_cols, int begin_y,
 		cdw_vdm ("ERROR: failed to create derived window for refresh\n");
 	} else {
 		wrefresh(window);
-		cdw_ncurses_window_delete(&window);
+		cdw_window_delete(&window);
 	}
 
 	return;
@@ -701,8 +710,18 @@ void cdw_main_ui_main_window_wrefresh(void)
 
 
 
-cdw_rv_t cdw_main_window_volume_info_view_update(bool rescan_selected_files, bool follow_symlinks)
+cdw_rv_t cdw_main_window_volume_info_view_update(long long disc_sectors_used, long long disc_sectors_total, bool rescan_selected_files, bool follow_symlinks)
 {
+	static long long disc_capacity_sectors_used = 0;
+	static long long disc_capacity_sectors_total = 0;
+
+	if (disc_sectors_total >= 0) {
+		disc_capacity_sectors_total = disc_sectors_total;
+		if (disc_sectors_used >= 0) {
+			disc_capacity_sectors_used = disc_sectors_used;
+		}
+	}
+
 	size_t n_files = cdw_file_manager_number_of_selected_files();
 	cdw_sdm ("INFO: number of selected files = %zd\n", n_files);
 
@@ -726,10 +745,9 @@ cdw_rv_t cdw_main_window_volume_info_view_update(bool rescan_selected_files, boo
 	bool show_disc_info = false;
 
 	if (global_config.volume_size_id == CDW_CONFIG_VOLUME_SIZE_AUTO) {
-		/* auto == "get disc capacity from disc" */
-		cdw_disc_t *disc = cdw_disc_get();
-		used_space = (double) disc->capacity.sectors_used / 512.0;
-		total_space = (int) disc->capacity.sectors_total / 512;
+		used_space = (double) disc_capacity_sectors_used / 512.0;
+		total_space = (int) disc_capacity_sectors_total / 512;
+
 		if (total_space > 0) {
 			show_disc_info = true;
 		} else {
@@ -808,6 +826,9 @@ void cdw_main_ui_volume_info_view_display_data_new(bool show_disc_info, size_t n
 	redrawwin(subwindow);
 	wrefresh(subwindow);
 
+	/* TODO: implement displaying size in kB or GB when applicable;
+	   this won't be simple as it will require changes in few
+	   different places */
 	mvwprintw(subwindow, 2, col, n_files ?
 		  /* 2TRANS: '%d' is a number of new files selected for
 		     burning, '%.1f' is a size of the files; keep tilde
@@ -819,7 +840,7 @@ void cdw_main_ui_volume_info_view_display_data_new(bool show_disc_info, size_t n
 		  _("Selected files (%d): %.1f MB"),
 		  n_files, selected_size);
 
-	if (total_space != 0) {
+	if (total_space > 0) {
 		if (show_disc_info) {
 			/* 2TRANS: this is information about disc usage
 			   and capacity: '%.1f' is size of data on disc,
@@ -960,10 +981,10 @@ cdw_rv_t cdw_main_ui_volume_info_view_create(void)
 		return CDW_ERROR;
 	}
 
-	cdw_ncurses_nice_box(cdw_ui.volume_info_view.subwindow,
-			     /* 2TRANS: this is title of area showing basic info (total
-				size, CD usage) about selected files */
-			     _("ISO volume"), (char *) NULL);
+	cdw_window_add_strings(cdw_ui.volume_info_view.subwindow,
+			       /* 2TRANS: this is title of area showing basic info (total
+				  size, CD usage) about selected files */
+			       _("ISO volume"), (char *) NULL);
 
 	return CDW_OK;
 }
@@ -1121,7 +1142,7 @@ void cdw_main_ui_handle_follow_symlinks_change(bool old_state, bool current_stat
 		   and this may affect size of selected files */
 		cdw_vdm ("INFO: changed \"follow symlinks\" to %s, recalculating files size\n",
 			 current_state ? "true" : "false");
-		cdw_main_window_volume_info_view_update(true, current_state);
+		cdw_main_window_volume_info_view_update(-1, -1, true, current_state);
 		cdw_file_manager_regenerate_selected_files_view();
 	} else {
 		cdw_vdm ("INFO: \"follow symlinks\" is still %s\n", current_state ? "true" : "false");
