@@ -61,7 +61,7 @@ void cdw_main_ui_main_window_wrefresh(void);
 //cdw_rv_t cdw_main_ui_disc_info_view_update(void);
 void cdw_main_ui_disc_info_view_display_data(const cdw_disc_t *disc);
 
-cdw_rv_t cdw_main_window_volume_info_view_update(bool rescan_selected_files, bool follow_symlinks);
+cdw_rv_t cdw_main_window_volume_info_view_update(long long disc_sectors_used, long long disc_sectors_total, bool rescan_selected_files, bool follow_symlinks);
 
 cdw_rv_t cdw_main_ui_add_to_selected_files(void);
 cdw_rv_t cdw_main_ui_delete_from_selected_files(void);
