@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "gettext.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_processwin.h"
 #include "cdw_debug.h"
 
@@ -160,7 +161,7 @@ cdw_rv_t cdw_processwin_create(const char *window_title, const char *window_labe
 		return CDW_ERROR;
 	}
 
-	cdw_ncurses_nice_box(processwin.window, window_title, (char *) NULL);
+	cdw_window_add_strings(processwin.window, window_title, (char *) NULL);
 
 	if (show_progress) {
 		cdw_processwin_add_progress_bar();
@@ -566,7 +567,7 @@ void cdw_processwin_display_text_info(int row, const char *string)
 
 	werase(window);
 	if (string != (char *) NULL && (strcmp(string, ""))) {
-		cdw_textarea_print_message(window, string, CDW_ALIGN_CENTER);
+		cdw_window_print_message(window, string, CDW_ALIGN_CENTER);
 	} else {
 		/* user may pass null pointer or empty string ("")
 		   to show that given text area should be erased */
@@ -602,6 +603,8 @@ void cdw_processwin_display_fifo_and_speed(int fifo_level, int speed_decimal, in
 		/* display fifo */
 
 		char fifo_string[PROCESSWIN_MAX_RTEXT_LEN + 1];
+		/* FIXME: digits disappear when fifo_level == 0; probably a problem
+		   with format string */
 		/* 2TRANS: this is message displayed in progress window, describing
 		   FIFO buffer usage (in percents) during some resources-consuming
 		   tasks */
