@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "cdw_main_window.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_debug.h"
 
 
@@ -297,7 +298,7 @@ cdw_rv_t cdw_text_file_viewer(FILE *file, const char *title)
 
 	/* 2TRANS: this is help text displayed at the bottom of text file
 	   viewer window: it explains which keys can be used */
-	cdw_ncurses_nice_box(display->window, title, _("Use ESC to close, use arrows, Home and End to move"));
+	cdw_window_add_strings(display->window, title, _("Use ESC to close, use arrows, Home and End to move"));
 
 	ssize_t n = cdw_text_file_viewer_read_file(file, &(display->list));
 	if (n != -1) {
