@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
    Writing speed values and modes that can be selected in wizard window are
    calculated every time the window is displayed and are disc-specific.
    They are calculated by functions defined in this file using values stored
-   in current_disc variable. Be sure to call run_command_cdrecord_atip(),
+   in disc argument. Be sure to call run_command_cdrecord_atip(),
    run_command_cdrecord_msinfo() or run_command_dvd_rw_mediainfo() before
    calling write_wizard().
 
@@ -52,6 +52,7 @@
 #include "cdw_string.h"
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
+#include "cdw_window.h"
 #include "cdw_write_wizard.h"
 #include "cdw_main_window.h"
 #include "cdw_debug.h"
@@ -617,7 +618,7 @@ void cdw_write_wizard_debug_verify_invisible(cdw_task_t *task, cdw_disc_t *disc)
 	if (disc->state_empty != CDW_TRUE) {
 		cdw_vdm ("INFO: \"verify\" checkbox is not visible because disc is not blank (according to cdio)\n");
 	}
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
+	if (disc->cdio->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
 		if (! (disc->type == CDW_DVD_RWP || disc->type == CDW_DVD_RW_RES)) {
 			cdw_vdm ("INFO: \"verify\" checkbox is not visible because disc is not DVD+RW or DVD-RW RES\n");
 		}
@@ -652,25 +653,25 @@ cdw_rv_t cdw_write_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 {
 	int begin_y = ((LINES - window_n_lines) / 2);
 	int begin_x = (COLS - window_n_cols) / 2;
-	wizard.cdw_form->window = cdw_ncurses_window_new((WINDOW *) NULL,
-							 window_n_lines, window_n_cols,
-							 begin_y, begin_x,
-							 CDW_COLORS_DIALOG,
-							 /* 2TRANS: this is title of wizard window; 'write' as in
-							    'writing to optical disc */
-							 _("Write wizard"),
-							 /* 2TRANS: this is tip at the bottom of window - user can
-							    switch between window elements using tab key */
-							 _("Use 'Tab' key to move"));
+	wizard.cdw_form->window = cdw_window_new((WINDOW *) NULL,
+						 window_n_lines, window_n_cols,
+						 begin_y, begin_x,
+						 CDW_COLORS_DIALOG,
+						 /* 2TRANS: this is title of wizard window; 'write' as in
+						    'writing to optical disc */
+						 _("Write wizard"),
+						 /* 2TRANS: this is tip at the bottom of window - user can
+						    switch between window elements using tab key */
+						 _("Use 'Tab' key to move"));
 	if (wizard.cdw_form->window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create window\n");
 		return CDW_ERROR;
 	}
 
-	wizard.cdw_form->subwindow = cdw_ncurses_window_new(wizard.cdw_form->window,
-							    subwindow_n_lines, subwindow_n_cols,
-							    1, 1,
-							    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
+	wizard.cdw_form->subwindow = cdw_window_new(wizard.cdw_form->window,
+						    subwindow_n_lines, subwindow_n_cols,
+						    1, 1,
+						    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
 
 	if (wizard.cdw_form->subwindow == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create subwindow\n");
@@ -679,7 +680,7 @@ cdw_rv_t cdw_write_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 
 	/* 2TRANS: this is message to user: current action will
 	   be writing to %s, where %s is DVD or CD */
-	int rv = asprintf(&(wizard.header), _("You are writing to %s disc"), disc->simple_type_label);
+	int rv = asprintf(&(wizard.header), _("You are writing to %s disc"), disc->cdio->simple_type_label);
 	if (rv == -1) {
 		cdw_vdm ("ERROR: failed to create top message\n");
 		return CDW_ERROR;
@@ -757,6 +758,7 @@ cdw_rv_t cdw_write_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 
 cdw_rv_t cdw_write_wizard_build_fields(cdw_task_t *task)
 {
+	/* 2TRANS: this is a button label of a button that opens configuration window */
 	char *other_options = _("Other options");
 	int other_options_n_cols = (int) strlen(other_options) + 2;
 
@@ -765,6 +767,7 @@ cdw_rv_t cdw_write_wizard_build_fields(cdw_task_t *task)
 
 		{ CDW_WIDGET_LABEL,      top_label_row,               first_col,      wide_label_n_cols,          1,    f_header_l,     wizard.header,                      0 },
 
+		/* 2TRANS: this is label next to input field, where user can enter a label describing created ISO9660 volume */
 		{ CDW_WIDGET_LABEL,      volume_id_label_row,         first_col,      wide_label_n_cols,          1,    f_volume_id_l,  _("Volume ID:"),                    0 },
 		{ CDW_WIDGET_INPUT,      volume_id_label_row + 1,     first_col + 3,  CDW_ISO9660_VOLI_LEN,       1,    f_volume_id_i,  task->create_image.volume_id,  CDW_ISO9660_VOLI_LEN },
 
@@ -789,6 +792,7 @@ cdw_rv_t cdw_write_wizard_build_fields(cdw_task_t *task)
 		{ CDW_WIDGET_LABEL,      dummy_checkbox_label_row,    first_col + 4,  narrow_label_n_cols,        1,    f_dummy_l,    _("Dummy write"),                         0 },
 		{ CDW_WIDGET_CHECKBOX,   dummy_checkbox_label_row,    first_col + 1,  1,                          1,    f_dummy_i,    (void *) NULL,     task->burn.dummy ? 1 : 0 },
 
+		/* 2TRANS: this is a checkbox label; "tray" is a tray of optical drive; "writing" is burning to optical disc */
 		{ CDW_WIDGET_LABEL,      eject_checkbox_label_row,    first_col + 4,  narrow_label_n_cols,        1,    f_eject_l,    _("Eject tray after writing"),            0 },
 		{ CDW_WIDGET_CHECKBOX,   eject_checkbox_label_row,    first_col + 1,  1,                          1,    f_eject_cb,   (void *) NULL,     task->burn.eject ? 1 : 0 },
 
@@ -891,10 +895,15 @@ static char *cdw_wizard_window_tip = (char *) NULL;
    aggregating data and functions that prepare the data; the data
    represents widgets displayed in pages of options window */
 static cdw_config_page_t c_pages[] = {
+	/* 2TRANS: label of a tab in configuration window with options related burning */
 	{ false, CDW_CDRECORD_OPTIONS_N_FIELDS,      cdw_cdrecord_options_form,        gettext_noop("Write"),   (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related burning */
 	{ false, CDW_GROWISOFS_OPTIONS_N_FIELDS,     cdw_growisofs_options_form,       gettext_noop("Write"),   (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related burning */
 	{ false, CDW_XORRISO_BURN_OPTIONS_N_FIELDS,  cdw_xorriso_burn_options_form,    gettext_noop("Write"),   (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related to ISO9660; probably you don't want to translate this */
 	{ false, CDW_MKISOFS_OPTIONS_N_FIELDS,       cdw_mkisofs_options_form,         gettext_noop("ISO9660"), (cdw_form_t *) NULL },
+	/* 2TRANS: label of a tab in configuration window with options related to ISO9660; probably you don't want to translate this */
 	{ false, CDW_XORRISO_ISO_OPTIONS_N_FIELDS,   cdw_xorriso_iso_options_form,     gettext_noop("ISO9660"), (cdw_form_t *) NULL }};
 
 
@@ -910,7 +919,9 @@ cdw_rv_t cdw_write_wizard_options(cdw_task_t *task)
 	/* 2TRANS: this is message at the bottom of options window;
 	   cancel means: quit without saving; '%d' is an integer used to
 	   create label of function key, e.g. "F10" */
-	int rv = asprintf(&cdw_wizard_window_tip, _("Press F%d key to save changes or ESC to cancel"), CDW_CONFIG_UI_SnC_KEY);
+	int rv = asprintf(&cdw_wizard_window_tip,
+			  _("Press F%d / F%d key to save changes or ESC to cancel"),
+			  CDW_CONFIG_UI_SnC_KEY_B, CDW_CONFIG_UI_SnC_KEY_A);
 	if (cdw_wizard_window_tip == (char *) NULL || rv == -1) {
 		cdw_vdm ("ERROR: failed to create window tip\n");
 		return CDW_ERROR;
@@ -961,10 +972,8 @@ cdw_rv_t cdw_write_wizard_options(cdw_task_t *task)
 	}
 
 
-	cdw_tabs_window_add_return_key(c_twindow, KEY_F(CDW_CONFIG_UI_SnC_KEY));
-	cdw_tabs_window_add_return_key(c_twindow, CDW_KEY_ESCAPE);
-	cdw_tabs_window_add_return_key(c_twindow, 'q');
-	cdw_tabs_window_add_return_key(c_twindow, 'Q');
+	cdw_tabs_window_add_return_keys(c_twindow, KEY_F(CDW_CONFIG_UI_SnC_KEY_A), KEY_F(CDW_CONFIG_UI_SnC_KEY_B),
+					CDW_KEY_ESCAPE, 'q', 'Q', 0);
 	c_twindow->driver_reader = cdw_config_ui_tabs_window_form_reader;
 	cdw_tabs_window_finalize(c_twindow);
 
@@ -992,7 +1001,8 @@ cdw_rv_t cdw_write_wizard_options(cdw_task_t *task)
 		for (int j = 0; j < n_tabs; j++) {
 			cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(j + CDW_CONFIG_UI_FX_START_KEY));
 		}
-		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_A));
+		cdw_form_add_return_char(c_pages[id].cdw_form, KEY_F(CDW_CONFIG_UI_SnC_KEY_B));
 	}
 	c_twindow->user_data = (void *) c_pages;
 
@@ -1066,7 +1076,8 @@ int cdw_write_wizard_options_window_driver(cdw_task_t *task)
 		int key = cdw_tabs_window_driver(c_twindow);
 		if (key == CDW_KEY_ESCAPE) {
 			break;
-		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY)) { /* SnC = Save and Close */
+		} else if (key == KEY_F(CDW_CONFIG_UI_SnC_KEY_A)
+			   || key == KEY_F(CDW_CONFIG_UI_SnC_KEY_B)) { /* SnC = Save and Close */
 
 			/* flush */
 			for (int i = 0; i < c_twindow->n_tabs; i++) {
