@@ -6,7 +6,7 @@
 #include "cdw_disc.h"
 
 
-cdw_rv_t write_wizard(cdw_task_t *task, cdw_disc_t *disc);
+cdw_rv_t cdw_write_wizard(cdw_task_t *task, cdw_disc_t *disc);
 
 
 #endif /* #ifndef H_CDW_WRITE_WIZARD */
