@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -296,18 +296,29 @@ size_t cdw_dll_length(cdw_dll_item_t *head)
 
 
 
-/**
- * \brief Remove all items from list
- *
- * \return CDW_OK
- */
-cdw_rv_t cdw_dll_clean(cdw_dll_item_t *head)
-{
-        cdw_assert (head != (cdw_dll_item_t *) NULL, "passed null list to the function\n");
 
-	cdw_dll_item_t *f;
-	for (f = head; f; ) {
-		cdw_dll_item_t *tmp = f->next;
+/**
+   \brief Remove all links of a list
+
+   \date Function's top-level comment reviewed on 2012-02-16
+   \date Function's body reviewed on 2012-02-16
+
+   Function deallocates all links of a list, but does not
+   deallocate payload of the list.
+
+   \param head - beginning of a list
+
+   \return CDW_OK
+*/
+cdw_rv_t cdw_dll_clean(cdw_dll_t *head)
+{
+	if (!head) {
+		cdw_vdm ("WARNING: passed null list to the function\n");
+		return CDW_OK;
+	}
+
+	for (cdw_dll_t *f = head; f; ) {
+		cdw_dll_t *tmp = f->next;
 
 		f->data = (void *) NULL;
 
@@ -322,6 +333,52 @@ cdw_rv_t cdw_dll_clean(cdw_dll_item_t *head)
 
 
 
+/**
+   \brief Deallocate a list (links and payload)
+
+   \date Function's top-level comment reviewed on 2012-02-16
+   \date Function's body reviewed on 2012-02-16
+
+   Function deallocates all links of a list, and all payload of the list.
+   Deallocation of the payload is made using \p dealloc function.
+   All pointers are set to NULL, including \p head.
+
+   \param head - beginning of a list
+   \param dealloc - deallocator function, freeing payload memory
+
+   \return CDW_OK
+*/
+cdw_rv_t cdw_dll_delete(cdw_dll_t **head, cdw_deallocator_t dealloc)
+{
+	cdw_assert (head, "ERROR: passed NULL pointer to a list\n");
+
+	if (!*head) {
+		cdw_vdm ("WARNING: passed null list to the function\n");
+		return CDW_OK;
+	}
+
+	for (cdw_dll_t *f = *head; f; ) {
+		cdw_dll_t *tmp = f->next;
+
+		dealloc(f->data);
+		f->data = (void *) NULL;
+
+		f->next = (cdw_dll_t *) NULL;
+		f->prev = (cdw_dll_t *) NULL;
+
+		free(f);
+		f = tmp;
+	}
+
+	*head = (cdw_dll_t *) NULL;
+
+	return CDW_OK;
+}
+
+
+
+
+
 /**
  * \brief Check if list is empty
  *
