@@ -11,6 +11,8 @@
 
 char *cdw_string_rtrim(char *str);
 char *cdw_string_ltrim(char *str);
+char *cdw_string_trim(const char *string);
+bool  cdw_string_is_all_spaces(const char *string);
 char *cdw_string_concat(const char *str, ...);
 cdw_rv_t cdw_string_append(char **head, const char *tail);
 cdw_rv_t cdw_string_set(char **target, const char *source);
@@ -19,14 +21,27 @@ cdw_rv_t cdw_string_security_parser(const char *string, char *invalid);
 
 bool cdw_string_starts_with_ci(const char *field, const char *fieldname);
 
-int cdw_string_get_words_end(const char *string, int chars_max);
+
+bool cdw_string_equal(const void *string1, const void *string2);
 
 void cdw_string_free_table_of_strings(char *labels[], int n_max);
 cdw_rv_t cdw_string_ints2labels_3(char **labels, int ints[], size_t n);
 cdw_rv_t cdw_string_get_bool_value(const char *buffer, bool *value);
 
-int cdw_string_wrap_for_form_get_n_lines(const char *message, int n_cols);
-char  *cdw_string_wrap_for_form(const char *message, int n_cols, int n_lines);
+
+/* align of text */
+enum {
+	CDW_ALIGN_LEFT,
+	CDW_ALIGN_RIGHT,
+	CDW_ALIGN_CENTER
+};
+
+
+
+char  *cdw_string_wrap(const char *string, size_t width_max, int align);
+size_t cdw_string_count_lines(const char *string, size_t width_max);
+
+size_t cdw_string_get_line_end(const char *string, size_t chars_max);
 
 char *cdw_string_get_printable_if_needed(const char *string);
 
