@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,12 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-//#include <sys/ioctl.h> /* struct winsize */
-
-
 #include "cdw_sys.h"
 #include "gettext.h"
 #include "cdw_debug.h"
 #include "cdw_string.h"
 #include "cdw_cdio.h"
+#include "cdw_ofs.h"
 
 /**
    \file cdw/src/utilities/cdw_sys.c
@@ -51,21 +49,104 @@
    \li handling signals.
 */
 
+#ifdef CDW_MAIN_WINDOW_CAN_RESIZE
+extern sig_atomic_t cdw_main_window_resize;
+#endif
+
 
 /* data structures and functions related to handling signals */
-static void cdw_sys_sigsegv_sa_handler(int signal_number);
-//static void cdw_sys_sigwinch_sa_handler(int signal_number);
-static void cdw_sys_sigchild_sa_handler(__attribute__((unused)) int signal_number);
-static struct sigaction cdw_sigsegv_action;
-//static struct sigaction cdw_sigwinch_action;
-static struct sigaction cdw_sigchild_action;
+static void cdw_sys_exit_signal_handler(int signal_number);
+static void cdw_sys_exit_signal_message(void);
+static void cdw_sys_sigwinch_handler(int signal_number);
+static void cdw_sys_sigchild_handler(__attribute__((unused)) int signal_number);
+
 static sig_atomic_t child_exit_status;
+static sig_atomic_t cdw_sys_signal_number;
 
 static cdw_rv_t cdw_sys_check_file_system_support_via_proc_fs(cdio_fs_t cdio_fs);
 static cdw_rv_t cdw_sys_check_file_system_support_via_modules(cdio_fs_t cdio_fs);
 static char *   cdw_sys_get_osrelease(void);
 static char *   cdw_sys_get_kernel_modules_location(void);
 
+/* Generic type of signal handler, regardless of actions performed
+   by specific implementation of a signal handler. */
+typedef void (*cdw_sa_handler)(int);
+
+#define CDW_SYS_SIGNAL_LABEL_LEN 10
+
+/* Signal information taken from signal(7) man page. */
+struct {
+	int id;
+	const char label[CDW_SYS_SIGNAL_LABEL_LEN];
+	cdw_sa_handler handler;
+} cdw_sys_signals[] = {
+	{ SIGHUP,  "SIGHUP",  NULL },                          /*      1       Term    Hangup detected on controlling terminal or death of controlling process */
+	{ SIGINT,  "SIGINT",  NULL },                          /*      2       Term    Interrupt from keyboard */
+        { SIGQUIT, "SIGQUIT", cdw_sys_exit_signal_handler },   /*      3       Core    Quit from keyboard */
+	{ SIGILL,  "SIGILL",  cdw_sys_exit_signal_handler },   /*      4       Core    Illegal Instruction */
+	{ SIGABRT, "SIGABRT", cdw_sys_exit_signal_handler },   /*      6       Core    Abort signal from abort(3) */
+        { SIGFPE,  "SIGFPE",  cdw_sys_exit_signal_handler },   /*      8       Core    Floating point exception */
+        { SIGKILL, "SIGKILL", NULL },                          /*      9       Term    Kill signal, can't be caught */
+        { SIGSEGV, "SIGSEGV", cdw_sys_exit_signal_handler },   /*     11       Core    Invalid memory reference */
+	{ SIGPIPE, "SIGPIPE", cdw_sys_exit_signal_handler },   /*     13       Term    Broken pipe: write to pipe with no readers */
+        { SIGALRM, "SIGALRM", NULL },                          /*     14       Term    Timer signal from alarm(2) */
+	{ SIGTERM, "SIGTERM", cdw_sys_exit_signal_handler },   /*     15       Term    Termination signal */
+	{ SIGUSR1, "SIGUSR1", NULL },                          /*  30,10,16    Term    User-defined signal 1 */
+	{ SIGUSR2, "SIGUSR2", NULL },                          /*  31,12,17    Term    User-defined signal 2 */
+	{ SIGCHLD, "SIGCHLD", cdw_sys_sigchild_handler    },   /*  20,17,18    Ign     Child stopped or terminated; to be handled by cdw separately */
+	{ SIGCONT, "SIGCONT", NULL },                          /*  19,18,25    Cont    Continue if stopped */
+	{ SIGSTOP, "SIGSTOP", NULL },                          /*  17,19,23    Stop    Stop process, can't be caught */
+	{ SIGTSTP, "SIGTSTP", cdw_sys_exit_signal_handler },   /*  18,20,24    Stop    Stop typed at tty */
+	{ SIGTTIN, "SIGTTIN", NULL },                          /*  21,21,26    Stop    tty input for background process */
+	{ SIGTTOU, "SIGTTOU", NULL },                          /*  22,22,27    Stop    tty output for background process */
+
+
+#if _POSIX_VERSION == 200112L
+	{ SIGBUS,  "SIGBUS",  cdw_sys_exit_signal_handler },   /*  10,7,10     Core    Bus error (bad memory access) */
+	{ SIGPOLL, "SIGPOLL", NULL },                          /*              Term    Pollable event (Sys V).; Synonym for SIGIO */
+	{ SIGPROF, "SIGPROF", NULL },                          /*  27,27,29    Term    Profiling timer expired */
+	{ SIGSYS,  "SIGSYS",  cdw_sys_exit_signal_handler },   /*  12,31,12    Core    Bad argument to routine (SVr4) */
+	{ SIGTRAP, "SIGTRAP", NULL },                          /*      5       Core    Trace/breakpoint trap */
+	{ SIGURG,  "SIGURG",  NULL },                          /*  16,23,21    Ign     Urgent condition on socket (4.2BSD) */
+	{ SIGVTALRM, "SIGVTALRM", NULL },                      /*  26,26,28    Term    Virtual alarm clock (4.2BSD) */
+	{ SIGXCPU, "SIGXCPU", NULL },                          /*  24,24,30    Core    CPU time limit exceeded (4.2BSD) */
+	{ SIGXFSZ, "SIGXFSZ", NULL },                          /*  25,25,31    Core    File size limit exceeded (4.2BSD) */
+#endif
+
+
+#ifdef SIGIOT
+	{ SIGIOT,  "SIGIOT",  cdw_sys_exit_signal_handler },   /*     6        Core    IOT trap. A synonym for SIGABRT */
+#endif
+#ifdef SIGEMT
+	{ SIGEMT,  "SIGEMT",  NULL },                          /*   7,-,7      Term */
+#endif
+#ifdef SIGSTKFLT
+	{ SIGSTKFLT, "SIGSTKFLT", NULL },                      /*   -,16,-     Term    Stack fault on coprocessor (unused) */
+#endif
+#ifdef SIGIO
+	{ SIGIO,   "SIGIO",   NULL },                          /*  23,29,22    Term    I/O now possible (4.2BSD) */
+#endif
+#ifdef SIGCLD
+	{ SIGCLD,  "SIGCLD",  cdw_sys_sigchild_handler },      /*   -,-,18     Ign     A synonym for SIGCHLD */
+#endif
+#ifdef SIGPWR
+	{ SIGPWR,  "SIGPWR",  NULL },                          /*  29,30,19    Term    Power failure (System V) */
+#endif
+#ifdef SIGINFO
+	{ SIGINFO, "SIGINFO", NULL },                          /*     29,-,-           A synonym for SIGPWR */
+#endif
+#ifdef SIGLOST
+	{ SIGLOST, "SIGLOST", NULL },                          /*   -,-,-      Term    File lock lost */
+#endif
+#ifdef SIGWINCH
+	{ SIGWINCH, "SIGWINCH", cdw_sys_sigwinch_handler },    /*  28,28,20    Ign     Window resize signal (4.3BSD, Sun) */
+#endif
+#ifdef SIGUNUSED
+	{ SIGUNUSED, "SIGUNUSED", cdw_sys_exit_signal_handler },  /*   -,31,-     Core    Synonymous with SIGSYS */
+#endif
+
+	{ -1,      "",        NULL }}; /* guard */
+
 
 
 
@@ -74,7 +155,7 @@ static char *   cdw_sys_get_kernel_modules_location(void);
 
   \param signal_number - number of handled signal, unused
 */
-void cdw_sys_sigchild_sa_handler(__attribute__((unused)) int signal_number)
+void cdw_sys_sigchild_handler(__attribute__((unused)) int signal_number)
 {
 	/* clean child process */
 	int status;
@@ -88,17 +169,11 @@ void cdw_sys_sigchild_sa_handler(__attribute__((unused)) int signal_number)
 
 
 
-void cdw_sys_sigsegv_sa_handler(int signal_number)
+void cdw_sys_exit_signal_handler(int signal_number)
 {
-	cdw_assert (signal_number == SIGSEGV, "ERROR: wrong signal to handle\n");
-	/* 2TRANS: this is a message printed to console when cdw needs to
-	   close unexpectedly. "signal" is a signal from operating system;
-	   "SIGSEGV" is a signal's name; keep leading and ending "\n";
-	   use "\n" in the middle of message as needed to keep line
-	   lengths no more than 76 characters */
-	fprintf(stderr, _("\ncdw received SIGSEGV signal and it needed to clean up after itself\nand leave. This means that there is a bug in cdw. Don't be mad...\n\n"));
+	cdw_sys_signal_number = signal_number;
 
-	/* there was a SIGSEGV, so let's better leave now;
+	/* there was an 'exit' signal, so let's better leave now;
 	   exit() triggers functions registered with atexit(); these
 	   are <module_name>_clean() functions */
 	exit(EXIT_FAILURE);
@@ -107,8 +182,8 @@ void cdw_sys_sigsegv_sa_handler(int signal_number)
 
 
 
-#if 0
-void cdw_sys_sigwinch_sa_handler(int signal_number)
+
+void cdw_sys_sigwinch_handler(int signal_number)
 {
 	cdw_assert (signal_number == SIGWINCH, "ERROR: wrong signal to handle\n");
 
@@ -118,47 +193,89 @@ void cdw_sys_sigwinch_sa_handler(int signal_number)
 	    - resize_term(), resizeterm();
 	    - clearok();
 	   and the conclusion is: there is no simple solution
-	   to the problem of resizing windows in ncurses app */
+	   to the problem of resizing windows in ncurses app;
+
+
+	   2012-02-20
+	   The function is called in SIGWINCH signal. I've tested
+	   this by adding only call to exit() in signal handler.
+	   So perhaps my attempts didn't work because I was trying
+	   to put call to regular library function inside a signal
+	   handler. */
+
+#ifdef CDW_MAIN_WINDOW_CAN_RESIZE
+	/* code in main event loop should check this flag and act
+	   accordingly (resize main window and all widgets embedded
+	   in the window) */
+	cdw_main_window_resize = 1;
+#endif
 
 	return;
 }
-#endif
+
+
+
+
+
+void cdw_sys_exit_signal_message(void)
+{
+	int i = 0;
+	while (cdw_sys_signals[i].id != -1) {
+		if (cdw_sys_signals[i].id == cdw_sys_signal_number) {
+			char message[] = "cdw: ";
+			write(2, message, sizeof (message));
+			write(2, cdw_sys_signals[i].label, sizeof (cdw_sys_signals[i].label));
+
+			/* yes, I know that I shouldn't call printf() in
+			   signal handlers, but since it will be called
+			   shortly before calling exit(), there shouldn't
+			   be much harm done */
+
+			/* 2TRANS: this is a message printed to console when cdw needs to
+			   close unexpectedly. "signal" is a signal from operating system;
+			   "%s" is a signal's name; keep leading and ending "\n";
+			   use "\n" in the middle of message as needed to keep line
+			   lengths no more than 76 characters */
+			fprintf(stderr, _("\n\ncdw received %s signal and it needed to clean up after itself\nand leave. This may mean that there is a bug in cdw. Don't be mad...\n\n"),
+				cdw_sys_signals[i].label);
+			break;
+		}
+		i++;
+	}
+
+	return;
+}
+
 
 
 
 
 int cdw_sys_signal_handlers_init(void)
 {
-	/* set up code dealing with zombie processes created during run_command() call */
-	cdw_vdm ("INFO: setting SIGCHILD handler\n");
-	memset(&cdw_sigchild_action, 0, sizeof(cdw_sigchild_action));
-	cdw_sigchild_action.sa_handler = &cdw_sys_sigchild_sa_handler;
-	int rv = sigaction(SIGCHLD, &cdw_sigchild_action, (struct sigaction *) NULL);
-	if (rv == -1) {
-		cdw_vdm ("ERROR: failed call to sigaction(SIGCHILD, ...)\n");
-		return -1;
-	}
-#if 0
-	/* set up code responding to window size change */
-	cdw_vdm ("INFO: setting SIGWINCH handler\n");
-	memset(&cdw_sigwinch_action, 0, sizeof(cdw_sigwinch_action));
-	cdw_sigwinch_action.sa_handler = &cdw_sys_sigwinch_sa_handler;
-	rv = sigaction(SIGWINCH, &cdw_sigwinch_action, (struct sigaction *) NULL);
-	if (rv == -1) {
-		cdw_vdm ("ERROR: failed call to sigaction(SIGWINCH, ...)\n");
-		return -1;
-	}
-#endif
-	/* set up code dealing with signals sent when program does invalid memory reference */
-	cdw_vdm ("INFO: setting SIGSEGV handler\n");
-	memset(&cdw_sigsegv_action, 0, sizeof(cdw_sigsegv_action));
-	cdw_sigsegv_action.sa_handler = cdw_sys_sigsegv_sa_handler;
-	rv = sigaction(SIGSEGV, &cdw_sigsegv_action, (struct sigaction *) NULL);
-	if (rv == -1) {
-		cdw_vdm ("ERROR: failed call to sigaction(SIGSEGV, ...)\n");
-		return -1;
+	int i = 0;
+	while (cdw_sys_signals[i].id != -1) {
+		if (cdw_sys_signals[i].handler) {
+			cdw_vdm ("INFO: setting %s handler\n", cdw_sys_signals[i].label);
+			struct sigaction action;
+			memset(&action, 0, sizeof(action));
+			action.sa_handler = cdw_sys_signals[i].handler;
+			action.sa_flags = 0;
+			int rv = sigaction(cdw_sys_signals[i].id, &action, (struct sigaction *) NULL);
+			if (rv == -1) {
+				cdw_vdm ("ERROR: failed call to sigaction(%s, ...)\n", cdw_sys_signals[i].label);
+				return -1;
+			}
+		}
+		i++;
 	}
 
+	/* signal_message isn't a module cleaner function, so I
+	   don't call atexit() in main(); this call is made to
+	   ensure that final error message is printed to stderr
+	   when cdw_sys_exit_signal_handler() calls exit() */
+
+	atexit(cdw_sys_exit_signal_message);
+
 	return 0;
 }
 
@@ -168,9 +285,9 @@ int cdw_sys_signal_handlers_init(void)
 
 cdw_rv_t cdw_sys_check_file_system_support(cdio_fs_t cdio_fs)
 {
-	cdw_assert (cdw_cdio_is_fs_iso(cdio_fs) || cdio_fs == CDIO_FS_UDF,
+	cdw_assert (cdw_ofs_is_iso(cdio_fs) || cdio_fs == CDIO_FS_UDF,
 		    "ERROR: unrecognized fs: %d\n", cdio_fs);
-	if (cdw_cdio_is_fs_iso(cdio_fs)) {
+	if (cdw_ofs_is_iso(cdio_fs)) {
 		/* there may be few subtypes of ISO9660 fs,
 		   let's simplify this for code using cdio_fs;
 		   CDIO_FS_ISO_UDF also counts as ISO9660 */
@@ -221,8 +338,8 @@ cdw_rv_t cdw_sys_check_file_system_support(cdio_fs_t cdio_fs)
 				   given file system type; "cancel" means
 				   here "we don't know, but you may try" */
 				return CDW_CANCEL;
-			} else { /* crv1 == CDW_GEN_ERROR */
-				/* two methods returned CDW_GEN_ERROR,
+			} else { /* crv1 == CDW_ERROR */
+				/* two methods returned CDW_ERROR,
 				   there is something very wrong;
 				   assume that given file system is not
 				   supported */
@@ -233,6 +350,9 @@ cdw_rv_t cdw_sys_check_file_system_support(cdio_fs_t cdio_fs)
 }
 
 
+
+
+
 cdw_rv_t cdw_sys_check_file_system_support_via_proc_fs(cdio_fs_t cdio_fs)
 {
 	FILE *fp = fopen("/proc/filesystems", "r");
@@ -363,6 +483,7 @@ cdw_rv_t cdw_sys_check_file_system_support_via_modules(cdio_fs_t cdio_fs)
 
 
 
+
 char *cdw_sys_get_osrelease(void)
 {
 	/* osrelease file stores string that is the same as
@@ -402,6 +523,7 @@ char *cdw_sys_get_osrelease(void)
 
 
 
+
 char *cdw_sys_get_kernel_modules_location(void)
 {
 	char *osrelease = cdw_sys_get_osrelease();
@@ -431,3 +553,44 @@ char *cdw_sys_get_kernel_modules_location(void)
 		}
 	}
 }
+
+
+
+
+
+/**
+   \brief Get copy of environment PATH variable
+
+   \date Function's top-level comment reviewed on 2012-02-16
+   \date Function's body reviewed on 2012-02-16
+
+   Function returns a copy of environment PATH variable.
+   Function may return NULL pointer if PATH environment variable is not
+   defined on user's system.
+
+   Returned pointer is owned by caller.
+
+   \return valid pointer on success
+   \return NULL pointer on failure
+*/
+char *cdw_sys_env_path(void)
+{
+	const char *tmp = getenv("PATH");
+	if (!tmp) {
+		cdw_vdm ("WARNING: getenv(\"PATH\") returns NULL\n");
+		return (char *) NULL;
+	}
+
+#if 0   /* test data */
+	const char *test_string = ":/home/acerion/bin: :/usr/local/bin::/usr/bin:/bin:/usr/games:";
+	char *path = strdup(test_string);
+#else
+	char *path = strdup(tmp);
+#endif
+	if (!path) {
+		cdw_vdm ("ERROR: failed to strdup() string \"%s\"\n", tmp);
+		return (char *) NULL;
+	} else {
+		return path;
+	}
+}
