@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2011 Kamil Ignacak
+ * Copyright (C) 2007 - 2012 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -163,6 +163,8 @@ int cdw_utils_process_commandline_arguments(int argc, char *argv[])
 		case '?':
 			/* 2TRANS: this is error message printed to terminal */
 			fprintf(stderr, _("ERROR: unknown option in command line\n"));
+			/* 2TRANS: this is error message printed to terminal */
+			fprintf(stderr, _("use '-h' option for help message\n"));
 			return -1;
 		default:
 			break;
@@ -180,11 +182,7 @@ void cdw_utils_print_version(void)
 	/* 2TRANS: this is message printed in terminal,
 	   first %s is program name, second %s is
 	   program version */
-	printf(_("%s-%s Copyright (C) 2002 - 2003 Balazs Varkonyi\n"), PACKAGE, VERSION);
-	/* 2TRANS: this is message printed in terminal,
-	   first %s is program name, second %s is
-	   program version */
-	printf(_("%s-%s Copyright (C) 2007 - 2011 Kamil Ignacak\n\n"), PACKAGE, VERSION);
+	printf(_("%s %s\n"), PACKAGE, VERSION);
 
 	return;
 }
@@ -195,6 +193,23 @@ void cdw_utils_print_version(void)
 
 void cdw_utils_print_help(const char *program_name)
 {
+	/* 2TRANS: this is message printed in terminal,
+	   first %s is program name, second %s is
+	   program version */
+	printf(_("%s %s\n"), PACKAGE, VERSION);
+	/* 2TRANS: this is message printed in terminal,
+	   first %s is program name, second %s is
+	   program version */
+	printf(_("Copyright (C) 2002 - 2003 Balazs Varkonyi\n"));
+	/* 2TRANS: this is message printed in terminal,
+	   first %s is program name, second %s is
+	   program version */
+	printf(_("Copyright (C) 2007 - 2012 Kamil Ignacak\n\n"));
+	/* 2TRANS: this is message printed in terminal,
+	   '2+' means "version 2, or later";
+	   keep double newline character */
+	printf(_("License: GNU General Public License, version 2+\n\n"));
+
 	/* 2TRANS: this is help text displayed in console;
 	   %s will be replaced by program name, don't change its
 	   position */
@@ -204,19 +219,19 @@ void cdw_utils_print_help(const char *program_name)
 #ifdef HAVE_LIBMYSQLCLIENT
 	/* 2TRANS: this is help text displayed in console;
 	   don't localize anything before colon */
-	printf(_("  --catalog | -c            : start in Disk Catalog mode\n"));
+	printf(_("  -c | --catalog            : start in Disk Catalog mode\n"));
 #endif
 	/* 2TRANS: this is help text displayed in console;
 	   don't localize anything before colon */
-	printf(_("  --help    | -h            : show this screen\n"));
+	printf(_("  -h | --help               : show this message\n"));
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  -v | --version            : show version of this software\n"));
 	/* 2TRANS: this is help text displayed in console;
 	   don't localize anything before colon */
 	printf(_("  --escdelay=X              : modify ESC key delay period;\n"));
 	/* 2TRANS: this is help text displayed in console */
 	printf(_("                              'X' is non-negative time in milliseconds\n"));
-	/* 2TRANS: this is help text displayed in console;
-	   don't localize anything before colon */
-	printf(_("  --version | -v            : show version\n"));
 	/* 2TRANS: this is help text displayed in console;
 	   don't localize anything before colon */
 	printf(_("  --enable-dvd-rp-dl        : enable support for DVD+R DL;\n"));
@@ -246,43 +261,6 @@ int cdw_locale_init(void)
 
 
 
-#if 0
-/* unused at this moment, replaced with registering functions with atexit() */
-/**
- * Deallocate all previously allocated resources, terminate ncurses
- *
- * Prepare application for closing:close all files, deallocate memory,
- * destroy custom and ncurses widgets. Exit ncurses. Try to do all this
- * in reverse order to the one found in main().
- */
-void clean_before_cdw_exit(void)
-{
-	cdw_config_module_clean(); /* cdw configuration module */
-
-	/* file manager has to clean up 'selected files' subwindow
-	   view before cdw_ui_main_clean() can remove parent window;
-	   FIXME: perhaps cdw_ui_main should call file manager's
-	   function to destroy the view */
-	cdw_file_manager_clean(); /* cdw file selector */
-
-	cdw_main_ui_clean(); /* main application window */
-
-	cdw_logging_clean();
-
-	cdw_ext_tools_clean();
-
-	cdw_fs_clean();
-
-	cdw_ncurses_clean();
-
-	cdw_cdio_clean();
-
-	return;
-}
-#endif
-
-
-
 
 #if 0
 /* currently unused, disabling to suppress compiler warnings */
@@ -535,14 +513,6 @@ const char *cdw_utils_id_label_table_get_label(cdw_id_clabel_t *table, cdw_id_t
 {
 	cdw_assert (table != (cdw_id_clabel_t *) NULL, "ERROR: input table is NULL\n");
 
-	/* this function uses while() loop for traversing table with id/label
-	   pairs, but since:
-	   a) this function is called only in debug builds
-	   b) table sizes are usually very small (usually less than 10 elements)
-
-	   then longer time of lookup (sequential rather than direct access)
-	   does not influence performance */
-
 #ifndef NDEBUG
 	int j = 0;
 	/* this piece of code is used to detect potential incorrectly built
@@ -569,6 +539,29 @@ const char *cdw_utils_id_label_table_get_label(cdw_id_clabel_t *table, cdw_id_t
 
 
 
+cdw_id_t cdw_utils_id_label_table_get_id(cdw_id_clabel_t *table, const char *label, cdw_id_t guard)
+{
+	cdw_assert (table, "ERROR: input table is NULL\n");
+
+	if (!label) {
+		cdw_vdm ("WARNING: looking for id of NULL string, returning guard \n");
+		return guard;
+	}
+
+	int i = 0;
+	while (table[i].id != guard) {
+		if (!strcmp(table[i].label, label)) {
+			return table[i].id;
+		}
+
+		i++;
+	}
+
+	return guard;
+}
+
+
+
 
 
 static cdw_id_clabel_t cdw_rv_t_table[] = {
@@ -772,7 +765,8 @@ int cdw_utils_compress_table_of_ints(int *table, int n_max)
 
 static void test_cdw_utils_eta_calculations(void);
 static void test_cdw_utils_compress_table_of_ints(void);
-
+static void test_cdw_utils_id_label_table_get_label(void);
+static void test_cdw_utils_id_label_table_get_id(void);
 
 void cdw_utils_run_tests(void)
 {
@@ -780,6 +774,8 @@ void cdw_utils_run_tests(void)
 
 	test_cdw_utils_eta_calculations();
 	test_cdw_utils_compress_table_of_ints();
+	test_cdw_utils_id_label_table_get_label();
+	test_cdw_utils_id_label_table_get_id();
 
 	fprintf(stderr, "done\n\n");
 
@@ -788,6 +784,8 @@ void cdw_utils_run_tests(void)
 
 
 
+
+
 void test_cdw_utils_eta_calculations(void)
 {
 	fprintf(stderr, "\ttesting cdw_utils_eta_calculations()... ");
@@ -877,5 +875,75 @@ void test_cdw_utils_compress_table_of_ints(void)
 
 
 
+
+void test_cdw_utils_id_label_table_get_label(void)
+{
+	fprintf(stderr, "\ttesting cdw_utils_id_label_table_get_label()... ");
+
+	cdw_id_clabel_t test_table[] = {
+		{    23,   "hello",               },
+		{    76,   "milky",               },
+		{   111,   "Tuesday",             },
+		{   487,   "robotic",             },
+		{ -5235,   "normalized",          },
+		{   385,   "every day",           },
+		{   244,   " so called solution " },
+		{   444,   " orange",             },
+		{    98,   (char *) NULL,         }};
+
+
+	int i = 0;
+	while (test_table[i].label) {
+		const char *label = cdw_utils_id_label_table_get_label(test_table, test_table[i].id);
+		cdw_assert (!strcmp(label, test_table[i].label),
+			    "ERROR: data set #%d: expected label \"%s\", got label \"%s\" (id = %lld)\n",
+			    i, test_table[i].label, label, test_table[i].id);
+		i++;
+	}
+
+	fprintf(stderr, "OK\n");
+
+	return;
+}
+
+
+
+
+
+void test_cdw_utils_id_label_table_get_id(void)
+{
+	fprintf(stderr, "\ttesting cdw_utils_id_label_table_get_id()... ");
+
+	cdw_id_t the_guard = 495;
+
+	cdw_id_clabel_t test_table[] = {
+		{        23,   "hello",               },
+		{        76,   "milky",               },
+		{       111,   "Tuesday",             },
+		{       487,   "robotic",             },
+		{     -5235,   "normalized",          },
+		{       385,   "every day",           },
+		{       244,   " so called solution " },
+		{       444,   " orange",             },
+		{        98,   "swift ",              },
+		{ the_guard,   "closed",              }};
+
+	int i = 0;
+	while (test_table[i].id != the_guard) {
+		cdw_id_t id = cdw_utils_id_label_table_get_id(test_table, test_table[i].label, the_guard);
+		cdw_assert (id == test_table[i].id,
+			    "ERROR: data set #%d: expected id = %lld, got id = %lld (label = \"%s\")\n",
+			    i, test_table[i].id, id, test_table[i].label);
+		i++;
+	}
+
+	fprintf(stderr, "OK\n");
+
+	return;
+}
+
+
+
+
 #endif /* #ifdef CDW_UNIT_TEST_CODE */
 
